@@ -193,7 +193,6 @@ int monetdb_is_initialized(void) {
 
 static char* monetdb_query_internal(monetdb_connection conn, char* query, char execute, monetdb_result** result, long* affected_rows, long* prepare_id, char language) {
 	str res = MAL_SUCCEED;
-	int sres;
 	Client c = (Client) conn;
 	mvc* m;
 	backend *b;
@@ -303,8 +302,7 @@ cleanup:
 	bstream_destroy(c->fdin);
 	c->fdin = NULL;
 
-	sres = SQLautocommit(c, m);
-	if (!sres && !res) {
+	if (SQLautocommit(m) != MAL_SUCCEED) {
 		return GDKstrdup("Cannot COMMIT/ROLLBACK without a valid transaction.");
 	}
 	if (res != MAL_SUCCEED && res_internal != NULL) {
@@ -390,7 +388,7 @@ char* monetdb_append(monetdb_connection conn, const char* schema, const char* ta
 			return(res);
 		}
 	}
-	SQLautocommit(c, m);
+	SQLautocommit(m);
 	return NULL;
 }
 
@@ -488,7 +486,7 @@ void monetdb_unregister_progress(monetdb_connection conn) {
 void monetdb_shutdown(void) {
 	MT_lock_set(&embedded_lock);
 	if (monetdb_embedded_initialized) {
-		mserver_reset(0);
+		mal_exit();
 		fclose(embedded_stdout);
 		monetdb_embedded_initialized = 0;
 	}
@@ -543,7 +541,7 @@ GENERATE_BASE_HEADERS(monetdb_data_timestamp, timestamp);
 			msg = GDKstrdup("Malloc failure!");                                \
 			goto wrapup;                                                       \
 		}                                                                      \
-		if (b->tdense && !b->tnodense) {                                       \
+		if (b->tdense) {                                       \
 			size_t it = 0;                                                     \
 			tpe val = b->T.seq;                                                \
 			/* bat is dense, materialize it */                                 \
@@ -734,7 +732,7 @@ monetdb_column* monetdb_result_fetch(monetdb_result* res, size_t column_index) {
 				bat_data->data[j] = NULL;
 			} else {
 				char *result = NULL;
-				int length = 0;
+				size_t length = 0;
 				if (BATatoms[bat_type].atomToStr(&result, &length, t) ==
 					0) {
 					msg = GDKstrdup("Failed to convert element to string");
