@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -306,37 +306,14 @@
 #define _GDK_H_
 
 /* standard includes upon which all configure tests depend */
-#include <stdio.h>
 #ifdef HAVE_SYS_TYPES_H
 # include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif
-#ifdef HAVE_STRING_H
-# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#ifdef HAVE_STRINGS_H
-# include <strings.h>
-#endif
-#ifdef HAVE_INTTYPES_H
-# include <inttypes.h>
-#else
-# ifdef HAVE_STDINT_H
-#  include <stdint.h>
-# endif
-#endif
+#include <stddef.h>
+#include <string.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -346,36 +323,13 @@
 #ifdef HAVE_SYS_FILE_H
 # include <sys/file.h>
 #endif
-#ifdef HAVE_SYS_PARAM_H
-# include <sys/param.h>		/* MAXPATHLEN */
-#endif
 
 #ifdef HAVE_DIRENT_H
 # include <dirent.h>
-#else
-# define dirent direct
-# ifdef HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# ifdef HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# ifdef HAVE_NDIR_H
-#  include <ndir.h>
-# endif
 #endif
 
 #include <limits.h>		/* for *_MIN and *_MAX */
 #include <float.h>		/* for FLT_MAX and DBL_MAX */
-#ifndef LLONG_MAX
-#ifdef LONGLONG_MAX
-#define LLONG_MAX LONGLONG_MAX
-#define LLONG_MIN LONGLONG_MIN
-#else
-#define LLONG_MAX LL_CONSTANT(9223372036854775807)
-#define LLONG_MIN (-LL_CONSTANT(9223372036854775807) - LL_CONSTANT(1))
-#endif
-#endif
 
 #include "gdk_system.h"
 #include "gdk_posix.h"
@@ -387,29 +341,18 @@
 #define MIN(A,B)	((A)>(B)?(B):(A))
 
 /* defines from ctype with casts that allow passing char values */
-#define GDKisspace(c)	isspace((int) (unsigned char) (c))
-#define GDKisalnum(c)	isalnum((int) (unsigned char) (c))
-#define GDKisdigit(c)	(((unsigned char) (c)) >= '0' && ((unsigned char) (c)) <= '9')
+#define GDKisspace(c)	isspace((unsigned char) (c))
+#define GDKisalnum(c)	isalnum((unsigned char) (c))
+#define GDKisdigit(c)	isdigit((unsigned char) (c))
 
-#ifndef NATIVE_WIN32
 #define BATDIR		"bat"
-#define DELDIR		"bat/DELETE_ME"
-#define BAKDIR		"bat/BACKUP"
-#define SUBDIR		"bat/BACKUP/SUBCOMMIT"
-#define LEFTDIR		"bat/LEFTOVERS"
-#else
-#define BATDIR		"bat"
-#define DELDIR		"bat\\DELETE_ME"
-#define BAKDIR		"bat\\BACKUP"
-#define SUBDIR		"bat\\BACKUP\\SUBCOMMIT"
-#define LEFTDIR		"bat\\LEFTOVERS"
-#endif
+#define TEMPDIR_NAME	"TEMP_DATA"
 
-#ifdef MAXPATHLEN
-#define PATHLENGTH	MAXPATHLEN
-#else
-#define PATHLENGTH	1024	/* maximum file pathname length */
-#endif
+#define DELDIR		BATDIR DIR_SEP_STR "DELETE_ME"
+#define BAKDIR		BATDIR DIR_SEP_STR "BACKUP"
+#define SUBDIR		BAKDIR DIR_SEP_STR "SUBCOMMIT" /* note K, not T */
+#define LEFTDIR		BATDIR DIR_SEP_STR "LEFTOVERS"
+#define TEMPDIR		BATDIR DIR_SEP_STR TEMPDIR_NAME
 
 /*
    See `man mserver5` or tools/mserver/mserver5.1
@@ -538,13 +481,15 @@
 #endif
 #define TYPE_any	255	/* limit types to <255! */
 
-typedef signed char bit;
-typedef signed char bte;
-typedef short sht;
+typedef int8_t bit;
+typedef int8_t bte;
+typedef int16_t sht;
+//typedef int64_t lng;
+//typedef uint64_t ulng;
 
-#define SIZEOF_OID	SIZEOF_SIZE_T
-typedef size_t oid;
-#define OIDFMT		SZFMT
+#define SIZEOF_OID	8
+typedef uint64_t oid;
+#define OIDFMT		ULLFMT
 
 typedef int bat;		/* Index into BBP */
 typedef void *ptr;		/* Internal coding of types */
@@ -554,15 +499,10 @@ typedef float flt;
 typedef double dbl;
 typedef char *str;
 
-#if SIZEOF_INT==8
-#	define LL_CONSTANT(val)	(val)
-#elif SIZEOF_LONG==8
-#	define LL_CONSTANT(val)	(val##L)
-#elif defined(HAVE_LONG_LONG)
-#	define LL_CONSTANT(val)	(val##LL)
-#elif defined(HAVE___INT64)
-#	define LL_CONSTANT(val)	(val##i64)
-#endif
+#define SIZEOF_LNG		8
+#define LL_CONSTANT(val)	INT64_C(val)
+#define LLFMT			"%" PRId64
+#define ULLFMT			"%" PRIu64
 
 typedef oid var_t;		/* type used for heap index of var-sized BAT */
 #define SIZEOF_VAR_T	SIZEOF_OID
@@ -571,21 +511,17 @@ typedef oid var_t;		/* type used for heap index of var-sized BAT */
 #if SIZEOF_VAR_T == SIZEOF_INT
 #define VAR_MAX		((var_t) INT_MAX)
 #else
-#define VAR_MAX		((var_t) LLONG_MAX)
+#define VAR_MAX		((var_t) INT64_MAX)
 #endif
 
 typedef oid BUN;		/* BUN position */
 #define SIZEOF_BUN	SIZEOF_OID
 #define BUNFMT		OIDFMT
-/* alternatively:
-typedef size_t BUN;
-#define SIZEOF_BUN	SIZEOF_SIZE_T
-#define BUNFMT		SZFMT
-*/
+
 #if SIZEOF_BUN == SIZEOF_INT
 #define BUN_NONE ((BUN) INT_MAX)
 #else
-#define BUN_NONE ((BUN) LLONG_MAX)
+#define BUN_NONE ((BUN) INT64_MAX)
 #endif
 #define BUN_MAX (BUN_NONE - 1)	/* maximum allowed size of a BAT */
 
@@ -599,10 +535,10 @@ typedef uint32_t BUN4type;
 #if SIZEOF_BUN > 4
 typedef uint64_t BUN8type;
 #endif
-#define BUN2_NONE ((BUN2type) 0xFFFF)
-#define BUN4_NONE ((BUN4type) 0xFFFFFFFF)
+#define BUN2_NONE ((BUN2type) UINT16_C(0xFFFF))
+#define BUN4_NONE ((BUN4type) UINT32_C(0xFFFFFFFF))
 #if SIZEOF_BUN > 4
-#define BUN8_NONE ((BUN8type) LL_CONSTANT(0xFFFFFFFFFFFFFFFF))
+#define BUN8_NONE ((BUN8type) UINT64_C(0xFFFFFFFFFFFFFFFF))
 #endif
 
 
@@ -629,15 +565,15 @@ typedef struct {
 	size_t free;		/* index where free area starts. */
 	size_t size;		/* size of the heap (bytes) */
 	char *base;		/* base pointer in memory. */
-	str filename;		/* file containing image of the heap */
+	char filename[100];	/* file containing image of the heap */
 
-	unsigned int copied:1,	/* a copy of an existing map. */
+	bool copied:1,		/* a copy of an existing map. */
 		hashash:1,	/* the string heap contains hash values */
 		forcemap:1,	/* force STORE_MMAP even if heap exists */
-		cleanhash:1;	/* string heaps must clean hash */
+		cleanhash:1,	/* string heaps must clean hash */
+		dirty:1;	/* specific heap dirty marker */
 	storage_t storage;	/* storage mode (mmap/malloc). */
 	storage_t newstorage;	/* new desired storage mode at re-allocation. */
-	bte dirty;		/* specific heap dirty marker */
 	bte farmid;		/* id of farm where heap is located */
 	bat parentid;		/* cache id of VIEW parent bat */
 } Heap;
@@ -650,7 +586,7 @@ typedef struct {
 	BUN mask;		/* number of hash buckets-1 (power of 2) */
 	void *Hash;		/* hash table */
 	void *Link;		/* collision list */
-	Heap *heap;		/* heap where the hash is stored */
+	Heap heap;		/* heap where the hash is stored */
 } Hash;
 
 typedef struct Imprints Imprints;
@@ -725,12 +661,13 @@ typedef struct {
 		hge hval;
 #endif
 	} val;
-	int len, vtype;
+	size_t len;
+	int vtype;
 } *ValPtr, ValRecord;
 
 /* interface definitions */
 gdk_export ptr VALconvert(int typ, ValPtr t);
-gdk_export int VALformat(char **buf, const ValRecord *res);
+gdk_export char *VALformat(const ValRecord *res);
 gdk_export ValPtr VALcopy(ValPtr dst, const ValRecord *src);
 gdk_export ValPtr VALinit(ValPtr d, int tpe, const void *s);
 gdk_export void VALempty(ValPtr v);
@@ -795,13 +732,13 @@ typedef struct {
 	MT_Id tid;		/* which thread created it */
 	unsigned int
 	 copiedtodisk:1,	/* once written */
-	 dirty:2,		/* dirty wrt disk? */
+	 dirty:1,		/* dirty wrt disk? */
 	 dirtyflushed:1,	/* was dirty before commit started? */
 	 descdirty:1,		/* bat descriptor dirty marker */
 	 restricted:2,		/* access privileges */
 	 persistence:1,		/* should the BAT persist on disk? */
 	 role:8,		/* role of the bat */
-	 unused:15;		/* value=0 for now */
+	 unused:15;		/* value=0 for now (sneakily used by mat.c) */
 	int sharecnt;		/* incoming view count */
 
 	/* delta status administration */
@@ -819,20 +756,18 @@ typedef struct {
 
 	unsigned short width;	/* byte-width of the atom array */
 	bte type;		/* type id. */
-	bte shift;		/* log2 of bunwidth */
-	unsigned int
-	 varsized:1,		/* varsized (1) or fixedsized (0) */
-	 key:1,			/* no duplicate values present */
-	 unique:1,		/* no duplicate values allowed */
-	 dense:1,		/* OID only: only consecutive values */
-	 nonil:1,		/* there are no nils in the column */
-	 nil:1,			/* there is a nil in the column */
-	 sorted:1,		/* column is sorted in ascending order */
-	 revsorted:1;		/* column is sorted in descending order */
+	bte shift;		/* log2 of bun width */
+	bool varsized:1,	/* varsized (true) or fixedsized (false) */
+		key:1,		/* no duplicate values present */
+		unique:1,	/* no duplicate values allowed */
+		dense:1,	/* OID only: only consecutive values */
+		nonil:1,	/* there are no nils in the column */
+		nil:1,		/* there is a nil in the column */
+		sorted:1,	/* column is sorted in ascending order */
+		revsorted:1;	/* column is sorted in descending order */
 	BUN nokey[2];		/* positions that prove key==FALSE */
 	BUN nosorted;		/* position that proves sorted==FALSE */
 	BUN norevsorted;	/* position that proves revsorted==FALSE */
-	BUN nodense;		/* position that proves dense==FALSE */
 	oid seq;		/* start of dense head sequence */
 
 	Heap heap;		/* space for the column. */
@@ -849,14 +784,15 @@ typedef struct {
 /* assert that atom width is power of 2, i.e., width == 1<<shift */
 #define assert_shift_width(shift,width) assert(((shift) == 0 && (width) == 0) || ((unsigned)1<<(shift)) == (unsigned)(width))
 
-#define GDKLIBRARY_SORTEDPOS	061030	/* version where we can't trust no(rev)sorted */
-#define GDKLIBRARY_OLDWKB	061031	/* old geom WKB format */
-#define GDKLIBRARY_INSERTED	061032	/* inserted and deleted in BBP.dir */
-#define GDKLIBRARY_HEADED	061033	/* head properties are stored */
-#define GDKLIBRARY_NOKEY	061034	/* nokey values can't be trusted */
-#define GDKLIBRARY_BADEMPTY	061035	/* possibility of duplicate empty str */
-#define GDKLIBRARY_TALIGN	061036	/* talign field in BBP.dir */
-#define GDKLIBRARY		061037
+#define GDKLIBRARY_SORTEDPOS	061030U	/* version where we can't trust no(rev)sorted */
+#define GDKLIBRARY_OLDWKB	061031U	/* old geom WKB format */
+#define GDKLIBRARY_INSERTED	061032U	/* inserted and deleted in BBP.dir */
+#define GDKLIBRARY_HEADED	061033U	/* head properties are stored */
+#define GDKLIBRARY_NOKEY	061034U	/* nokey values can't be trusted */
+#define GDKLIBRARY_BADEMPTY	061035U	/* possibility of duplicate empty str */
+#define GDKLIBRARY_TALIGN	061036U	/* talign field in BBP.dir */
+#define GDKLIBRARY_NIL_NAN	061037U	/* flt/dbl NIL not represented by NaN */
+#define GDKLIBRARY		061040U
 
 typedef struct BAT {
 	/* static bat properties */
@@ -905,7 +841,6 @@ typedef struct BATiter {
 #define tnokey		T.nokey
 #define tnosorted	T.nosorted
 #define tnorevsorted	T.norevsorted
-#define tnodense	T.nodense
 #define theap		T.heap
 #define tvheap		T.vheap
 #define thash		T.hash
@@ -1072,7 +1007,7 @@ gdk_export bte ATOMelmshift(int sz);
  * pointer and BUN identifier.
  * @itemize
  * @item
- * BAThtype(b) and  BATttype(b) find out the head and tail type of a BAT.
+ * BATttype(b) finds out the type of a BAT.
  * @item
  * BUNlast(b) returns the BUN pointer directly after the last BUN
  * in the BAT.
@@ -1110,7 +1045,7 @@ gdk_export bte ATOMelmshift(int sz);
 		} else {						\
 			ATOMputFIX((b)->ttype, (p), v);			\
 		}							\
-	} while (0)
+	} while (false)
 #define Treplacevalue(b, p, v)						\
 	do {								\
 		if ((b)->tvarsized && (b)->ttype) {			\
@@ -1135,7 +1070,7 @@ gdk_export bte ATOMelmshift(int sz);
 			if ((b)->twidth < SIZEOF_VAR_T &&		\
 			    ((b)->twidth <= 2 ? _d - GDK_VAROFFSET : _d) >= ((size_t) 1 << (8 * (b)->twidth))) { \
 				/* doesn't fit in current heap, upgrade it */ \
-				if (GDKupgradevarheap((b), _d, 0, (b)->batRestricted == BAT_READ) != GDK_SUCCEED) \
+				if (GDKupgradevarheap((b), _d, false, (b)->batRestricted == BAT_READ) != GDK_SUCCEED) \
 					goto bunins_failed;		\
 			}						\
 			_ptr = (p);					\
@@ -1156,7 +1091,7 @@ gdk_export bte ATOMelmshift(int sz);
 		} else {						\
 			ATOMreplaceFIX((b)->ttype, (p), v);		\
 		}							\
-	} while (0)
+	} while (false)
 #else
 #define Tputvalue(b, p, v, copyall)					\
 	do {								\
@@ -1185,7 +1120,7 @@ gdk_export bte ATOMelmshift(int sz);
 		} else {						\
 			ATOMputFIX((b)->ttype, (p), v);			\
 		}							\
-	} while (0)
+	} while (false)
 #define Treplacevalue(b, p, v)						\
 	do {								\
 		if ((b)->tvarsized && (b)->ttype) {			\
@@ -1207,7 +1142,7 @@ gdk_export bte ATOMelmshift(int sz);
 			if ((b)->twidth < SIZEOF_VAR_T &&		\
 			    ((b)->twidth <= 2 ? _d - GDK_VAROFFSET : _d) >= ((size_t) 1 << (8 * (b)->twidth))) { \
 				/* doesn't fit in current heap, upgrade it */ \
-				if (GDKupgradevarheap((b), _d, 0, (b)->batRestricted == BAT_READ) != GDK_SUCCEED) \
+				if (GDKupgradevarheap((b), _d, false, (b)->batRestricted == BAT_READ) != GDK_SUCCEED) \
 					goto bunins_failed;		\
 			}						\
 			_ptr = (p);					\
@@ -1225,40 +1160,40 @@ gdk_export bte ATOMelmshift(int sz);
 		} else {						\
 			ATOMreplaceFIX((b)->ttype, (p), v);		\
 		}							\
-	} while (0)
+	} while (false)
 #endif
 #define tfastins_nocheck(b, p, v, s)			\
 	do {						\
 		(b)->theap.free += (s);			\
 		(b)->theap.dirty |= (s) != 0;		\
 		Tputvalue((b), Tloc((b), (p)), (v), 0);	\
-	} while (0)
+	} while (false)
 
 #define bunfastapp_nocheck(b, p, t, ts)		\
 	do {					\
 		tfastins_nocheck(b, p, t, ts);	\
 		(b)->batCount++;		\
-	} while (0)
+	} while (false)
 
 #define bunfastapp_nocheck_inc(b, p, t)			\
 	do {						\
 		bunfastapp_nocheck(b, p, t, Tsize(b));	\
 		p++;					\
-	} while (0)
+	} while (false)
 
 #define bunfastapp(b, t)						\
 	do {								\
 		BUN _p = BUNlast(b);					\
 		if (_p >= BATcapacity(b)) {				\
 			if (_p == BUN_MAX || BATcount(b) == BUN_MAX) {	\
-				GDKerror("bunfastapp: too many elements to accomodate (" BUNFMT ")\n", BUN_MAX); \
+				GDKerror("bunfastapp: too many elements to accomodate\n"); \
 				goto bunins_failed;			\
 			}						\
 			if (BATextend((b), BATgrows(b)) != GDK_SUCCEED)	\
 				goto bunins_failed;			\
 		}							\
 		bunfastapp_nocheck(b, _p, t, Tsize(b));			\
-	} while (0)
+	} while (false)
 
 gdk_export gdk_return GDKupgradevarheap(BAT *b, var_t v, int copyall, int mayshare)
 	__attribute__ ((__warn_unused_result__));
@@ -1290,13 +1225,13 @@ gdk_export BUN ORDERfndlast(BAT *b, const void *v);
 gdk_export BUN BUNfnd(BAT *b, const void *right);
 
 #define BUNfndVOID(b, v)						\
-	(((*(const oid*)(v) == oid_nil) ^ ((b)->tseqbase == oid_nil)) | \
+	((is_oid_nil(*(const oid*)(v)) ^ is_oid_nil((b)->tseqbase)) |	\
 		(*(const oid*)(v) < (b)->tseqbase) |			\
 		(*(const oid*)(v) >= (b)->tseqbase + (b)->batCount) ?	\
 	 BUN_NONE :							\
 	 (BUN) (*(const oid*)(v) - (b)->tseqbase))
 
-#define BATttype(b)	((b)->ttype == TYPE_void && (b)->tseqbase != oid_nil ? \
+#define BATttype(b)	((b)->ttype == TYPE_void && !is_oid_nil((b)->tseqbase) ? \
 			 TYPE_oid : (b)->ttype)
 #define Tbase(b)	((b)->tvheap->base)
 
@@ -1398,22 +1333,22 @@ gdk_export void BATsetcount(BAT *b, BUN cnt);
 gdk_export BUN BATgrows(BAT *b);
 gdk_export gdk_return BATkey(BAT *b, int onoff);
 gdk_export gdk_return BATmode(BAT *b, int onoff);
-gdk_export void BATroles(BAT *b, const char *tnme);
+gdk_export gdk_return BATroles(BAT *b, const char *tnme);
 gdk_export void BAThseqbase(BAT *b, oid o);
 gdk_export void BATtseqbase(BAT *b, oid o);
 gdk_export gdk_return BATsetaccess(BAT *b, int mode);
 gdk_export int BATgetaccess(BAT *b);
 
 
-#define BATdirty(b)	((b)->batCopiedtodisk == 0 || (b)->batDirty ||	\
+#define BATdirty(b)	(!(b)->batCopiedtodisk || (b)->batDirty ||	\
 			 (b)->batDirtydesc ||				\
 			 (b)->theap.dirty ||				\
-			 ((b)->tvheap?(b)->tvheap->dirty:0))
+			 ((b)->tvheap?(b)->tvheap->dirty:false))
 
 #define PERSISTENT		0
 #define TRANSIENT		1
 #define LOG_DIR			2
-#define SHARED_LOG_DIR	3
+#define SHARED_LOG_DIR		3
 
 #define BAT_WRITE		0	/* all kinds of access allowed */
 #define BAT_READ		1	/* only read-access allowed */
@@ -1469,8 +1404,6 @@ gdk_export gdk_return BATgroup(BAT **groups, BAT **extents, BAT **histo, BAT *b,
 
 gdk_export void BATmsync(BAT *b);
 
-gdk_export size_t BATmemsize(BAT *b, int dirty);
-
 #define NOFARM (-1) /* indicate to GDKfilepath to create relative path */
 
 gdk_export char *GDKfilepath(int farmid, const char *dir, const char *nme, const char *ext);
@@ -1522,69 +1455,69 @@ gdk_export gdk_return BATsort(BAT **sorted, BAT **order, BAT **groups, BAT *b, B
 	__attribute__ ((__warn_unused_result__));
 
 
-gdk_export void GDKqsort(void *h, void *t, const void *base, size_t n, int hs, int ts, int tpe);
-gdk_export void GDKqsort_rev(void *h, void *t, const void *base, size_t n, int hs, int ts, int tpe);
+gdk_export void GDKqsort(void *restrict h, void *restrict t, const void *restrict base, size_t n, int hs, int ts, int tpe);
+gdk_export void GDKqsort_rev(void *restrict h, void *restrict t, const void *restrict base, size_t n, int hs, int ts, int tpe);
 
 #define BATtordered(b)	((b)->ttype == TYPE_void || (b)->tsorted)
-#define BATtrevordered(b) (((b)->ttype == TYPE_void && (b)->tseqbase == oid_nil) || (b)->trevsorted)
-#define BATtdense(b)	(BATtvoid(b) && (b)->tseqbase != oid_nil)
+#define BATtrevordered(b) (((b)->ttype == TYPE_void && is_oid_nil((b)->tseqbase)) || (b)->trevsorted)
+#define BATtdense(b)	(BATtvoid(b) && !is_oid_nil((b)->tseqbase))
 #define BATtvoid(b)	(((b)->tdense && (b)->tsorted) || (b)->ttype==TYPE_void)
-#define BATtkey(b)	(b->tkey != FALSE || BATtdense(b))
+#define BATtkey(b)	(b->tkey || BATtdense(b))
 
 /* set some properties that are trivial to deduce */
 #define BATsettrivprop(b)						\
 	do {								\
-		assert((b)->hseqbase != oid_nil);			\
-		(b)->batDirtydesc = 1;	/* likely already set */	\
+		assert(!is_oid_nil((b)->hseqbase));			\
+		(b)->batDirtydesc = true;	/* likely already set */ \
 		/* the other head properties should already be correct */ \
 		if ((b)->ttype == TYPE_void) {				\
-			if ((b)->tseqbase == oid_nil) {			\
+			if (is_oid_nil((b)->tseqbase)) {		\
 				(b)->tnonil = (b)->batCount == 0;	\
 				(b)->tnil = !(b)->tnonil;		\
-				(b)->trevsorted = 1;			\
+				(b)->trevsorted = true;			\
 				(b)->tkey = (b)->batCount <= 1;		\
-				(b)->tdense = 0;			\
+				(b)->tdense = false;			\
 			} else {					\
-				(b)->tdense = 1;			\
-				(b)->tnonil = 1;			\
-				(b)->tnil = 0;				\
-				(b)->tkey = 1;				\
+				(b)->tdense = true;			\
+				(b)->tnonil = true;			\
+				(b)->tnil = false;			\
+				(b)->tkey = true;			\
 				(b)->trevsorted = (b)->batCount <= 1;	\
 			}						\
-			(b)->tsorted = 1;				\
+			(b)->tsorted = true;				\
 		} else if ((b)->batCount <= 1) {			\
 			if (ATOMlinear((b)->ttype)) {			\
-				(b)->tsorted = 1;			\
-				(b)->trevsorted = 1;			\
+				(b)->tsorted = true;			\
+				(b)->trevsorted = true;			\
 			}						\
-			(b)->tkey = 1;					\
+			(b)->tkey = true;				\
 			if ((b)->batCount == 0) {			\
-				(b)->tnonil = 1;			\
-				(b)->tnil = 0;				\
+				(b)->tnonil = true;			\
+				(b)->tnil = false;			\
 				if ((b)->ttype == TYPE_oid) {		\
-					(b)->tdense = 1;		\
+					(b)->tdense = true;		\
 					(b)->tseqbase = 0;		\
 				}					\
 			} else if ((b)->ttype == TYPE_oid) {		\
 				/* b->batCount == 1 */			\
-				oid sqbs;				\
-				if ((sqbs = ((oid *) (b)->theap.base)[0]) == oid_nil) { \
-					(b)->tdense = 0;		\
-					(b)->tnonil = 0;		\
-					(b)->tnil = 1;			\
+				oid sqbs = ((const oid *) (b)->theap.base)[0]; \
+				if (is_oid_nil(sqbs)) {			\
+					(b)->tdense = false;		\
+					(b)->tnonil = false;		\
+					(b)->tnil = true;		\
 				} else {				\
-					(b)->tdense = 1;		\
-					(b)->tnonil = 1;		\
-					(b)->tnil = 0;			\
+					(b)->tdense = true;		\
+					(b)->tnonil = true;		\
+					(b)->tnil = false;		\
 				}					\
 				(b)->tseqbase = sqbs;			\
 			}						\
 		}							\
 		if (!ATOMlinear((b)->ttype)) {				\
-			(b)->tsorted = 0;				\
-			(b)->trevsorted = 0;				\
+			(b)->tsorted = false;				\
+			(b)->trevsorted = false;			\
 		}							\
-	} while (0)
+	} while (false)
 
 /*
  * @+ BAT Buffer Pool
@@ -1608,11 +1541,7 @@ gdk_export void GDKqsort_rev(void *h, void *t, const void *base, size_t n, int h
  * @end multitable
  *
  * The BAT Buffer Pool module contains the code to manage the storage
- * location of BATs. It uses two tables BBPlogical and BBphysical to
- * relate the BAT name with its corresponding file system name.  This
- * information is retained in an ASCII file within the database home
- * directory for ease of inspection. It is loaded upon restart of the
- * server and saved upon transaction commit (if necessary).
+ * location of BATs.
  *
  * The remaining BBP tables contain status information to load, swap
  * and migrate the BATs. The core table is BBPcache which contains a
@@ -1638,11 +1567,11 @@ gdk_export void GDKqsort_rev(void *h, void *t, const void *base, size_t n, int h
  */
 typedef struct {
 	BAT *cache;		/* if loaded: BAT* handle */
-	str logical;		/* logical name */
-	str bak;		/* logical name backup */
+	char *logical;		/* logical name (may point at bak) */
+	char bak[16];		/* logical name backup (tmp_%o) */
 	bat next;		/* next BBP slot in linked list */
 	BAT *desc;		/* the BAT descriptor */
-	str physical;		/* dir + basename for storage */
+	char physical[30];	/* dir + basename for storage */
 	str options;		/* A string list of options */
 	int refs;		/* in-memory references on which the loaded status of a BAT relies */
 	int lrefs;		/* logical references on which the existence of a BAT relies */
@@ -1658,7 +1587,12 @@ gdk_export bat BBPlimit;
 #define BBPINITLOG	14
 #endif
 #define BBPINIT		(1 << BBPINITLOG)
-/* absolute maximum number of BATs is N_BBPINIT * BBPINIT */
+/* absolute maximum number of BATs is N_BBPINIT * BBPINIT
+ * this also gives the longest possible "physical" name and "bak" name
+ * of a BAT: the "bak" name is "tmp_%o", so at most 12 + \0 bytes on
+ * 64 bit architecture and 11 + \0 on 32 bit architecture; the
+ * physical name is a bit more complicated, but the longest possible
+ * name is 17 + \0 bytes (16 + \0 on 32 bits) */
 gdk_export BBPrec *BBP[N_BBPINIT];
 
 /* fast defines without checks; internal use only  */
@@ -1694,8 +1628,6 @@ gdk_export void BBPlock(void);
 
 gdk_export void BBPunlock(void);
 
-gdk_export str BBPlogical(bat b, str buf);
-gdk_export str BBPphysical(bat b, str buf);
 gdk_export BAT *BBPquickdesc(bat b, int delaccess);
 
 /*
@@ -1717,18 +1649,16 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  * @tab ATOMdelete      (int id);
  * @item str
  * @tab ATOMname        (int id);
- * @item int
+ * @item unsigned int
  * @tab ATOMsize        (int id);
  * @item int
- * @tab ATOMalign       (int id);
- * @item int
  * @tab ATOMvarsized    (int id);
  * @item ptr
  * @tab ATOMnilptr      (int id);
- * @item int
- * @tab ATOMfromstr     (int id, str s, int* len, ptr* v_dst);
- * @item int
- * @tab ATOMtostr       (int id, str s, int* len, ptr* v_dst);
+ * @item ssize_t
+ * @tab ATOMfromstr     (int id, str s, size_t* len, ptr* v_dst);
+ * @item ssize_t
+ * @tab ATOMtostr       (int id, str s, size_t* len, ptr* v_dst);
  * @item hash_t
  * @tab ATOMhash        (int id, ptr val, in mask);
  * @item int
@@ -1743,11 +1673,11 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  * @tab ATOMput         (int id, Heap *hp, BUN pos_dst, ptr val_src);
  * @item int
  * @tab ATOMdel         (int id, Heap *hp, BUN v_src);
- * @item int
+ * @item size_t
  * @tab ATOMlen         (int id, ptr val);
  * @item ptr
  * @tab ATOMnil         (int id);
- * @item int
+ * @item ssize_t
  * @tab ATOMformat      (int id, ptr val, char** buf);
  * @item int
  * @tab ATOMprint       (int id, ptr val, stream *fd);
@@ -1776,11 +1706,6 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  *
  * @item The @emph{ATOMsize()} operation returns the atoms fixed size.
  *
- * @item The @emph{ATOMalign()} operation returns the atoms minimum
- * alignment. If the alignment info was not specified explicitly
- * during atom install, it assumes the maximum value of @verb{ {
- * }1,2,4,8@verb{ } } smaller than the atom size.
- *
  * @item The @emph{ATOMnilptr()} operation returns a pointer to the
  * nil-value of an atom. We usually take one dedicated value halfway
  * down the negative extreme of the atom range (if such a concept
@@ -1796,7 +1721,7 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  * value. `val' is a direct pointer to the atom value. Its return
  * value should be an hash_t between 0 and 'mask'.
  *
- * @item The @emph{ATOMcmp()} operation computes two atomic
+ * @item The @emph{ATOMcmp()} operation compares two atomic
  * values. Its parameters are pointers to atomic values.
  *
  * @item The @emph{ATOMlen()} operation computes the byte length for a
@@ -1824,7 +1749,8 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  * @item The @emph{ATOMfromstr()} parses an atom value from string
  * `s'. The memory allocation policy is the same as in
  * @emph{ATOMget()}. The return value is the number of parsed
- * characters.
+ * characters or -1 on failure.  Also in case of failure, the output
+ * parameter buf is a valid pointer or NULL.
  *
  * @item The @emph{ATOMprint()} prints an ASCII description of the
  * atom value pointed to by `val' on file descriptor `fd'. The return
@@ -1846,20 +1772,32 @@ gdk_export BAT *BBPquickdesc(bat b, int delaccess);
  * and @emph{ATOMformat()}) just have the atom id parameter prepended
  * to them.
  */
+
+/* atomFromStr returns the number of bytes of the input string that
+ * were processed.  atomToStr returns the length of the string
+ * produced.  Both functions return -1 on (any kind of) failure.  If
+ * *dst is not NULL, *len specifies the available space.  If there is
+ * not enough space, of if *dst is NULL, *dst will be freed (if not
+ * NULL) and a new buffer will be allocated and returned in *dst.
+ * *len will be set to reflect the actual size allocated.  If
+ * allocation fails, *dst will be NULL on return and *len is
+ * undefined.  In any case, if the function returns, *buf is either
+ * NULL or a valid pointer and then *len is the size of the area *buf
+ * points to. */
+
 typedef struct {
 	/* simple attributes */
 	char name[IDLENGTH];
-	short storage;		/* stored as another type? */
-	short linear;		/* atom can be ordered linearly */
-	short size;		/* fixed size of atom */
-	short align;		/* alignment condition for values */
+	uint8_t storage;	/* stored as another type? */
+	bool linear;		/* atom can be ordered linearly */
+	unsigned short size;	/* fixed size of atom */
 
 	/* automatically generated fields */
 	const void *atomNull;	/* global nil value */
 
 	/* generic (fixed + varsized atom) ADT functions */
-	int (*atomFromStr) (const char *src, int *len, ptr *dst);
-	int (*atomToStr) (str *dst, int *len, const void *src);
+	ssize_t (*atomFromStr) (const char *src, size_t *len, ptr *dst);
+	ssize_t (*atomToStr) (str *dst, size_t *len, const void *src);
 	void *(*atomRead) (void *dst, stream *s, size_t cnt);
 	gdk_return (*atomWrite) (const void *src, stream *s, size_t cnt);
 	int (*atomCmp) (const void *v1, const void *v2);
@@ -1871,7 +1809,7 @@ typedef struct {
 	/* varsized atom-only ADT functions */
 	var_t (*atomPut) (Heap *, var_t *off, const void *src);
 	void (*atomDel) (Heap *, var_t *atom);
-	int (*atomLen) (const void *atom);
+	size_t (*atomLen) (const void *atom);
 	void (*atomHeap) (Heap *, size_t);
 } atomDesc;
 
@@ -1882,11 +1820,10 @@ gdk_export int ATOMallocate(const char *nme);
 gdk_export int ATOMindex(const char *nme);
 
 gdk_export str ATOMname(int id);
-gdk_export int ATOMlen(int id, const void *v);
+gdk_export size_t ATOMlen(int id, const void *v);
 gdk_export ptr ATOMnil(int id);
-gdk_export int ATOMcmp(int id, const void *v_1, const void *v_2);
 gdk_export int ATOMprint(int id, const void *val, stream *fd);
-gdk_export int ATOMformat(int id, const void *val, char **buf);
+gdk_export char *ATOMformat(int id, const void *val);
 
 gdk_export ptr ATOMdup(int id, const void *val);
 
@@ -1994,8 +1931,6 @@ gdk_export str GDKstrdup(const char *s)
 gdk_export str GDKstrndup(const char *s, size_t n)
 	__attribute__ ((__warn_unused_result__));
 
-
-
 /*
  * @- GDK error handling
  *  @multitable @columnfractions 0.08 0.7
@@ -2069,7 +2004,7 @@ gdk_export void GDKclrerr(void);
 #include "gdk_utils.h"
 
 /* functions defined in gdk_bat.c */
-gdk_export BUN void_replace_bat(BAT *b, BAT *p, BAT *u, bit force)
+gdk_export gdk_return void_replace_bat(BAT *b, BAT *p, BAT *u, bit force)
 	__attribute__ ((__warn_unused_result__));
 gdk_export gdk_return void_inplace(BAT *b, oid id, const void *val, bit force)
 	__attribute__ ((__warn_unused_result__));
@@ -2122,7 +2057,7 @@ typedef struct threadStruct {
 	MT_Id pid;		/* physical thread id (pointer-sized) from the OS thread library */
 	str name;
 	ptr data[THREADDATA];
-	size_t sp;
+	uintptr_t sp;
 } ThreadRec, *Thread;
 
 
@@ -2157,7 +2092,7 @@ gdk_export void *THRdata[THREADDATA];
 static inline bat
 BBPcheck(bat x, const char *y)
 {
-	if (x && x != bat_nil) {
+	if (!is_bat_nil(x)) {
 		assert(x > 0);
 
 		if (x < 0 || x >= getBBPsize() || BBP_logical(x) == NULL) {
@@ -2187,7 +2122,7 @@ static inline char *
 Tpos(BATiter *bi, BUN p)
 {
 	bi->tvid = bi->b->tseqbase;
-	if (bi->tvid != oid_nil)
+	if (!is_oid_nil(bi->tvid))
 		bi->tvid += p;
 	return (char*)&bi->tvid;
 }
@@ -2310,8 +2245,6 @@ gdk_export void BATundo(BAT *b);
  * @tab ALIGNsync   (BAT *b1, BAT *b2)
  * @item int
  * @tab ALIGNrelated (BAT *b1, BAT *b2)
- * @item int
- * @tab ALIGNsetT    ((BAT *dst, BAT *src)
  *
  * @item BAT*
  * @tab VIEWcreate   (oid seq, BAT *b)
@@ -2323,8 +2256,6 @@ gdk_export void BATundo(BAT *b);
  * @tab VIEWtparent   (BAT *b)
  * @item BAT*
  * @tab VIEWreset    (BAT *b)
- * @item BAT*
- * @tab BATmaterialize  (BAT *b)
  * @end multitable
  *
  * Alignments of two columns of a BAT means that the system knows
@@ -2346,9 +2277,6 @@ gdk_export void BATundo(BAT *b);
  * VIEWreset creates a normal BAT with the same contents as its view
  * parameter (it converts void columns with seqbase!=nil to
  * materialized oid columns).
- *
- * The BATmaterialize materializes a VIEW (TODO) or void bat inplace.
- * This is useful as materialization is usually needed for updates.
  */
 gdk_export int ALIGNsynced(BAT *b1, BAT *b2);
 
@@ -2359,18 +2287,17 @@ gdk_export void BATassertProps(BAT *b);
 #define BATPROPS_CHECK  3	/* BATPROPS_ALL, but start from scratch and report illegally set properties */
 
 gdk_export BAT *VIEWcreate(oid seq, BAT *b);
-gdk_export BAT *VIEWcreate_(oid seq, BAT *b, int stable);
 gdk_export void VIEWbounds(BAT *b, BAT *view, BUN l, BUN h);
 
-/* low level functions */
-gdk_export void ALIGNsetH(BAT *b1, BAT *b2);
-gdk_export void ALIGNsetT(BAT *b1, BAT *b2);
-
-#define ALIGNinp(x,y,f,e)	do {if (!(f)) VIEWchk(x,y,BAT_READ|BAT_APPEND,e); } while (0)
-#define ALIGNapp(x,y,f,e)	do {if (!(f)) VIEWchk(x,y,BAT_READ,e); } while (0)
-
-#define BAThrestricted(b) ((b)->batRestricted)
-#define BATtrestricted(b) (VIEWtparent(b) ? BBP_cache(VIEWtparent(b))->batRestricted : (b)->batRestricted)
+#define ALIGNapp(x, y, f, e)						\
+	do {								\
+		if (!(f) && ((x)->batRestricted == BAT_READ ||		\
+			     (x)->batSharecnt > 0)) {			\
+			GDKerror("%s: access denied to %s, aborting.\n", \
+				 (y), BATgetId(x));			\
+			return (e);					\
+		}							\
+	} while (false)
 
 /* The batRestricted field indicates whether a BAT is readonly.
  * we have modes: BAT_WRITE  = all permitted
@@ -2378,14 +2305,9 @@ gdk_export void ALIGNsetT(BAT *b1, BAT *b2);
  *                BAT_READ   = read-only
  * VIEW bats are always mapped read-only.
  */
-#define	VIEWchk(x,y,z,e)						\
-	do {								\
-		if ((((x)->batRestricted & (z)) != 0) | ((x)->batSharecnt > 0)) { \
-			GDKerror("%s: access denied to %s, aborting.\n", \
-				 (y), BATgetId(x));			\
-			return (e);					\
-		}							\
-	} while (0)
+
+#define BAThrestricted(b) ((b)->batRestricted)
+#define BATtrestricted(b) (VIEWtparent(b) ? BBP_cache(VIEWtparent(b))->batRestricted : (b)->batRestricted)
 
 /* the parentid in a VIEW is correct for the normal view. We must
  * correct for the reversed view.
@@ -2498,7 +2420,7 @@ gdk_export void ALIGNsetT(BAT *b1, BAT *b2);
 	for (hb = HASHget(h, hash_##TYPE(h, v));		\
 	     hb != HASHnil(h);					\
 	     hb = HASHgetlink(h,hb))				\
-		if (simple_EQ(v, BUNtloc(bi, hb), TYPE))
+		if (* (const TYPE *) v == * (const TYPE *) BUNtloc(bi, hb))
 
 #define HASHloop_bte(bi, h, hb, v)	HASHloop_TYPE(bi, h, hb, v, bte)
 #define HASHloop_sht(bi, h, hb, v)	HASHloop_TYPE(bi, h, hb, v, sht)
