@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef _GDK_DELTA_H_
@@ -15,19 +15,12 @@
  * e.g. a BAT[void,bit] is (at least) integer aligned.  This optimizes
  * processing on such BATs (DDBENCH).
  */
-#define DELTAinit(P1)							\
-	do {								\
-		BATsetcount((P1), 0);					\
-		(P1)->theap.free = 0;					\
-		(P1)->batInserted = 0;					\
-		(P1)->tshift = ATOMelmshift(Tsize(P1));			\
-		DELTADEBUG fprintf(stderr,				\
-			"#DELTAinit %s free " SZFMT " ins " BUNFMT	\
-			" base " PTRFMT "\n",				\
-			BATgetId(P1),					\
-			(P1)->theap.free,				\
-			(P1)->batInserted,				\
-			PTRFMTCAST (P1)->theap.base);			\
+#define DELTAinit(P1)						\
+	do {							\
+		BATsetcount((P1), 0);				\
+		(P1)->theap.free = 0;				\
+		(P1)->batInserted = 0;				\
+		(P1)->tshift = ATOMelmshift(Tsize(P1));		\
 	} while (0)
 /*
  * Upon saving a BAT, we should convert the delta marker BUN pointers
