@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -87,12 +87,26 @@
 			siftup(OPER, i - 1, SWAP);	\
 	} while (0)
 
-#define LTany(p1, p2)	(cmp(BUNtail(bi, oids[p1] - b->hseqbase), \
+#define LTany(p1, p2)	(cmp(BUNtail(bi, oids[p1] - b->hseqbase),	\
 			     BUNtail(bi, oids[p2] - b->hseqbase)) < 0)
-#define GTany(p1, p2)	(cmp(BUNtail(bi, oids[p1] - b->hseqbase), \
+#define GTany(p1, p2)	(cmp(BUNtail(bi, oids[p1] - b->hseqbase),	\
 			     BUNtail(bi, oids[p2] - b->hseqbase)) > 0)
-#define LTfix(p1, p2)	(vals[oids[p1] - b->hseqbase] < vals[oids[p2] - b->hseqbase])
-#define GTfix(p1, p2)	(vals[oids[p1] - b->hseqbase] > vals[oids[p2] - b->hseqbase])
+#define LTflt(a, b)	((bit) (!is_flt_nil(b) && (is_flt_nil(a) || (a) < (b))))
+#define LTdbl(a, b)	((bit) (!is_dbl_nil(b) && (is_dbl_nil(a) || (a) < (b))))
+#define GTflt(a, b)	((bit) (!is_flt_nil(a) && (is_flt_nil(b) || (a) > (b))))
+#define GTdbl(a, b)	((bit) (!is_dbl_nil(a) && (is_dbl_nil(b) || (a) > (b))))
+#define LTfltfix(p1, p2)	LTflt(vals[oids[p1] - b->hseqbase],	\
+				      vals[oids[p2] - b->hseqbase])
+#define GTfltfix(p1, p2)	GTflt(vals[oids[p1] - b->hseqbase],	\
+				      vals[oids[p2] - b->hseqbase])
+#define LTdblfix(p1, p2)	LTdbl(vals[oids[p1] - b->hseqbase],	\
+				      vals[oids[p2] - b->hseqbase])
+#define GTdblfix(p1, p2)	GTdbl(vals[oids[p1] - b->hseqbase],	\
+				      vals[oids[p2] - b->hseqbase])
+#define LTfix(p1, p2)		LT(vals[oids[p1] - b->hseqbase],	\
+				   vals[oids[p2] - b->hseqbase])
+#define GTfix(p1, p2)		GT(vals[oids[p1] - b->hseqbase],	\
+				   vals[oids[p2] - b->hseqbase])
 #define SWAP1(p1, p2)				\
 	do {					\
 		item = oids[p1];		\
@@ -248,10 +262,10 @@ BATfirstn_unique(BAT *b, BAT *s, BUN n, int asc, oid *lastp)
 			break;
 #endif
 		case TYPE_flt:
-			shuffle_unique(flt, LT);
+			shuffle_unique(flt, LTflt);
 			break;
 		case TYPE_dbl:
-			shuffle_unique(dbl, LT);
+			shuffle_unique(dbl, LTdbl);
 			break;
 		default:
 			heapify(LTany, SWAP1);
@@ -285,10 +299,10 @@ BATfirstn_unique(BAT *b, BAT *s, BUN n, int asc, oid *lastp)
 			break;
 #endif
 		case TYPE_flt:
-			shuffle_unique(flt, GT);
+			shuffle_unique(flt, GTflt);
 			break;
 		case TYPE_dbl:
-			shuffle_unique(dbl, GT);
+			shuffle_unique(dbl, GTdbl);
 			break;
 		default:
 			heapify(GTany, SWAP1);
@@ -316,14 +330,30 @@ BATfirstn_unique(BAT *b, BAT *s, BUN n, int asc, oid *lastp)
 	return bn;
 }
 
-#define LTfixgrp(p1, p2)						\
-	(goids[p1] < goids[p2] ||					\
-	 (goids[p1] == goids[p2] &&					\
-	  vals[oids[p1] - b->hseqbase] < vals[oids[p2] - b->hseqbase]))
-#define GTfixgrp(p1, p2)						\
-	(goids[p1] < goids[p2] ||					\
-	 (goids[p1] == goids[p2] &&					\
-	  vals[oids[p1] - b->hseqbase] > vals[oids[p2] - b->hseqbase]))
+#define LTfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  LTfix(p1, p2)))
+#define LTfltfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  LTfltfix(p1, p2)))
+#define LTdblfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  LTdblfix(p1, p2)))
+#define GTfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  GTfix(p1, p2)))
+#define GTfltfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  GTfltfix(p1, p2)))
+#define GTdblfixgrp(p1, p2)			\
+	(goids[p1] < goids[p2] ||		\
+	 (goids[p1] == goids[p2] &&		\
+	  GTdblfix(p1, p2)))
 #define LTvoidgrp(p1, p2)					\
 	(goids[p1] < goids[p2] ||				\
 	 (goids[p1] == goids[p2] && oids[p1] < oids[p2]))
@@ -333,13 +363,11 @@ BATfirstn_unique(BAT *b, BAT *s, BUN n, int asc, oid *lastp)
 #define LTanygrp(p1, p2)					\
 	(goids[p1] < goids[p2] ||				\
 	 (goids[p1] == goids[p2] &&				\
-	  cmp(BUNtail(bi, oids[p1] - b->hseqbase),		\
-	      BUNtail(bi, oids[p2] - b->hseqbase)) < 0))
+	  LTany(p1, p2)))
 #define GTanygrp(p1, p2)					\
 	(goids[p1] < goids[p2] ||				\
 	 (goids[p1] == goids[p2] &&				\
-	  cmp(BUNtail(bi, oids[p1] - b->hseqbase),		\
-	      BUNtail(bi, oids[p2] - b->hseqbase)) > 0))
+	  GTany(p1, p2)))
 #define SWAP2(p1, p2)				\
 	do {					\
 		item = oids[p1];		\
@@ -498,10 +526,10 @@ BATfirstn_unique_with_groups(BAT *b, BAT *s, BAT *g, BUN n, int asc, oid *lastp,
 			break;
 #endif
 		case TYPE_flt:
-			shuffle_unique_with_groups(flt, LT);
+			shuffle_unique_with_groups(flt, LTflt);
 			break;
 		case TYPE_dbl:
-			shuffle_unique_with_groups(dbl, LT);
+			shuffle_unique_with_groups(dbl, LTdbl);
 			break;
 		default:
 			heapify(LTanygrp, SWAP2);
@@ -553,10 +581,10 @@ BATfirstn_unique_with_groups(BAT *b, BAT *s, BAT *g, BUN n, int asc, oid *lastp,
 			break;
 #endif
 		case TYPE_flt:
-			shuffle_unique_with_groups(flt, GT);
+			shuffle_unique_with_groups(flt, GTflt);
 			break;
 		case TYPE_dbl:
-			shuffle_unique_with_groups(dbl, GT);
+			shuffle_unique_with_groups(dbl, GTdbl);
 			break;
 		default:
 			heapify(GTanygrp, SWAP2);
