@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -159,12 +159,20 @@
 		}							\
 	} while(0)
 
+#define flt_neq(a, b)	(is_flt_nil(a) ? !is_flt_nil(b) : is_flt_nil(b) || (a) != (b))
+#define dbl_neq(a, b)	(is_dbl_nil(a) ? !is_dbl_nil(b) : is_dbl_nil(b) || (a) != (b))
+#define bte_neq(a, b)	((a) != (b))
+#define sht_neq(a, b)	((a) != (b))
+#define int_neq(a, b)	((a) != (b))
+#define lng_neq(a, b)	((a) != (b))
+#define hge_neq(a, b)	((a) != (b))
+
 #define GRP_compare_consecutive_values_tpe(TYPE)		\
 	GRP_compare_consecutive_values(				\
 	/* INIT_0 */	const TYPE *w = (TYPE *) Tloc(b, 0);	\
 			TYPE pw = 0			,	\
 	/* INIT_1 */					,	\
-	/* COMP   */	w[p] != pw			,	\
+	/* COMP   */	TYPE##_neq(w[p], pw)		,	\
 	/* KEEP   */	pw = w[p]				\
 	)
 
@@ -227,12 +235,20 @@
 		}							\
 	} while(0)
 
+#define flt_equ(a, b)	(is_flt_nil(a) ? is_flt_nil(b) : !is_flt_nil(b) && (a) == (b))
+#define dbl_equ(a, b)	(is_dbl_nil(a) ? is_dbl_nil(b) : !is_dbl_nil(b) && (a) == (b))
+#define bte_equ(a, b)	((a) == (b))
+#define sht_equ(a, b)	((a) == (b))
+#define int_equ(a, b)	((a) == (b))
+#define lng_equ(a, b)	((a) == (b))
+#define hge_equ(a, b)	((a) == (b))
+
 #define GRP_subscan_old_groups_tpe(TYPE)			\
 	GRP_subscan_old_groups(					\
 	/* INIT_0 */	const TYPE *w = (TYPE *) Tloc(b, 0);	\
 		    	TYPE pw = 0			,	\
 	/* INIT_1 */					,	\
-	/* COMP   */	w[p] == pw			,	\
+	/* COMP   */	TYPE##_equ(w[p], pw)		,	\
 	/* KEEP   */	pw = w[p]				\
 	)
 
@@ -249,9 +265,8 @@
  * The algorithm is simple.  We go through b and for each value we
  * follow the hash chain starting at the next element after that value
  * to find one that is equal to the value we're currently looking at.
- * If we found such a value (including the preexisting group if we're
- * refining), we add the value to the same group.  If we reach the end
- * of the chain, we create a new group.
+ * If we found such a value, we add the value to the same group.  If
+ * we reach the end of the chain, we create a new group.
  *
  * If b (the original, that is) is a view on another BAT, and this
  * other BAT has a hash, we use that.  The lo and hi values are the
@@ -263,96 +278,47 @@
 #define GRP_use_existing_hash_table(INIT_0,INIT_1,COMP)			\
 	do {								\
 		INIT_0;							\
-		if (grps) {						\
-			for (r = 0; r < cnt; r++) {			\
+		assert(grps == NULL);					\
+		for (r = 0; r < cnt; r++) {				\
+			if (cand) {					\
+				p = cand[r] - hseqb + lo;		\
+			} else {					\
+				p = start + r;				\
+			}						\
+			assert(p < end);				\
+			INIT_1;						\
+			/* this loop is similar, but not */		\
+			/* equal, to HASHloop: the difference */	\
+			/* is that we only consider BUNs */		\
+			/* smaller than the one we're looking */	\
+			/* up (p) */					\
+			for (hb = HASHgetlink(hs, p);			\
+			     hb != HASHnil(hs) && hb >= start;		\
+			     hb = HASHgetlink(hs, hb)) {		\
+				oid grp;				\
+				assert(hb < p);				\
 				if (cand) {				\
-					p = cand[r] - hseqb + lo;	\
+					q = r;				\
+					while (q != 0 && cand[--q] > hb) \
+						;			\
+					if (cand[q] - hseqb != hb - lo)	\
+						continue;		\
+					grp = ngrps[q];			\
 				} else {				\
-					p = start + r;			\
+					grp = ngrps[hb - lo];		\
 				}					\
-				assert(p < end);			\
-				INIT_1;					\
-				/* this loop is similar, but not */	\
-				/* equal, to HASHloop: the difference */ \
-				/* is that we only consider BUNs */	\
-				/* smaller than the one we're looking */ \
-				/* up (p), and that we also consider */	\
-				/* the input groups */			\
-				for (hb = HASHgetlink(hs, p);		\
-				     hb != HASHnil(hs) && hb >= start;	\
-				     hb = HASHgetlink(hs, hb)) {	\
-					oid grp;			\
-					assert(hb < p);			\
-					if (cand) {			\
-						q = r;			\
-						while (q != 0 && cand[--q] - hseqb > hb - lo) \
-							;		\
-						if (cand[q] - hseqb != hb - lo)	\
-							continue;	\
-						if (grps[q] != grps[r])	\
-							continue;	\
-						grp = ngrps[q];		\
-					} else {			\
-						if (grps[hb - lo] != grps[r]) \
-							continue;	\
-						grp = ngrps[hb - lo];	\
-					}				\
-					if (COMP) {			\
-						ngrps[r] = grp;		\
-						if (histo)		\
-							cnts[grp]++;	\
-						if (gn->tsorted &&	\
-						    grp != ngrp - 1)	\
-							gn->tsorted = 0; \
-						break;			\
-					}				\
-				}					\
-				if (hb == HASHnil(hs) || hb < lo) {	\
-					GRPnotfound();			\
+				if (COMP) {				\
+					ngrps[r] = grp;			\
+					if (histo)			\
+						cnts[grp]++;		\
+					if (gn->tsorted &&		\
+					    grp != ngrp - 1)		\
+						gn->tsorted = 0;	\
+					break;				\
 				}					\
 			}						\
-		} else {						\
-			for (r = 0; r < cnt; r++) {			\
-				if (cand) {				\
-					p = cand[r] - hseqb + lo;	\
-				} else {				\
-					p = start + r;			\
-				}					\
-				assert(p < end);			\
-				INIT_1;					\
-				/* this loop is similar, but not */	\
-				/* equal, to HASHloop: the difference */ \
-				/* is that we only consider BUNs */	\
-				/* smaller than the one we're looking */ \
-				/* up (p) */				\
-				for (hb = HASHgetlink(hs, p);		\
-				     hb != HASHnil(hs) && hb >= start;	\
-				     hb = HASHgetlink(hs, hb)) {	\
-					oid grp;			\
-					assert(hb < p);			\
-					if (cand) {			\
-						q = r;			\
-						while (q != 0 && cand[--q] > hb) \
-							;		\
-						if (cand[q] - hseqb != hb - lo)	\
-							continue;	\
-						grp = ngrps[q];		\
-					} else {			\
-						grp = ngrps[hb - lo];	\
-					}				\
-					if (COMP) {			\
-						ngrps[r] = grp;		\
-						if (histo)		\
-							cnts[grp]++;	\
-						if (gn->tsorted &&	\
-						    grp != ngrp - 1)	\
-							gn->tsorted = 0; \
-						break;			\
-					}				\
-				}					\
-				if (hb == HASHnil(hs) || hb < lo) {	\
-					GRPnotfound();			\
-				}					\
+			if (hb == HASHnil(hs) || hb < lo) {		\
+				GRPnotfound();				\
 			}						\
 		}							\
 	} while(0)
@@ -361,7 +327,7 @@
 	GRP_use_existing_hash_table(				\
 	/* INIT_0 */	const TYPE *w = (TYPE *) Tloc(b, 0),	\
 	/* INIT_1 */					,	\
-	/* COMP   */	w[p] == w[hb]				\
+	/* COMP   */	TYPE##_equ(w[p], w[hb])			\
 	)
 
 #define GRP_use_existing_hash_table_any()			\
@@ -490,7 +456,7 @@ pop(oid x)
 		if (grps) {						\
 			if (gc) {					\
 				GRP_create_partial_hash_table_core(INIT_1,HASH,COMP,assert(HASHgetlink(hs, hb) == HASHnil(hs) || HASHgetlink(hs, hb) < hb),GCGRPTST); \
-			} else {				\
+			} else {					\
 				GRP_create_partial_hash_table_core(INIT_1,HASH ^ (rev(grps[r]) >> bits),COMP,(void)0,GRPTST); \
 			}						\
 		} else {						\
@@ -503,7 +469,7 @@ pop(oid x)
 	/* INIT_0 */	const TYPE *w = (TYPE *) Tloc(b, 0),	\
 	/* INIT_1 */					,	\
 	/* HASH   */	hash_##TYPE(hs, &w[p])		,	\
-	/* COMP   */	w[p] == w[hb]				\
+	/* COMP   */	TYPE##_equ(w[p], w[hb])			\
 	)
 
 #define GRP_create_partial_hash_table_any()			\
@@ -529,7 +495,6 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 	BUN p, q, r;
 	const void *v, *pv;
 	BATiter bi;
-	char *ext = NULL;
 	Hash *hs = NULL;
 	BUN hb;
 	BUN maxgrps;
@@ -609,7 +574,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 				en->tnonil = 1;
 				en->tdense = 0;
 			} else {
-				en = BATdense(0, b->hseqbase, cnt);
+				en = BATdense(0, b->hseqbase + start, cnt);
 				if (en == NULL)
 					goto error;
 			}
@@ -634,9 +599,13 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 			prop = BATgetprop(g, GDK_MAX_VALUE);
 			if (prop)
 				maxgrp = prop->v.val.oval;
+			else if (BATordered(g) && BATordered_rev(g))
+				maxgrp = 0;
 		}
 		if (maxgrp == 0)
 			g = NULL; /* single group */
+		else
+			grps = (const oid *) Tloc(g, 0);
 	}
 	if (BATordered(b) && BATordered_rev(b)) {
 		/* all values are equal */
@@ -698,7 +667,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 			gn = COLcopy(g, g->ttype, 0, TRANSIENT);
 			if (gn == NULL)
 				goto error;
-			if (maxgrp != oid_nil) {
+			if (!is_oid_nil(maxgrp)) {
 				prop = BATgetprop(g, GDK_MAX_VALUE);
 				if (prop)
 					BATsetprop(gn, GDK_MAX_VALUE, TYPE_oid, &maxgrp);
@@ -728,7 +697,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 		goto error;
 	ngrps = (oid *) Tloc(gn, 0);
 	maxgrps = cnt / 10;
-	if (maxgrp != oid_nil && maxgrps < maxgrp)
+	if (!is_oid_nil(maxgrp) && maxgrps < maxgrp)
 		maxgrps += maxgrp;
 	if (e && maxgrps < BATcount(e))
 		maxgrps += BATcount(e);
@@ -753,9 +722,6 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 	}
 	ngrp = 0;
 	BATsetcount(gn, cnt);
-	/* don't bother looking at old groups if there is only one */
-	if (g && (!BATordered(g) || !BATordered_rev(g)))
-		grps = (const oid *) Tloc(g, 0);
 
 	hseqb = b->hseqbase;	/* abbreviation */
 
@@ -990,19 +956,23 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 			r++;
 		}
 		GDKfree(sgrps);
-	} else if (BATcheckhash(b) ||
-		   (b->batPersistence == PERSISTENT &&
-		    BAThash(b, 0) == GDK_SUCCEED)
+	} else if (g == NULL &&
+		   (BATcheckhash(b) ||
+		    (b->batPersistence == PERSISTENT &&
+		     BAThash(b, 0) == GDK_SUCCEED)
 #ifndef DISABLE_PARENT_HASH
-		   || ((parent = VIEWtparent(b)) != 0 &&
-		       BATcheckhash(BBPdescriptor(parent)))
+		    || ((parent = VIEWtparent(b)) != 0 &&
+			BATcheckhash(BBPdescriptor(parent)))
 #endif
-		) {
+		)) {
 		BUN lo;
-
 		/* we already have a hash table on b, or b is
 		 * persistent and we could create a hash table, or b
-		 * is a view on a bat that already has a hash table */
+		 * is a view on a bat that already has a hash table;
+		 * but don't do this if we're checking for subgroups
+		 * since we may have to go through long lists of
+		 * duplicates in the hash table to find an old
+		 * group */
 		ALGODEBUG fprintf(stderr, "#BATgroup(b=%s#" BUNFMT "[%s],"
 				  "s=%s#" BUNFMT ","
 				  "g=%s#" BUNFMT ","
@@ -1065,8 +1035,6 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 	} else {
 		bit gc = g && (BATordered(g) || BATordered_rev(g));
 		const char *nme;
-		size_t nmelen;
-		Heap *hp = NULL;
 		BUN prb;
 		int bits;
 		BUN mask;
@@ -1091,28 +1059,18 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 				  h ? BATgetId(h) : "NULL", h ? BATcount(h) : 0,
 				  subsorted, gc ? " (g clustered)" : "");
 		nme = GDKinmemory() ? ":inmemory" : BBP_physical(b->batCacheid);
-		nmelen = strlen(nme);
 		mask = MAX(HASHmask(cnt), 1 << 16);
 		/* mask is a power of two, so pop(mask - 1) tells us
 		 * which power of two */
 		bits = 8 * SIZEOF_OID - pop(mask - 1);
-		if ((hp = GDKzalloc(sizeof(Heap))) == NULL ||
-		    (hp->farmid = BBPselectfarm(TRANSIENT, b->ttype, hashheap)) < 0 ||
-		    (hp->filename = GDKmalloc(nmelen + 30)) == NULL ||
-		    snprintf(hp->filename, nmelen + 30,
-			     "%s.hash" SZFMT, nme, MT_getpid()) < 0 ||
-		    (ext = GDKstrdup(hp->filename + nmelen + 1)) == NULL ||
-		    (hs = HASHnew(hp, b->ttype, BUNlast(b),
-				  mask, BUN_NONE)) == NULL) {
-			if (hp) {
-				if (hp->filename)
-					GDKfree(hp->filename);
-				GDKfree(hp);
-			}
-			if (ext)
-				GDKfree(ext);
-			hp = NULL;
-			ext = NULL;
+		if ((hs = GDKzalloc(sizeof(Hash))) == NULL ||
+		    (hs->heap.farmid = BBPselectfarm(TRANSIENT, b->ttype, hashheap)) < 0 ||
+		    snprintf(hs->heap.filename, sizeof(hs->heap.filename),
+			     "%s.hash%d", nme, THRgettid()) < 0 ||
+		    HASHnew(hs, b->ttype, BUNlast(b),
+			    mask, BUN_NONE) != GDK_SUCCEED) {
+			GDKfree(hs);
+			hs = NULL;
 			GDKerror("BATgroup: cannot allocate hash table\n");
 			goto error;
 		}
@@ -1120,7 +1078,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 
 		switch (t) {
 		case TYPE_bte:
-			if (grps && maxgrp != oid_nil
+			if (grps && !is_oid_nil(maxgrp)
 #if SIZEOF_OID == SIZEOF_LNG
 			    && maxgrp < ((oid) 1 << (SIZEOF_LNG * 8 - 8))
 #endif
@@ -1137,7 +1095,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 				GRP_create_partial_hash_table_tpe(bte);
 			break;
 		case TYPE_sht:
-			if (grps && maxgrp != oid_nil
+			if (grps && !is_oid_nil(maxgrp)
 #if SIZEOF_OID == SIZEOF_LNG
 			    && maxgrp < ((oid) 1 << (SIZEOF_LNG * 8 - 16))
 #endif
@@ -1154,7 +1112,7 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 				GRP_create_partial_hash_table_tpe(sht);
 			break;
 		case TYPE_int:
-			if (grps && maxgrp != oid_nil
+			if (grps && !is_oid_nil(maxgrp)
 #if SIZEOF_OID == SIZEOF_LNG
 			    && maxgrp < ((oid) 1 << (SIZEOF_LNG * 8 - 32))
 #endif
@@ -1200,10 +1158,8 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 			GRP_create_partial_hash_table_any();
 		}
 
-		HEAPfree(hp, 1);
-		GDKfree(hp);
+		HEAPfree(&hs->heap, 1);
 		GDKfree(hs);
-		GDKfree(ext);
 	}
 	if (extents) {
 		BATsetcount(en, (BUN) ngrp);
@@ -1238,6 +1194,10 @@ BATgroup_internal(BAT **groups, BAT **extents, BAT **histo,
 	*groups = gn;
 	return GDK_SUCCEED;
   error:
+	if (hs != NULL && hs != b->thash) {
+		HEAPfree(&hs->heap, 1);
+		GDKfree(hs);
+	}
 	if (gn)
 		BBPunfix(gn->batCacheid);
 	if (en)
