@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -81,6 +81,16 @@
 #define printf(fmt,...) ((void) 0)
 #endif
 
+#ifdef WIN32
+#define getfilepos _ftelli64
+#else
+#ifdef HAVE_FSEEKO
+#define getfilepos ftello
+#else
+#define getfilepos ftell
+#endif
+#endif
+
 static char *log_commands[] = {
 	NULL,
 	"LOG_START",
@@ -203,7 +213,7 @@ log_read_string(logger *l)
 
 	if ((nr = mnstr_read(l->log, buf, 1, len)) != (ssize_t) len) {
 		buf[len - 1] = 0;
-		fprintf(stderr, "!ERROR: log_read_string: couldn't read name (%s) " SSZFMT "\n", buf, nr);
+		fprintf(stderr, "!ERROR: log_read_string: couldn't read name (%s)\n", buf);
 		GDKfree(buf);
 		return NULL;
 	}
@@ -310,6 +320,45 @@ log_read_seq(logger *lg, logformat *l)
 	return LOG_OK;
 }
 
+#ifdef GDKLIBRARY_NIL_NAN
+static void *
+fltRead(void *dst, stream *s, size_t cnt)
+{
+	flt *ptr;
+	size_t i;
+
+	if ((ptr = BATatoms[TYPE_flt].atomRead(dst, s, cnt)) == NULL)
+		return NULL;
+	for (i = 0; i < cnt; i++)
+		if (ptr[i] == GDK_flt_min)
+			ptr[i] = flt_nil;
+	return ptr;
+}
+
+static void *
+dblRead(void *dst, stream *s, size_t cnt)
+{
+	dbl *ptr;
+	size_t i;
+
+	if ((ptr = BATatoms[TYPE_dbl].atomRead(dst, s, cnt)) == NULL)
+		return NULL;
+	for (i = 0; i < cnt; i++)
+		if (ptr[i] == GDK_dbl_min)
+			ptr[i] = dbl_nil;
+	return ptr;
+}
+
+static void *
+mbrRead(void *dst, stream *s, size_t cnt)
+{
+	/* an MBR consists of 4 flt values; here we don't care about
+	 * anything else, we just need to convert the old NIL to NaN
+	 * for all those values */
+	return fltRead(dst, s, cnt * 4);
+}
+#endif
+
 static log_return
 log_read_updates(logger *lg, trans *tr, logformat *l, char *name)
 {
@@ -324,7 +373,7 @@ log_read_updates(logger *lg, trans *tr, logformat *l, char *name)
 	if (b) {
 		ht = TYPE_void;
 		tt = b->ttype;
-		if (tt == TYPE_void && b->tseqbase != oid_nil)
+		if (tt == TYPE_void && !is_oid_nil(b->tseqbase))
 			tseq = 1;
 	} else {		/* search trans action for create statement */
 		int i;
@@ -354,6 +403,16 @@ log_read_updates(logger *lg, trans *tr, logformat *l, char *name)
 
 		if (ATOMstorage(tt) < TYPE_str)
 			tv = lg->buf;
+#ifdef GDKLIBRARY_NIL_NAN
+		if (lg->convert_nil_nan) {
+			if (tt == TYPE_flt)
+				rt = fltRead;
+			else if (tt == TYPE_dbl)
+				rt = dblRead;
+			else if (tt > TYPE_str && strcmp(BATatoms[tt].name, "mbr") == 0)
+				rt = mbrRead;
+		}
+#endif
 
 		assert(l->nr <= (lng) BUN_MAX);
 		if (l->flag == LOG_UPDATE) {
@@ -485,9 +544,9 @@ la_bat_updates(logger *lg, logaction *la)
 				/* if value doesn't exist, insert it;
 				 * if b void headed, maintain that by
 				 * inserting nils */
-				if (b->batCount == 0 && h != oid_nil)
+				if (b->batCount == 0 && !is_oid_nil(h))
 					b->hseqbase = h;
-				if (b->hseqbase != oid_nil && h != oid_nil) {
+				if (!is_oid_nil(b->hseqbase) && !is_oid_nil(h)) {
 					const void *tv = ATOMnilptr(b->ttype);
 
 					while (b->hseqbase + b->batCount < h) {
@@ -576,25 +635,11 @@ log_read_create(logger *lg, trans *tr, char *name)
 		return LOG_ERR;
 	}
 	*ta++ = 0;		/* skip over , */
-	if (strcmp(ha, "wrd") == 0) {
-#if SIZEOF_SSIZE_T == SIZEOF_INT
-		ha = "int";
-#else
-		ha = "lng";
-#endif
-	}
 	if (strcmp(ha, "vid") == 0) {
 		ht = -1;
 	} else {
 		ht = ATOMindex(ha);
 	}
-	if (strcmp(ta, "wrd") == 0) {
-#if SIZEOF_SSIZE_T == SIZEOF_INT
-		ta = "int";
-#else
-		ta = "lng";
-#endif
-	}
 	if (strcmp(ta, "vid") == 0) {
 		tt = -1;
 	} else {
@@ -986,7 +1031,8 @@ logger_readlog(logger *lg, char *filename)
 			lng fpos;
 			t0 = t1;
 			/* not more than once every 10 seconds */
-			if (mnstr_fgetpos(lg->log, &fpos) == 0) {
+			fpos = (lng) getfilepos(getFile(lg->log));
+			if (fpos >= 0) {
 				printf("# still reading write-ahead log \"%s\" (%d%% done)\n", filename, (int) ((fpos * 100 + 50) / sb.st_size));
 				fflush(stdout);
 			}
@@ -1123,7 +1169,7 @@ logger_readlogs(logger *lg, FILE *fp, char *filename)
 	}
 
 	while (fgets(id, sizeof(id), fp) != NULL) {
-		char log_filename[PATHLENGTH];
+		char log_filename[FILENAME_MAX];
 		lng lid = strtoll(id, NULL, 10);
 
 		if (lg->debug & 1) {
@@ -1262,7 +1308,7 @@ logger_switch_bat(BAT *old, BAT *new, const char *fn, const char *name)
 		GDKerror("Logger_new: cannot convert old %s to transient", name);
 		return GDK_FAIL;
 	}
-	snprintf(bak, sizeof(bak), "tmp_%o", old->batCacheid);
+	snprintf(bak, sizeof(bak), "tmp_%o", (unsigned) old->batCacheid);
 	if (BBPrename(old->batCacheid, bak) != 0) {
 		return GDK_FAIL;
 	}
@@ -1387,8 +1433,8 @@ logger_set_logdir_path(char *filename, const char *fn,
 	int role = PERSISTENT; /* default role is persistent, i.e. the default dbfarm */
 
 	if (MT_path_absolute(logdir)) {
-		char logdir_parent_path[PATHLENGTH] = "";
-		char logdir_name[PATHLENGTH] = "";
+		char logdir_parent_path[FILENAME_MAX-3] = "";
+		char logdir_name[FILENAME_MAX-3] = "";
 
 		/* split the logdir string into absolute parent dir
 		 * path and (relative) log dir name */
@@ -1396,7 +1442,7 @@ logger_set_logdir_path(char *filename, const char *fn,
 			/* set the new relative logdir location
 			 * including the logger function name
 			 * subdir */
-			snprintf(filename, PATHLENGTH, "%s%c%s%c",
+			snprintf(filename, FILENAME_MAX, "%s%c%s%c",
 				 logdir_name, DIR_SEP, fn, DIR_SEP);
 
 			/* add a new dbfarm for the logger directory
@@ -1414,7 +1460,7 @@ logger_set_logdir_path(char *filename, const char *fn,
 		}
 	} else {
 		/* just concat the logdir and fn with appropriate separators */
-		snprintf(filename, PATHLENGTH, "%s%c%s%c",
+		snprintf(filename, FILENAME_MAX, "%s%c%s%c",
 			 logdir, DIR_SEP, fn, DIR_SEP);
 	}
 
@@ -1426,18 +1472,18 @@ logger_set_logdir_path(char *filename, const char *fn,
  * unless running in read-only mode
  * Load data and persist it in the BATs */
 static gdk_return
-logger_load(int debug, const char *fn, char filename[PATHLENGTH], logger *lg)
+logger_load(int debug, const char *fn, char filename[FILENAME_MAX], logger *lg)
 {
 	int id = LOG_SID;
 	FILE *fp;
-	char bak[PATHLENGTH];
+	char bak[FILENAME_MAX];
 	str filenamestr = NULL;
 	log_bid snapshots_bid = 0;
 	bat catalog_bid, catalog_nme, dcatalog, bid;
 	int farmid = BBPselectfarm(lg->dbfarm_role, 0, offheap);
 
 	filenamestr = GDKfilepath(farmid, lg->dir, LOGFILE, NULL);
-	snprintf(filename, PATHLENGTH, "%s", filenamestr);
+	snprintf(filename, FILENAME_MAX, "%s", filenamestr);
 	snprintf(bak, sizeof(bak), "%s.bak", filename);
 	GDKfree(filenamestr);
 
@@ -1531,7 +1577,7 @@ logger_load(int debug, const char *fn, char filename[PATHLENGTH], logger *lg)
 		lg->id ++;
 		if (fprintf(fp, "%06d\n\n" LLFMT "\n", lg->version, lg->id) < 0) {
 			fclose(fp);
-			unlink(filename);
+			remove(filename);
 			GDKerror("logger_load: writing log file %s failed",
 				 filename);
 			goto error;
@@ -1547,7 +1593,7 @@ logger_load(int debug, const char *fn, char filename[PATHLENGTH], logger *lg)
 #endif
 			    ) ||
 		    fclose(fp) < 0) {
-			unlink(filename);
+			remove(filename);
 			GDKerror("logger_load: closing log file %s failed",
 				 filename);
 			goto error;
@@ -1560,7 +1606,7 @@ logger_load(int debug, const char *fn, char filename[PATHLENGTH], logger *lg)
 
 		if (bm_subcommit(lg, lg->catalog_bid, lg->catalog_nme, lg->catalog_bid, lg->catalog_nme, lg->dcatalog, NULL, lg->debug) != GDK_SUCCEED) {
 			/* cannot commit catalog, so remove log */
-			unlink(filename);
+			remove(filename);
 			BBPrelease(lg->catalog_bid->batCacheid);
 			BBPrelease(lg->catalog_nme->batCacheid);
 			BBPrelease(lg->dcatalog->batCacheid);
@@ -1800,15 +1846,124 @@ logger_load(int debug, const char *fn, char filename[PATHLENGTH], logger *lg)
 	}
 
 	if (fp != NULL) {
+#ifdef GDKLIBRARY_NIL_NAN
+		char cvfile[FILENAME_MAX];
+#endif
+
 		if (check_version(lg, fp) != GDK_SUCCEED) {
 			goto error;
 		}
 
+#ifdef GDKLIBRARY_NIL_NAN
+		/* When a file *_nil-nan-convert exists in the
+		 * database, it was left there by the BBP
+		 * initialization code when it did a conversion of old
+		 * style NILs to NaNs.  If the file exists, we first
+		 * create a file called convert-nil-nan in the log
+		 * directory and we write the current log ID into that
+		 * file.  After this file is created, we delete the
+		 * *_nil-nan-convert file in the database.  We then
+		 * know that while reading the logs, we have to
+		 * convert old style NILs to NaNs (this is indicated
+		 * by setting the convert_nil_nan flag).  When we're
+		 * done reading the logs, we remove the file and
+		 * reset the flag.  If we get interrupted before we
+		 * have written this file, the file in the database
+		 * will still exist, so the next time we're started,
+		 * BBPinit will not convert NILs (that was done before
+		 * we got interrupted), but we will still know to
+		 * convert the NILs ourselves.  If we get interrupted
+		 * after we have deleted the file from the database,
+		 * we check whether the file convert-nil-nan exists
+		 * and if it contains the expected ID.  If it does, we
+		 * again know that we have to convert.  If the ID is
+		 * not what we expect, the conversion was apparently
+		 * done already, and so we can delete the file. */
+
+		/* Do not do conversion if logger is shared/read-only */
+		if (!lg->shared) {
+			FILE *fp1;
+			fpos_t off;
+			int curid;
+
+			snprintf(cvfile, sizeof(cvfile), "%sconvert-nil-nan",
+				 lg->dir);
+			snprintf(bak, sizeof(bak), "%s_nil-nan-convert", fn);
+			/* read the current log id without disturbing
+			 * the file pointer */
+			if (fgetpos(fp, &off) != 0)
+				goto error; /* should never happen */
+			if (fscanf(fp, "%d", &curid) != 1)
+				curid = -1; /* shouldn't happen? */
+			if (fsetpos(fp, &off) != 0)
+				goto error; /* should never happen */
+
+			if ((fp1 = GDKfileopen(0, NULL, bak, NULL, "r")) != NULL) {
+				/* file indicating that we need to do
+				 * a NIL to NaN conversion exists;
+				 * record the fact in case we get
+				 * interrupted, and set the flag so
+				 * that we actually do what's asked */
+				fclose(fp1);
+				/* first create a versioned file using
+				 * the current log id */
+				if ((fp1 = GDKfileopen(farmid, NULL, cvfile, NULL, "w")) == NULL ||
+				    fprintf(fp1, "%d\n", curid) < 2 ||
+				    fflush(fp1) != 0 || /* make sure it's save on disk */
+#if defined(_MSC_VER)
+				    _commit(_fileno(fp1)) < 0 ||
+#elif defined(HAVE_FDATASYNC)
+				    fdatasync(fileno(fp1)) < 0 ||
+#elif defined(HAVE_FSYNC)
+				    fsync(fileno(fp1)) < 0 ||
+#endif
+				    fclose(fp1) != 0) {
+					GDKerror("logger_load: failed to write %s\n", cvfile);
+					goto error;
+				}
+				/* then remove the unversioned file
+				 * that gdk_bbp created (in this
+				 * order!) */
+				if (GDKunlink(0, NULL, bak, NULL) != GDK_SUCCEED) {
+					GDKerror("logger_load: failed to unlink %s\n", bak);
+					goto error;
+				}
+				/* set the flag that we need to convert */
+				lg->convert_nil_nan = 1;
+			} else if ((fp1 = GDKfileopen(farmid, NULL, cvfile, NULL, "r")) != NULL) {
+				/* the versioned conversion file
+				 * exists: check version */
+				int newid;
+
+				if (fscanf(fp1, "%d", &newid) == 1 &&
+				    newid == curid) {
+					/* versions match, we need to
+					 * convert */
+					lg->convert_nil_nan = 1;
+				}
+				fclose(fp1);
+				if (!lg->convert_nil_nan) {
+					/* no conversion, so we can
+					 * remove the versioned
+					 * file */
+					GDKunlink(0, NULL, cvfile, NULL);
+				}
+			}
+		}
+#endif
 		if (logger_readlogs(lg, fp, filename) != GDK_SUCCEED) {
 			goto error;
 		}
 		fclose(fp);
 		fp = NULL;
+#ifdef GDKLIBRARY_NIL_NAN
+		if (lg->convert_nil_nan && !lg->shared) {
+			/* we converted, remove versioned file and
+			 * reset conversion flag */
+			GDKunlink(0, NULL, cvfile, NULL);
+			lg->convert_nil_nan = 0;
+		}
+#endif
 		if (lg->postfuncp && (*lg->postfuncp)(lg) != GDK_SUCCEED)
 			goto error;
 
@@ -1844,8 +1999,8 @@ static logger *
 logger_new(int debug, const char *fn, const char *logdir, int version, preversionfix_fptr prefuncp, postversionfix_fptr postfuncp, int shared, const char *local_logdir)
 {
 	logger *lg = (struct logger *) GDKmalloc(sizeof(struct logger));
-	char filename[PATHLENGTH];
-	char shared_log_filename[PATHLENGTH];
+	char filename[FILENAME_MAX];
+	char shared_log_filename[FILENAME_MAX];
 
 	if (lg == NULL) {
 		fprintf(stderr, "!ERROR: logger_new: allocating logger structure failed\n");
@@ -1861,6 +2016,9 @@ logger_new(int debug, const char *fn, const char *logdir, int version, preversio
 	lg->id = 1;
 
 	lg->tid = 0;
+#ifdef GDKLIBRARY_NIL_NAN
+	lg->convert_nil_nan = 0;
+#endif
 
 	lg->dbfarm_role = logger_set_logdir_path(filename, fn, logdir, shared);;
 	lg->fn = GDKstrdup(fn);
@@ -1948,7 +2106,7 @@ logger_new(int debug, const char *fn, const char *logdir, int version, preversio
 gdk_return
 logger_reload(logger *lg)
 {
-	char filename[PATHLENGTH];
+	char filename[FILENAME_MAX];
 
 	snprintf(filename, sizeof(filename), "%s", lg->dir);
 	if (lg->debug & 1) {
@@ -2042,7 +2200,7 @@ gdk_return
 logger_exit(logger *lg)
 {
 	FILE *fp;
-	char filename[PATHLENGTH];
+	char filename[FILENAME_MAX];
 	int farmid = BBPselectfarm(lg->dbfarm_role, 0, offheap);
 
 	logger_close(lg);
@@ -2054,7 +2212,7 @@ logger_exit(logger *lg)
 
 	snprintf(filename, sizeof(filename), "%s%s", lg->dir, LOGFILE);
 	if ((fp = GDKfileopen(farmid, NULL, filename, NULL, "w")) != NULL) {
-		char ext[PATHLENGTH];
+		char ext[FILENAME_MAX];
 
 		if (fprintf(fp, "%06d\n\n", lg->version) < 0) {
 			(void) fclose(fp);
@@ -2139,9 +2297,9 @@ logger_unlink(int farmid, const char *dir, const char *nme, const char *ext)
 	path = GDKfilepath(farmid, dir, nme, ext);
 	if (path == NULL)
 		return GDK_FAIL;
-	u = unlink(path);
+	u = remove(path);
 	GDKfree(path);
-	return u < 0 ? GDK_FAIL : GDK_SUCCEED;
+	return u != 0 ? GDK_FAIL : GDK_SUCCEED;
 }
 
 static void
@@ -2251,7 +2409,7 @@ logger_changes(logger *lg)
 lng
 logger_read_last_transaction_id(logger *lg, char *dir, char *logger_file, int role)
 {
-	char filename[PATHLENGTH];
+	char filename[FILENAME_MAX];
 	FILE *fp;
 	char id[BUFSIZ];
 	lng lid = GDK_FAIL;
@@ -2564,7 +2722,8 @@ pre_allocate(logger *lg)
 	// FIXME: this causes serious issues on Windows at least with MinGW
 #ifndef WIN32
 	lng p;
-	if (mnstr_fgetpos(lg->log, &p) != 0)
+	p = (lng) getfilepos(getFile(lg->log));
+	if (p == -1)
 		return GDK_FAIL;
 	if (p + DBLKSZ > lg->end) {
 		p &= ~(DBLKSZ - 1);
@@ -2776,7 +2935,7 @@ bm_commit(logger *lg)
 			return GDK_FAIL;
 		}
 
-		assert(lb->batRestricted > BAT_WRITE);
+		assert(lb->batRestricted != BAT_WRITE);
 		logbat_destroy(lb);
 
 		if (lg->debug & 1)
