@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -52,14 +52,14 @@ BATidxsync(void *arg)
 
 /* return TRUE if we have a orderidx on the tail, even if we need to read
  * one from disk */
-int
+bool
 BATcheckorderidx(BAT *b)
 {
-	int ret;
+	bool ret;
 	lng t = 0;
 
 	if (b == NULL)
-		return 0;
+		return false;
 	assert(b->batCacheid > 0);
 	ALGODEBUG t = GDKusec();
 	MT_lock_set(&GDKhashLock(b->batCacheid));
@@ -70,9 +70,8 @@ BATcheckorderidx(BAT *b)
 
 		b->torderidx = NULL;
 		if ((hp = GDKzalloc(sizeof(*hp))) != NULL &&
-		    (hp->farmid = BBPselectfarm(b->batRole, b->ttype, orderidxheap)) >= 0 &&
-		    (hp->filename = GDKmalloc(strlen(nme) + 11)) != NULL) {
-			sprintf(hp->filename, "%s.torderidx", nme);
+		    (hp->farmid = BBPselectfarm(b->batRole, b->ttype, orderidxheap)) >= 0) {
+			snprintf(hp->filename, sizeof(hp->filename), "%s.torderidx", nme);
 
 			/* check whether a persisted orderidx can be found */
 			if ((fd = GDKfdlocate(hp->farmid, nme, "rb+", "torderidx")) >= 0) {
@@ -94,13 +93,12 @@ BATcheckorderidx(BAT *b)
 					b->torderidx = hp;
 					ALGODEBUG fprintf(stderr, "#BATcheckorderidx: reusing persisted orderidx %d\n", b->batCacheid);
 					MT_lock_unset(&GDKhashLock(b->batCacheid));
-					return 1;
+					return true;
 				}
 				close(fd);
 				/* unlink unusable file */
 				GDKunlink(hp->farmid, BATDIR, nme, "torderidx");
 			}
-			GDKfree(hp->filename);
 		}
 		GDKfree(hp);
 		GDKclrerr();	/* we're not currently interested in errors */
@@ -116,7 +114,6 @@ Heap *
 createOIDXheap(BAT *b, int stable)
 {
 	Heap *m;
-	size_t nmelen;
 	oid *restrict mv;
 	const char *nme;
 
@@ -124,14 +121,10 @@ createOIDXheap(BAT *b, int stable)
 	if (GDKinmemory()) {
 		nme = ":inmemory";
 	}
-	nmelen = strlen(nme) + 12;
 	if ((m = GDKzalloc(sizeof(Heap))) == NULL ||
 	    (m->farmid = BBPselectfarm(b->batRole, b->ttype, orderidxheap)) < 0 ||
-	    (m->filename = GDKmalloc(nmelen)) == NULL ||
-	    snprintf(m->filename, nmelen, "%s.torderidx", nme) < 0 ||
+	    snprintf(m->filename, sizeof(m->filename), "%s.torderidx", nme) < 0 ||
 	    HEAPalloc(m, BATcount(b) + ORDERIDXOFF, SIZEOF_OID) != GDK_SUCCEED) {
-		if (m)
-			GDKfree(m->filename);
 		GDKfree(m);
 		return NULL;
 	}
@@ -317,13 +310,12 @@ GDKmergeidx(BAT *b, BAT**a, int n_ar)
 {
 	Heap *m;
 	int i;
-	size_t nmelen;
 	oid *restrict mv;
 	const char *nme = BBP_physical(b->batCacheid);
 
 	if (BATcheckorderidx(b))
 		return GDK_SUCCEED;
-	switch (ATOMstorage(b->ttype)) {
+	switch (ATOMbasetype(b->ttype)) {
 	case TYPE_bte:
 	case TYPE_sht:
 	case TYPE_int:
@@ -344,14 +336,10 @@ GDKmergeidx(BAT *b, BAT**a, int n_ar)
 		MT_lock_unset(&GDKhashLock(b->batCacheid));
 		return GDK_SUCCEED;
 	}
-	nmelen = strlen(nme) + 12;
 	if ((m = GDKzalloc(sizeof(Heap))) == NULL ||
 	    (m->farmid = BBPselectfarm(b->batRole, b->ttype, orderidxheap)) < 0 ||
-	    (m->filename = GDKmalloc(nmelen)) == NULL ||
-	    snprintf(m->filename, nmelen, "%s.torderidx", nme) < 0 ||
+	    snprintf(m->filename, sizeof(m->filename), "%s.torderidx", nme) < 0 ||
 	    HEAPalloc(m, BATcount(b) + ORDERIDXOFF, SIZEOF_OID) != GDK_SUCCEED) {
-		if (m)
-			GDKfree(m->filename);
 		GDKfree(m);
 		MT_lock_unset(&GDKhashLock(b->batCacheid));
 		return GDK_FAIL;
@@ -393,7 +381,7 @@ GDKmergeidx(BAT *b, BAT**a, int n_ar)
 		q0 = p0 + BATcount(a[0]);
 		q1 = p1 + BATcount(a[1]);
 
-		switch (ATOMstorage(b->ttype)) {
+		switch (ATOMbasetype(b->ttype)) {
 		case TYPE_bte: BINARY_MERGE(bte); break;
 		case TYPE_sht: BINARY_MERGE(sht); break;
 		case TYPE_int: BINARY_MERGE(int); break;
@@ -435,7 +423,7 @@ GDKmergeidx(BAT *b, BAT**a, int n_ar)
 			q[i] = p[i] + BATcount(a[i]);
 		}
 
-		switch (ATOMstorage(b->ttype)) {
+		switch (ATOMbasetype(b->ttype)) {
 		case TYPE_bte: NWAY_MERGE(bte); break;
 		case TYPE_sht: NWAY_MERGE(sht); break;
 		case TYPE_int: NWAY_MERGE(int); break;
