@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -18,7 +18,7 @@
 
 #define project_loop(TYPE)						\
 static gdk_return							\
-project_##TYPE(BAT *bn, BAT *l, BAT *r, int nilcheck)			\
+project_##TYPE(BAT *bn, BAT *l, BAT *r, bool nilcheck)			\
 {									\
 	oid lo, hi;							\
 	const TYPE *restrict rt;					\
@@ -37,13 +37,13 @@ project_##TYPE(BAT *bn, BAT *l, BAT *r, int nilcheck)			\
 	if (nilcheck) {							\
 		for (; lo < hi; lo++) {					\
 			if (o[lo] < rseq || o[lo] >= rend) {		\
-				if (o[lo] == oid_nil) {			\
+				if (is_oid_nil(o[lo])) {		\
 					bt[lo] = TYPE##_nil;		\
-					bn->tnonil = 0;			\
-					bn->tnil = 1;			\
-					bn->tsorted = 0;		\
-					bn->trevsorted = 0;		\
-					bn->tkey = 0;			\
+					bn->tnonil = false;		\
+					bn->tnil = true;		\
+					bn->tsorted = false;		\
+					bn->trevsorted = false;		\
+					bn->tkey = false;		\
 					lo++;				\
 					break;				\
 				} else {				\
@@ -53,9 +53,9 @@ project_##TYPE(BAT *bn, BAT *l, BAT *r, int nilcheck)			\
 			} else {					\
 				v = rt[o[lo] - rseq];			\
 				bt[lo] = v;				\
-				if (v == TYPE##_nil && bn->tnonil) {	\
-					bn->tnonil = 0;			\
-					bn->tnil = 1;			\
+				if (is_##TYPE##_nil(v) && bn->tnonil) {	\
+					bn->tnonil = false;		\
+					bn->tnil = true;		\
 					lo++;				\
 					break;				\
 				}					\
@@ -64,13 +64,13 @@ project_##TYPE(BAT *bn, BAT *l, BAT *r, int nilcheck)			\
 	}								\
 	for (; lo < hi; lo++) {						\
 		if (o[lo] < rseq || o[lo] >= rend) {			\
-			if (o[lo] == oid_nil) {				\
+			if (is_oid_nil(o[lo])) {			\
 				bt[lo] = TYPE##_nil;			\
-				bn->tnonil = 0;				\
-				bn->tnil = 1;				\
-				bn->tsorted = 0;			\
-				bn->trevsorted = 0;			\
-				bn->tkey = 0;				\
+				bn->tnonil = false;			\
+				bn->tnil = true;			\
+				bn->tsorted = false;			\
+				bn->trevsorted = false;			\
+				bn->tkey = false;			\
 			} else {					\
 				GDKerror("BATproject: does not match always\n"); \
 				return GDK_FAIL;			\
@@ -105,25 +105,25 @@ project_void(BAT *bn, BAT *l, BAT *r)
 	const oid *o;
 	oid rseq, rend;
 
-	assert(r->tseqbase != oid_nil);
+	assert(!is_oid_nil(r->tseqbase));
 	o = (const oid *) Tloc(l, 0);
 	bt = (oid *) Tloc(bn, 0);
 	bn->tsorted = l->tsorted;
 	bn->trevsorted = l->trevsorted;
-	bn->tkey = l->tkey & 1;
-	bn->tnonil = 1;
-	bn->tnil = 0;
+	bn->tkey = l->tkey;
+	bn->tnonil = true;
+	bn->tnil = false;
 	rseq = r->hseqbase;
 	rend = rseq + BATcount(r);
 	for (lo = 0, hi = lo + BATcount(l); lo < hi; lo++) {
 		if (o[lo] < rseq || o[lo] >= rend) {
-			if (o[lo] == oid_nil) {
+			if (is_oid_nil(o[lo])) {
 				bt[lo] = oid_nil;
-				bn->tnonil = 0;
-				bn->tnil = 1;
-				bn->tsorted = 0;
-				bn->trevsorted = 0;
-				bn->tkey = 0;
+				bn->tnonil = false;
+				bn->tnil = true;
+				bn->tsorted = false;
+				bn->trevsorted = false;
+				bn->tkey = false;
 			} else {
 				GDKerror("BATproject: does not match always\n");
 				return GDK_FAIL;
@@ -138,7 +138,7 @@ project_void(BAT *bn, BAT *l, BAT *r)
 }
 
 static gdk_return
-project_any(BAT *bn, BAT *l, BAT *r, int nilcheck)
+project_any(BAT *bn, BAT *l, BAT *r, bool nilcheck)
 {
 	BUN n;
 	oid lo, hi;
@@ -156,13 +156,13 @@ project_any(BAT *bn, BAT *l, BAT *r, int nilcheck)
 	rend = rseq + BATcount(r);
 	for (lo = 0, hi = lo + BATcount(l); lo < hi; lo++, n++) {
 		if (o[lo] < rseq || o[lo] >= rend) {
-			if (o[lo] == oid_nil) {
+			if (is_oid_nil(o[lo])) {
 				tfastins_nocheck(bn, n, nil, Tsize(bn));
-				bn->tnonil = 0;
-				bn->tnil = 1;
-				bn->tsorted = 0;
-				bn->trevsorted = 0;
-				bn->tkey = 0;
+				bn->tnonil = false;
+				bn->tnil = true;
+				bn->tsorted = false;
+				bn->trevsorted = false;
+				bn->tkey = false;
 			} else {
 				GDKerror("BATproject: does not match always\n");
 				goto bunins_failed;
@@ -171,8 +171,8 @@ project_any(BAT *bn, BAT *l, BAT *r, int nilcheck)
 			v = BUNtail(ri, o[lo] - rseq);
 			tfastins_nocheck(bn, n, v, Tsize(bn));
 			if (nilcheck && bn->tnonil && cmp(v, nil) == 0) {
-				bn->tnonil = 0;
-				bn->tnil = 1;
+				bn->tnonil = false;
+				bn->tnil = true;
 			}
 		}
 	}
@@ -189,7 +189,8 @@ BATproject(BAT *l, BAT *r)
 	BAT *bn;
 	oid lo, hi;
 	gdk_return res;
-	int tpe = ATOMtype(r->ttype), nilcheck = 1, stringtrick = 0;
+	int tpe = ATOMtype(r->ttype);
+	bool nilcheck = true, stringtrick = false;
 	BUN lcount = BATcount(l), rcount = BATcount(r);
 	lng t0 = 0;
 
@@ -226,9 +227,10 @@ BATproject(BAT *l, BAT *r)
 				  bn->tkey ? "-key" : "");
 		return bn;
 	}
+	/* if l has type void, it is either empty or not dense (i.e. nil) */
 	if (l->ttype == TYPE_void || BATcount(l) == 0 ||
-	    (r->ttype == TYPE_void && r->tseqbase == oid_nil)) {
-		/* trivial: all values are nil */
+	    (r->ttype == TYPE_void && is_oid_nil(r->tseqbase))) {
+		/* trivial: all values are nil (includes no entries at all) */
 		const void *nil = ATOMnilptr(r->ttype);
 
 		bn = BATconstant(l->hseqbase, r->ttype == TYPE_oid ? TYPE_void : r->ttype,
@@ -237,7 +239,7 @@ BATproject(BAT *l, BAT *r)
 			return NULL;
 		if (ATOMtype(bn->ttype) == TYPE_oid &&
 		    BATcount(bn) == 0) {
-			bn->tdense = 1;
+			bn->tdense = true;
 			BATtseqbase(bn, 0);
 		}
 		ALGODEBUG fprintf(stderr, "#BATproject(l=%s,r=%s)=%s#"BUNFMT"%s%s%s\n",
@@ -256,34 +258,34 @@ BATproject(BAT *l, BAT *r)
 	     lcount > (rcount >> 3) ||
 	     r->batRestricted == BAT_READ)) {
 		/* insert strings as ints, we need to copy the string
-		 * heap whole sale; we can not do this if there are
-		 * nils in the left column, and we will not do it if
-		 * the left is much smaller than the right and the
-		 * right is writable (meaning we have to actually copy
-		 * the right string heap) */
+		 * heap whole sale; we can't do this if there are nils
+		 * in the left column, and we won't do it if the left
+		 * is much smaller than the right and the right is
+		 * writable (meaning we have to actually copy the
+		 * right string heap) */
 		tpe = r->twidth == 1 ? TYPE_bte : (r->twidth == 2 ? TYPE_sht : (r->twidth == 4 ? TYPE_int : TYPE_lng));
 		/* int's nil representation is a valid offset, so
 		 * don't check for nils */
-		nilcheck = 0;
-		stringtrick = 1;
+		nilcheck = false;
+		stringtrick = true;
 	}
 	bn = COLnew(l->hseqbase, tpe, BATcount(l), TRANSIENT);
 	if (bn == NULL)
 		return NULL;
 	if (stringtrick) {
 		/* "string type" */
-		bn->tsorted = 0;
-		bn->trevsorted = 0;
-		bn->tkey = 0;
-		bn->tnonil = 0;
+		bn->tsorted = false;
+		bn->trevsorted = false;
+		bn->tkey = false;
+		bn->tnonil = false;
 	} else {
 		/* be optimistic, we'll clear these if necessary later */
-		bn->tnonil = 1;
-		bn->tsorted = 1;
-		bn->trevsorted = 1;
-		bn->tkey = 1;
+		bn->tnonil = true;
+		bn->tsorted = true;
+		bn->trevsorted = true;
+		bn->tkey = true;
 		if (l->tnonil && r->tnonil)
-			nilcheck = 0; /* don't bother checking: no nils */
+			nilcheck = false; /* don't bother checking: no nils */
 		if (tpe != TYPE_oid &&
 		    tpe != ATOMstorage(tpe) &&
 		    !ATOMvarsized(tpe) &&
@@ -298,7 +300,7 @@ BATproject(BAT *l, BAT *r)
 			tpe = ATOMstorage(tpe);
 		}
 	}
-	bn->tnil = 0;
+	bn->tnil = false;
 
 	switch (tpe) {
 	case TYPE_bte:
@@ -357,28 +359,23 @@ BATproject(BAT *l, BAT *r)
 				goto bailout;
 			bn->tvheap->parentid = bn->batCacheid;
 			bn->tvheap->farmid = BBPselectfarm(bn->batRole, TYPE_str, varheap);
-			if (r->tvheap->filename) {
-				char *nme = BBP_physical(bn->batCacheid);
-				bn->tvheap->filename = GDKfilepath(NOFARM, NULL, nme, "theap");
-				if (bn->tvheap->filename == NULL)
-					goto bailout;
-			}
+			snprintf(bn->tvheap->filename, sizeof(bn->tvheap->filename), "%s.theap", BBP_physical(bn->batCacheid));
 			if (HEAPcopy(bn->tvheap, r->tvheap) != GDK_SUCCEED)
 				goto bailout;
 		}
 		bn->ttype = r->ttype;
-		bn->tvarsized = 1;
+		bn->tvarsized = true;
 		bn->twidth = r->twidth;
 		bn->tshift = r->tshift;
 
-		bn->tnil = 0; /* we don't know */
+		bn->tnil = false; /* we don't know */
 	}
 	/* some properties follow from certain combinations of input
 	 * properties */
 	if (BATcount(bn) <= 1) {
-		bn->tkey = 1;
-		bn->tsorted = 1;
-		bn->trevsorted = 1;
+		bn->tkey = true;
+		bn->tsorted = true;
+		bn->trevsorted = true;
 	} else {
 		bn->tkey = l->tkey && r->tkey;
 		bn->tsorted = (l->tsorted & r->tsorted) | (l->trevsorted & r->trevsorted);
@@ -404,14 +401,15 @@ BATproject(BAT *l, BAT *r)
 
 /* Calculate a chain of BATproject calls.
  * The argument is a NULL-terminated array of BAT pointers.
- * This function is equivalent to a sequence of calls
+ * This function is equivalent (apart from reference counting) to a
+ * sequence of calls
  * bn = BATproject(bats[0], bats[1]);
  * bn = BATproject(bn, bats[2]);
  * ...
  * bn = BATproject(bn, bats[n-1]);
  * return bn;
  * where none of the intermediates are actually produced (and bats[n]==NULL).
- * Note that all BATs except the last must be oid/void tailed.
+ * Note that all BATs except the last must have type oid/void.
  */
 BAT *
 BATprojectchain(BAT **bats)
@@ -437,8 +435,8 @@ BATprojectchain(BAT **bats)
 	const void *nil;	/* nil representation for last BAT */
 	BUN p, cnt, off;
 	oid hseq, tseq;
-	int allnil = 0, nonil = 1;
-	int stringtrick = 0;
+	bool allnil = false, nonil = true;
+	bool stringtrick = false;
 
 	/* count number of participating BATs and allocate some
 	 * temporary work space */
@@ -453,8 +451,7 @@ BATprojectchain(BAT **bats)
 	tseq = oid_nil;		/* initialize, but overwritten before use */
 	off = 0;		/* this will be the BUN offset into last BAT */
 	for (i = n = 0; b != NULL; n++, i++) {
-		if (!b->tnonil)
-			nonil = 0; /* not guaranteed without nils */
+		nonil &= b->tnonil; /* not guaranteed without nils */
 		if (!allnil) {
 			if (n > 0 && ba[i-1].vals == NULL) {
 				/* previous BAT was dense-tailed: we will
@@ -493,9 +490,9 @@ BATprojectchain(BAT **bats)
 						}
 					}
 					if (b->ttype == TYPE_void &&
-					    b->tseqbase == oid_nil) {
+					    is_oid_nil(b->tseqbase)) {
 						tseq = oid_nil;
-						allnil = 1;
+						allnil = true;
 					} else
 						ba[i].vals = (const oid *) Tloc(b, off);
 				}
@@ -509,8 +506,8 @@ BATprojectchain(BAT **bats)
 				} else {
 					tseq = oid_nil;
 					if (b->ttype == TYPE_void &&
-					    b->tseqbase == oid_nil)
-						allnil = 1;
+					    is_oid_nil(b->tseqbase))
+						allnil = true;
 					else
 						ba[i].vals = (const oid *) Tloc(b, 0);
 				}
@@ -527,7 +524,7 @@ BATprojectchain(BAT **bats)
 	}
 	assert(n >= 1);		/* not too few inputs */
 	b = bats[-2];		/* the last BAT in the list (bats[-1]==NULL) */
-	tpe = b->ttype;		/* its type */
+	tpe = ATOMtype(b->ttype); /* its type */
 	nil = ATOMnilptr(tpe);
 	if (allnil) {
 		/* somewhere on the way we encountered a void-nil BAT */
@@ -558,16 +555,16 @@ BATprojectchain(BAT **bats)
 	    cnt > 0 &&
 	    ATOMstorage(b->ttype) == TYPE_str &&
 	    b->batRestricted == BAT_READ) {
-		stringtrick = 1;
+		stringtrick = true;
 		tpe = b->twidth == 1 ? TYPE_bte : (b->twidth == 2 ? TYPE_sht : (b->twidth == 4 ? TYPE_int : TYPE_lng));
 	}
 
-	bn = COLnew(hseq, ATOMtype(tpe), cnt, TRANSIENT);
+	bn = COLnew(hseq, tpe, cnt, TRANSIENT);
 	if (bn == NULL || cnt == 0) {
 		GDKfree(ba);
 		return bn;
 	}
-	bn->tnil = bn->tnonil = 0; /* we're not paying attention to this */
+	bn->tnil = bn->tnonil = false; /* we're not paying attention to this */
 	n = i - 1;		/* ba[n] is last BAT */
 
 /* figure out the "other" type, i.e. not compatible with oid */
@@ -604,7 +601,7 @@ BATprojectchain(BAT **bats)
 					}
 					o = ba[i].vals[o];
 				}
-				if (o == oid_nil) {
+				if (is_oid_nil(o)) {
 					*v++ = *(oid *) nil;
 				} else {
 					if (o < ba[n].hlo || o >= ba[n].cnt) {
@@ -622,7 +619,7 @@ BATprojectchain(BAT **bats)
 					o -= ba[i].hlo;
 					if (o >= ba[i].cnt) {
 						if (o == oid_nil - ba[i].hlo) {
-							bn->tnil = 1;
+							bn->tnil = true;
 							o = oid_nil;
 							break;
 						}
@@ -631,7 +628,7 @@ BATprojectchain(BAT **bats)
 					}
 					o = ba[i].vals[o];
 				}
-				*v++ = (o == oid_nil) & !stringtrick ? *(oid *) nil : o;
+				*v++ = (is_oid_nil(o)) & !stringtrick ? *(oid *) nil : o;
 			}
 		}
 		assert(v == (oid *) Tloc(bn, cnt));
@@ -646,7 +643,7 @@ BATprojectchain(BAT **bats)
 				o -= ba[i].hlo;
 				if (o >= ba[i].cnt) {
 					if (o == oid_nil - ba[i].hlo) {
-						bn->tnil = 1;
+						bn->tnil = true;
 						o = oid_nil;
 						break;
 					}
@@ -655,7 +652,7 @@ BATprojectchain(BAT **bats)
 				}
 				o = ba[i].vals[o];
 			}
-			if (o == oid_nil) {
+			if (is_oid_nil(o)) {
 				*dst++ = * (OTPE *) nil;
 			} else {
 				o -= ba[n].hlo;
@@ -678,7 +675,7 @@ BATprojectchain(BAT **bats)
 				o -= ba[i].hlo;
 				if (o >= ba[i].cnt) {
 					if (o == oid_nil - ba[i].hlo) {
-						bn->tnil = 1;
+						bn->tnil = true;
 						v = nil;
 						o = oid_nil;
 						break;
@@ -688,7 +685,7 @@ BATprojectchain(BAT **bats)
 				}
 				o = ba[i].vals[o];
 			}
-			if (o != oid_nil) {
+			if (!is_oid_nil(o)) {
 				o -= ba[n].hlo;
 				if (o >= ba[n].cnt) {
 					GDKerror("BATprojectchain: does not match always\n");
@@ -709,7 +706,7 @@ BATprojectchain(BAT **bats)
 				o -= ba[i].hlo;
 				if (o >= ba[i].cnt) {
 					if (o == oid_nil - ba[i].hlo) {
-						bn->tnil = 1;
+						bn->tnil = true;
 						v = nil;
 						o = oid_nil;
 						break;
@@ -719,7 +716,7 @@ BATprojectchain(BAT **bats)
 				}
 				o = ba[i].vals[o];
 			}
-			if (o != oid_nil) {
+			if (!is_oid_nil(o)) {
 				o -= ba[n].hlo;
 				if (o >= ba[n].cnt) {
 					GDKerror("BATprojectchain: does not match always\n");
@@ -732,17 +729,17 @@ BATprojectchain(BAT **bats)
 	}
 	BATsetcount(bn, cnt);
 	if (stringtrick) {
-		bn->tnonil = bn->tnil = 0;
-		bn->tkey = 0;
+		bn->tnonil = bn->tnil = false;
+		bn->tkey = false;
 		BBPshare(b->tvheap->parentid);
 		bn->tvheap = b->tvheap;
 		bn->ttype = b->ttype;
-		bn->tvarsized = 1;
+		bn->tvarsized = true;
 		bn->twidth = b->twidth;
 		bn->tshift = b->tshift;
 	}
 	bn->tsorted = bn->trevsorted = cnt <= 1;
-	bn->tdense = 0;
+	bn->tdense = false;
 	GDKfree(ba);
 	return bn;
 
