@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -32,26 +32,16 @@
 #include "gdk_system.h"
 #include "gdk_system_private.h"
 
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <time.h>
 
-#ifndef HAVE_GETTIMEOFDAY
 #ifdef HAVE_FTIME
-#include <sys/timeb.h>
+#include <sys/timeb.h>		/* ftime */
 #endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>		/* gettimeofday */
 #endif
 
-#ifdef HAVE_SIGNAL_H
-# include <signal.h>
-#endif
+#include <signal.h>
 
 #include <unistd.h>		/* for sysconf symbols */
 
@@ -64,92 +54,14 @@ ATOMIC_TYPE volatile GDKlocksleepcnt;
 MT_Lock * volatile GDKlocklist = 0;
 ATOMIC_FLAG volatile GDKlocklistlock = ATOMIC_FLAG_INIT;
 
-/* merge sort of linked list */
-static MT_Lock *
-sortlocklist(MT_Lock *l)
-{
-	MT_Lock *r, *t, *ll = NULL;
-
-	if (l == NULL || l->next == NULL) {
-		/* list is trivially sorted (0 or 1 element) */
-		return l;
-	}
-	/* break list into two (almost) equal pieces:
-	* l is start of "left" list, r of "right" list, ll last
-	* element of "left" list */
-	for (t = r = l; t && t->next; t = t->next->next) {
-		ll = r;
-		r = r->next;
-	}
-	ll->next = NULL;	/* break list into two */
-	/* recursively sort both sublists */
-	l = sortlocklist(l);
-	r = sortlocklist(r);
-	/* merge
-	 * t is new list, ll is last element of new list, l and r are
-	 * start of unprocessed part of left and right lists */
-	t = ll = NULL;
-	while (l && r) {
-		if (l->sleep < r->sleep ||
-		    (l->sleep == r->sleep &&
-		     l->contention < r->contention) ||
-		    (l->sleep == r->sleep &&
-		     l->contention == r->contention &&
-		     l->count <= r->count)) {
-			/* l is smaller */
-			if (ll == NULL) {
-				assert(t == NULL);
-				t = ll = l;
-			} else {
-				ll->next = l;
-				ll = ll->next;
-			}
-			l = l->next;
-		} else {
-			/* r is smaller */
-			if (ll == NULL) {
-				assert(t == NULL);
-				t = ll = r;
-			} else {
-				ll->next = r;
-				ll = ll->next;
-			}
-			r = r->next;
-		}
-	}
-	/* append rest of remaining list */
-	ll->next = l ? l : r;
-	return t;
-}
-
 void
 GDKlockstatistics(int what)
 {
-	MT_Lock *l;
-
-	if (ATOMIC_TAS(GDKlocklistlock, dummy) != 0) {
-		fprintf(stderr, "#WARNING: GDKlocklistlock is set, so cannot access lock list\n");
-		return;
-	}
-	GDKlocklist = sortlocklist(GDKlocklist);
-	fprintf(stderr, "# lock name\tcount\tcontention\tsleep\tlocked\t(un)locker\n");
-	for (l = GDKlocklist; l; l = l->next)
-		if (what == 0 ||
-		    (what == 1 && l->count) ||
-		    (what == 2 && l->contention) ||
-		    (what == 3 && l->lock))
-			fprintf(stderr, "# %-18s\t" SZFMT "\t" SZFMT "\t" SZFMT "\t%s\t%s\n",
-				l->name ? l->name : "unknown",
-				l->count, l->contention, l->sleep,
-				l->lock ? "locked" : "",
-				l->locker ? l->locker : "");
-	fprintf(stderr, "#total lock count " SZFMT "\n", (size_t) GDKlockcnt);
-	fprintf(stderr, "#lock contention  " SZFMT "\n", (size_t) GDKlockcontentioncnt);
-	fprintf(stderr, "#lock sleep count " SZFMT "\n", (size_t) GDKlocksleepcnt);
-	ATOMIC_CLEAR(GDKlocklistlock, dummy);
+	(void) what;
 }
 #endif
 
+
 static struct posthread {
 	struct posthread *next;
 	pthread_t tid;
@@ -206,7 +118,7 @@ rm_posthread_locked(struct posthread *p)
 		*pp = p->next;
 }
 
-static void
+static void *
 thread_starter(void *arg)
 {
 	struct posthread *p = (struct posthread *) arg;
@@ -219,6 +131,19 @@ thread_starter(void *arg)
 	if ((p = find_posthread_locked(tid)) != NULL)
 		p->exited = 1;
 	pthread_mutex_unlock(&posthread_lock);
+	return NULL;
+}
+
+static void *
+thread_starter_simple(void *arg)
+{
+	struct posthread *p = (struct posthread *) arg;
+	void (*pfunc)(void *) = p->func;
+	void *parg = p->arg;
+
+	free(p);
+	(*pfunc)(parg);
+	return NULL;
 }
 
 static void
@@ -276,6 +201,7 @@ MT_create_thread(MT_Id *t, void (*f) (void *), void *arg, enum MT_thr_detach d)
 	pthread_t newt, *newtp;
 	int ret;
 	struct posthread *p = NULL;
+	void *(*pf) (void *);
 
 	join_threads();
 #ifdef HAVE_PTHREAD_SIGMASK
@@ -292,39 +218,39 @@ MT_create_thread(MT_Id *t, void (*f) (void *), void *arg, enum MT_thr_detach d)
 		pthread_attr_destroy(&attr);
 		return -1;
 	}
-	if (d == MT_THR_DETACHED) {
-		p = malloc(sizeof(struct posthread));
-		if (p == NULL) {
+	p = malloc(sizeof(struct posthread));
+	if (p == NULL) {
 #ifdef HAVE_PTHREAD_SIGMASK
-			MT_thread_sigmask(&orig_mask, NULL);
+		MT_thread_sigmask(&orig_mask, NULL);
 #endif
-			pthread_attr_destroy(&attr);
-			return -1;
-		}
-		p->func = f;
-		p->arg = arg;
-		p->exited = 0;
-		f = thread_starter;
-		arg = p;
+		pthread_attr_destroy(&attr);
+		return -1;
+	}
+	p->func = f;
+	p->arg = arg;
+	p->exited = 0;
+	if (d == MT_THR_DETACHED) {
+		pf = thread_starter;
 		newtp = &p->tid;
 	} else {
+		pf = thread_starter_simple;
 		newtp = &newt;
 		assert(d == MT_THR_JOINABLE);
 	}
-	ret = pthread_create(newtp, &attr, (void *(*)(void *)) f, arg);
+	ret = pthread_create(newtp, &attr, pf, p);
 	if (ret == 0) {
 #ifdef PTW32
 		*t = (MT_Id) (((size_t) newtp->p) + 1);	/* use pthread-id + 1 */
 #else
 		*t = (MT_Id) (((size_t) *newtp) + 1);	/* use pthread-id + 1 */
 #endif
-		if (p) {
+		if (d == MT_THR_DETACHED) {
 			pthread_mutex_lock(&posthread_lock);
 			p->next = posthreads;
 			posthreads = p;
 			pthread_mutex_unlock(&posthread_lock);
 		}
-	} else if (p) {
+	} else {
 		free(p);
 	}
 #ifdef HAVE_PTHREAD_SIGMASK
@@ -345,6 +271,7 @@ MT_exiting_thread(void)
 	pthread_mutex_unlock(&posthread_lock);
 }
 
+/* coverity[+kill] */
 void
 MT_exit_thread(int s)
 {
@@ -453,54 +380,6 @@ MT_getpid(void)
 #endif
 }
 
-#define SMP_TOLERANCE 0.40
-#define SMP_ROUNDS 1024*1024*128
-
-static void
-smp_thread(void *data)
-{
-	unsigned int s = 1, r;
-
-	(void) data;
-	for (r = 0; r < SMP_ROUNDS; r++)
-		s = s * r + r;
-	(void) s;
-}
-
-static int
-MT_check_nr_cores_(void)
-{
-	int i, curr = 1, cores = 1, failed = 0;
-	double lasttime = 0, thistime;
-
-	while (!failed) {
-		lng t0, t1;
-		MT_Id *threads = malloc(sizeof(MT_Id) * curr);
-
-		if (threads == NULL)
-			break;
-
-		t0 = GDKusec();
-		for (i = 0; i < curr; i++)
-			if (MT_create_thread(threads + i, smp_thread, NULL, MT_THR_JOINABLE) < 0) {
-				curr = i;
-				failed = 1;
-				break;
-			}
-		for (i = 0; i < curr; i++)
-			MT_join_thread(threads[i]);
-		t1 = GDKusec();
-		free(threads);
-		thistime = (double) (t1 - t0) / 1000000;
-		if (lasttime > 0 && thistime / lasttime > 1 + SMP_TOLERANCE)
-			break;
-		lasttime = thistime;
-		cores = curr;
-		curr *= 2;	/* only check for powers of 2 */
-	}
-	return cores ? cores : 1;
-}
-
 int
 MT_check_nr_cores(void)
 {
@@ -530,7 +409,7 @@ MT_check_nr_cores(void)
 	 * http://ndevilla.free.fr/threads/ */
 
 	if (ncpus <= 0)
-		ncpus = MT_check_nr_cores_();
+		ncpus = 1;
 #if SIZEOF_SIZE_T == SIZEOF_INT
 	/* On 32-bits systems with large amounts of cpus/cores, we quickly
 	 * run out of space due to the amount of threads in use.  Since it
