@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef _MAL_ATOM_H
@@ -11,8 +11,9 @@
 
 /* #define MAL_ATOM_DEBUG  */
 
-mal_export int malAtomSize(int size, int align, char *name);
-mal_export int malAtomProperty(MalBlkPtr mb, InstrPtr pci);
-mal_export int malAtomDefinition(stream *out, str name, int tpe);
+mal_export int malAtomSize(int size, const char *name);
+mal_export str malAtomProperty(MalBlkPtr mb, InstrPtr pci);
+mal_export str malAtomDefinition(str name, int tpe);
+mal_export void mal_atom_reset(void);
 
 #endif /*  _MAL_ATOM_H*/
