@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -549,7 +549,7 @@ DFLOWinitBlk(DataFlow flow, MalBlkPtr mb, int size)
 	PARDEBUG fprintf(stderr, "#Initialize dflow block\n");
 	assign = (int *) GDKzalloc(mb->vtop * sizeof(int));
 	if (assign == NULL)
-		throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+		throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	etop = flow->stop - flow->start;
 	for (n = 0, pc = flow->start; pc < flow->stop; pc++, n++) {
 		p = getInstrPtr(mb, pc);
@@ -588,13 +588,13 @@ DFLOWinitBlk(DataFlow flow, MalBlkPtr mb, int size)
 						tmp = (int*) GDKrealloc(flow->nodes, sizeof(int) * 2 * size);
 						if (tmp == NULL) {
 							GDKfree(assign);
-							throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+							throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 						}
 						flow->nodes = tmp;
 						tmp = (int*) GDKrealloc(flow->edges, sizeof(int) * 2 * size);
 						if (tmp == NULL) {
 							GDKfree(assign);
-							throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+							throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 						}
 						flow->edges = tmp;
 						size *=2;
@@ -632,13 +632,13 @@ DFLOWinitBlk(DataFlow flow, MalBlkPtr mb, int size)
 							tmp = (int*) GDKrealloc(flow->nodes, sizeof(int) * 2 * size);
 							if (tmp == NULL) {
 								GDKfree(assign);
-								throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+								throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 							}
 							flow->nodes = tmp;
 							tmp = (int*) GDKrealloc(flow->edges, sizeof(int) * 2 * size);
 							if (tmp == NULL) {
 								GDKfree(assign);
-								throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+								throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 							}
 							flow->edges = tmp;
 							size *=2;
@@ -897,7 +897,7 @@ runMALdataflow(Client cntxt, MalBlkPtr mb, int startpc, int stoppc, MalStkPtr st
 
 	flow = (DataFlow)GDKzalloc(sizeof(DataFlowRec));
 	if (flow == NULL)
-		throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+		throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 
 	flow->cntxt = cntxt;
 	flow->mb = mb;
@@ -921,7 +921,7 @@ runMALdataflow(Client cntxt, MalBlkPtr mb, int startpc, int stoppc, MalStkPtr st
 		q_destroy(flow->done);
 		MT_lock_destroy(&flow->flowlock);
 		GDKfree(flow);
-		throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+		throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	}
 	size = DFLOWgraphSize(mb, startpc, stoppc);
 	size += stoppc - startpc;
@@ -931,7 +931,7 @@ runMALdataflow(Client cntxt, MalBlkPtr mb, int startpc, int stoppc, MalStkPtr st
 		q_destroy(flow->done);
 		MT_lock_destroy(&flow->flowlock);
 		GDKfree(flow);
-		throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+		throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	}
 	flow->edges = (int*)GDKzalloc(sizeof(int) * size);
 	if (flow->edges == NULL) {
@@ -940,7 +940,7 @@ runMALdataflow(Client cntxt, MalBlkPtr mb, int startpc, int stoppc, MalStkPtr st
 		q_destroy(flow->done);
 		MT_lock_destroy(&flow->flowlock);
 		GDKfree(flow);
-		throw(MAL, "dataflow", MAL_MALLOC_FAIL);
+		throw(MAL, "dataflow", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	}
 	msg = DFLOWinitBlk(flow, mb, size);
 
