@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef _MAL_IMPORT_H
@@ -14,9 +14,9 @@
 #include "mal_session.h"
 #include "mal_utils.h"
 
-mal_export void slash_2_dir_sep(str fname);
-mal_export str evalFile(Client c, str fname, int listing);
-mal_export str compileString(Symbol *fcn, Client c, str s);
-mal_export int callString(Client c, str s, int listing);
 mal_export str malInclude(Client c, str name, int listing);
+mal_export void slash_2_dir_sep(str fname);
+mal_export str evalFile(str fname, int listing);
+mal_export str compileString(Symbol *fcn, Client c, str s);
+mal_export str callString(Client c, str s, int listing);
 #endif /*  _MAL_IMPORT_H */
