@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef _MAL_LINKER_H
@@ -23,13 +23,12 @@
 
 /* #define DEBUG_MAL_LINKER */
 #define MONET64 1
-mal_export MALfcn getAddress(stream *out, str filename, str fcnname,int silent);
+mal_export MALfcn getAddress(str fcnname);
 mal_export char *MSP_locate_sqlscript(const char *mod_name, bit recurse);
 mal_export str loadLibrary(str modulename, int flag);
 mal_export char *locate_file(const char *basename, const char *ext, bit recurse);
 mal_export int malLibraryEnabled(str name);
 mal_export char* malLibraryHowToEnable(str name);
 mal_export char * MSP_locate_script(const char *filename);
-mal_export void mal_linker_reset(void);
 
 #endif /* _MAL_LINKER_H */
