@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef _MAL_RESOLVE_H
@@ -19,10 +19,10 @@
 */
 #define MAXTYPEVAR  10
 
-mal_export void chkProgram(stream *out, Module s, MalBlkPtr mb);
-mal_export int chkInstruction(stream *out, Module s, MalBlkPtr mb, InstrPtr p);
-mal_export void chkTypes(stream *out, Module s, MalBlkPtr mb, int silent);
-mal_export void typeChecker(stream *out,  Module scope, MalBlkPtr mb, InstrPtr p, int silent);
+mal_export void chkProgram(Module s, MalBlkPtr mb);
+mal_export int chkInstruction(Module s, MalBlkPtr mb, InstrPtr p);
+mal_export void chkTypes(Module s, MalBlkPtr mb, int silent);
+mal_export void typeChecker(Module scope, MalBlkPtr mb, InstrPtr p, int silent);
 
 extern str traceFcnName;
 mal_export void expandMacro(MalBlkPtr mb, InstrPtr p, MalBlkPtr mc);
