@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -90,9 +90,6 @@
  * defined for code modules in general.
  *
  */
-/*
- * @-
- */
 #include "monetdb_config.h"
 #include "mal_scenario.h"
 #include "mal_linker.h"		/* for getAddress() */
@@ -114,7 +111,8 @@ static struct SCENARIO scenarioRec[MAXSCEN] = {
 	 "MALparser", (MALfcn) &MALparser, 0,
 	 "MALoptimizer", 0, 0,
 	 0, 0, 0,
-	 "MALengine", (MALfcn) &MALengine, 0, 0},
+	 "MALengine", (MALfcn) &MALengine, 0,
+	 "MALcallback", (MALfcn) &MALcallback, 0,0 },
 	{"profiler","profiler",			/* name */
 	 0, 0,			/* initClient */
 	 0, 0,			/* exitClient */
@@ -124,17 +122,19 @@ static struct SCENARIO scenarioRec[MAXSCEN] = {
 	 "MALparser", (MALfcn) &MALparser, 0,		/* parser */
 	 0, 0, 0,		/* optimizer */
 	 0, 0, 0,		/* scheduler */
-	 0, 0, 0, 0		/* engine */
+	 0, 0, 0,		/* callback */
+	 0, 0, 0,0		/* engine */
 	 },
-	{0,0,			/* name */
-	 0, 0,			/* init */
-	 0, 0,			/* exit */
-	 0, 0,			/* initClient */
-	 0, 0,			/* exitClient */
+	{0, 0,		/* name */
+	 0, 0,		/* init */
+	 0, 0,		/* exit */
+	 0, 0,		/* initClient */
+	 0, 0,		/* exitClient */
 	 0, 0, 0,		/* reader */
 	 0, 0, 0,		/* parser */
 	 0, 0, 0,		/* optimizer */
 	 0, 0, 0,		/* scheduler */
+	 0, 0, 0,		/* callback */
 	 0, 0, 0, 0		/* engine */
 	 }
 };
@@ -156,12 +156,8 @@ getFreeScenario(void)
 	MT_lock_set(&scenarioLock);
 	for (i = 0; i < MAXSCEN && scenarioRec[i].name; i++)
 		;
-
-	if (i == MAXSCEN) {
-		showException(GDKout, MAL,"freeScenario", "no scenario space left (%d); adjust MAXSCEN and recompile", MAXSCEN);
-	} else {
+	if (i < MAXSCEN) 
 		scen = scenarioRec + i;
-	}
 	MT_lock_unset(&scenarioLock);
 
 	return scen;
@@ -187,7 +183,7 @@ initScenario(Client c, Scenario s)
 	/* prepare for conclicts */
 	MT_lock_set(&mal_contextLock);
 	if (s->initSystem && s->initSystemCmd == 0) {
-		s->initSystemCmd = (MALfcn) getAddress(c->fdout, l, s->initSystem,0);
+		s->initSystemCmd = (MALfcn) getAddress(s->initSystem);
 		if (s->initSystemCmd) {
 			msg = (*s->initSystemCmd) (c);
 		} else {
@@ -202,21 +198,23 @@ initScenario(Client c, Scenario s)
 	}
 
 	if (s->exitSystem && s->exitSystemCmd == 0)
-		s->exitSystemCmd = (MALfcn) getAddress(c->fdout, l, s->exitSystem,0);
+		s->exitSystemCmd = (MALfcn) getAddress(s->exitSystem);
 	if (s->initClient && s->initClientCmd == 0)
-		s->initClientCmd = (MALfcn) getAddress(c->fdout, l, s->initClient,0);
+		s->initClientCmd = (MALfcn) getAddress(s->initClient);
 	if (s->exitClient && s->exitClientCmd == 0)
-		s->exitClientCmd = (MALfcn) getAddress(c->fdout, l, s->exitClient,0);
+		s->exitClientCmd = (MALfcn) getAddress(s->exitClient);
 	if (s->reader && s->readerCmd == 0)
-		s->readerCmd = (MALfcn) getAddress(c->fdout, l, s->reader,0);
+		s->readerCmd = (MALfcn) getAddress(s->reader);
 	if (s->parser && s->parserCmd == 0)
-		s->parserCmd = (MALfcn) getAddress(c->fdout, l, s->parser,0);
+		s->parserCmd = (MALfcn) getAddress(s->parser);
 	if (s->optimizer && s->optimizerCmd == 0)
-		s->optimizerCmd = (MALfcn) getAddress(c->fdout, l, s->optimizer,0);
+		s->optimizerCmd = (MALfcn) getAddress(s->optimizer);
 	if (s->tactics && s->tacticsCmd == 0)
-		s->tacticsCmd = (MALfcn) getAddress(c->fdout, l, s->tactics,0);
+		s->tacticsCmd = (MALfcn) getAddress(s->tactics);
+	if (s->callback && s->callbackCmd == 0)
+		s->callbackCmd = (MALfcn) getAddress(s->callback);
 	if (s->engine && s->engineCmd == 0)
-		s->engineCmd = (MALfcn) getAddress(c->fdout, l, s->engine,0);
+		s->engineCmd = (MALfcn) getAddress(s->engine);
 	MT_lock_unset(&mal_contextLock);
 	return(fillScenario(c, s));
 }
@@ -256,6 +254,7 @@ showScenario(stream *f, Scenario scen)
 	print_scenarioCommand(f, scen->parser, scen->parserCmd);
 	print_scenarioCommand(f, scen->optimizer, scen->optimizerCmd);
 	print_scenarioCommand(f, scen->tactics, scen->tacticsCmd);
+	print_scenarioCommand(f, scen->callback, scen->callbackCmd);
 	print_scenarioCommand(f, scen->engine, scen->engineCmd);
 	mnstr_printf(f, "]\n");
 }
@@ -315,6 +314,10 @@ updateScenario(str nme, str fnme, MALfcn fcn)
 		scen->tacticsCmd = fcn;
 		phase = MAL_SCENARIO_SCHEDULER;
 	}
+	if (scen->callback && strcmp(scen->callback, fnme) == 0) {
+		scen->callbackCmd = fcn;
+		phase = MAL_SCENARIO_CALLBACK;
+	}
 	if (scen->engine && strcmp(scen->engine, fnme) == 0) {
 		scen->engineCmd = fcn;
 		phase = MAL_SCENARIO_ENGINE;
@@ -380,6 +383,7 @@ fillScenario(Client c, Scenario scen)
 	c->phase[MAL_SCENARIO_PARSER] = scen->parserCmd;
 	c->phase[MAL_SCENARIO_OPTIMIZE] = scen->optimizerCmd;
 	c->phase[MAL_SCENARIO_SCHEDULER] = scen->tacticsCmd;
+	c->phase[MAL_SCENARIO_CALLBACK] = scen->callbackCmd;
 	c->phase[MAL_SCENARIO_ENGINE] = scen->engineCmd;
 	c->phase[MAL_SCENARIO_INITCLIENT] = scen->initClientCmd;
 	c->phase[MAL_SCENARIO_EXITCLIENT] = scen->exitClientCmd;
@@ -410,19 +414,19 @@ setScenario(Client c, str nme)
 
 	if (c->scenario) {
 		c->oldscenario = c->scenario;
-		for (i = 0; i < 7; i++) {
+		for (i = 0; i < SCENARIO_PROPERTIES; i++) {
 			c->oldstate[i] = c->state[i];
 			c->oldphase[i] = c->phase[i];
 		}
 	}
-	for (i = 0; i < 7; i++)
+	for (i = 0; i < SCENARIO_PROPERTIES; i++)
 		c->state[i] = 0;
 
 	msg = initScenario(c, scen);
 	if (msg) {
 		/* error occurred, reset the scenario , assume default always works */
 		c->scenario = c->oldscenario;
-		for (i = 0; i < 7; i++) {
+		for (i = 0; i < SCENARIO_PROPERTIES; i++) {
 			c->state[i] = c->oldstate[i];
 			c->phase[i] = c->oldphase[i];
 			c->oldstate[i] = NULL;
@@ -460,7 +464,7 @@ resetScenario(Client c)
 	if (scen != NULL && scen->exitClientCmd)
 		(*scen->exitClientCmd) (c);
 	c->scenario = c->oldscenario;
-	for (i = 0; i < 7; i++) {
+	for (i = 0; i < SCENARIO_PROPERTIES; i++) {
 		c->state[i] = c->oldstate[i];
 		c->phase[i] = c->oldphase[i];
 	}
@@ -499,7 +503,7 @@ static str
 runPhase(Client c, int phase)
 {
 	str msg = MAL_SUCCEED;
-	if (c->phase[c->stage = phase])
+	if (c->phase[phase])
 	    return msg = (str) (*c->phase[phase])(c);
 	return msg;
 }
@@ -509,60 +513,57 @@ runPhase(Client c, int phase)
  * running a scenario should be explicitly permitted.
  */
 static str
-runScenarioBody(Client c)
+runScenarioBody(Client c, int once)
 {
 	str msg= MAL_SUCCEED;
-	lng start;
 
 	c->exception_buf_initialized = 1;
 	if (setjmp( c->exception_buf) < 0)
 		c->mode = FINISHCLIENT;
-	while ((c->mode > FINISHCLIENT || msg != MAL_SUCCEED) && !GDKexiting()) {
+	while (c->mode > FINISHCLIENT && !GDKexiting()) {
+		// be aware that a MAL call  may initialize a different scenario
+		if ( !c->state[0] && (msg = runPhase(c, MAL_SCENARIO_INITCLIENT)) ) 
+			goto wrapup;
+		if ( c->mode <= FINISHCLIENT ||  (msg = runPhase(c, MAL_SCENARIO_READER)) )
+			goto wrapup;
+		if ( c->mode <= FINISHCLIENT  || (msg = runPhase(c, MAL_SCENARIO_PARSER)) || c->blkmode)
+			goto wrapup;
+		if ( c->mode <= FINISHCLIENT ||  (msg = runPhase(c, MAL_SCENARIO_OPTIMIZE)) )
+			goto wrapup;
+		if ( c->mode <= FINISHCLIENT || (msg = runPhase(c, MAL_SCENARIO_SCHEDULER)))
+			goto wrapup;
+		if ( c->mode <= FINISHCLIENT || (msg = runPhase(c, MAL_SCENARIO_ENGINE)))
+			goto wrapup;
+	wrapup:
 		if (msg != MAL_SUCCEED){
-/* we should actually show it [postponed]
-			mnstr_printf(c->fdout,"!%s\n",msg);
-*/
-			freeException(msg);
-			msg = MAL_SUCCEED;
+			if(c->phase[MAL_SCENARIO_CALLBACK])
+				msg = (str) (*c->phase[MAL_SCENARIO_CALLBACK])(c, msg);
+			if (msg) {
+				mnstr_printf(c->fdout,"!%s%s", msg, (msg[strlen(msg)-1] == '\n'? "":"\n"));
+				freeException(msg);
+				msg = MAL_SUCCEED;
+			}
 		}
-		if (!c->state[0] &&
-		    (msg = runPhase(c, MAL_SCENARIO_INITCLIENT)) != MAL_SUCCEED)
-			continue;
-		if (c->mode <= FINISHCLIENT ||
-		    (msg = runPhase(c, MAL_SCENARIO_READER)) != MAL_SUCCEED)
-			continue;
-		c->lastcmd= time(0);
-		start= GDKusec();
-		if (c->mode <= FINISHCLIENT ||
-		    (msg = runPhase(c, MAL_SCENARIO_PARSER)) != MAL_SUCCEED)
-			continue;
-		if (c->mode <= FINISHCLIENT ||
-		    (msg = runPhase(c, MAL_SCENARIO_OPTIMIZE)) != MAL_SUCCEED)
-			continue;
-		if (c->mode <= FINISHCLIENT ||
-                    (msg = runPhase(c, MAL_SCENARIO_SCHEDULER)) != MAL_SUCCEED)
-			continue;
-		if (c->mode <= FINISHCLIENT ||
-		    (msg = runPhase(c, MAL_SCENARIO_ENGINE)) != MAL_SUCCEED)
-			continue;
+		if( GDKerrbuf && GDKerrbuf[0])
+			mnstr_printf(c->fdout,"!GDKerror: %s\n",GDKerrbuf);
+		assert(c->curprg->def->errors == NULL);
 		c->actions++;
-		start = GDKusec()-start;
-		c->totaltime += start;
+		if( once) break;
 	}
-	if (c->phase[MAL_SCENARIO_EXITCLIENT])
-		(*c->phase[MAL_SCENARIO_EXITCLIENT]) (c);
 	c->exception_buf_initialized = 0;
+	if (once == 0 && c->phase[MAL_SCENARIO_EXITCLIENT])
+		msg = (*c->phase[MAL_SCENARIO_EXITCLIENT]) (c);
 	return msg;
 }
 
 str
-runScenario(Client c)
+runScenario(Client c, int once)
 {
 	str msg = MAL_SUCCEED;
 
 	if (c == 0 || c->phase[MAL_SCENARIO_READER] == 0)
 		return msg;
-	msg = runScenarioBody(c);
+	msg = runScenarioBody(c,once);
 	if (msg != MAL_SUCCEED &&
 			strcmp(msg,"MALException:client.quit:Server stopped."))
 		mnstr_printf(c->fdout,"!%s\n",msg);
