@@ -1,9 +1,10 @@
+
 /*
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /* (author) M.L. Kersten
@@ -11,7 +12,7 @@
 #include "monetdb_config.h"
 #include "mal_session.h"
 #include "mal_instruction.h" /* for pushEndInstruction() */
-#include "mal_interpreter.h" /* for showErrors(), runMAL(), garbageElement() */
+#include "mal_interpreter.h" /* for runMAL(), garbageElement() */
 #include "mal_parser.h"	     /* for parseMAL() */
 #include "mal_namespace.h"
 #include "mal_builder.h"
@@ -24,50 +25,40 @@
  * consist of include statements to load modules of general interest.
  * The startup script is run as user Admin.
  */
-int
+str
 malBootstrap(void)
 {
 	Client c = NULL;
 	str msg = NULL, bootfile = "mal_init";
-	char ret = 0;
 
 	c = MCinitClient((oid) 0, 0, 0);
 	if (!c) {
-		GDKerror("malBootstrap: Failed to initialize client");
-		goto cleanup;
+		return GDKstrdup("malBootstrap: Failed to initialize client");
+	}
+	c->curmodule = c->usermodule = userModule();
+	if(c->usermodule == NULL) {
+		return GDKstrdup("#malBootstrap:Failed to initialise client MAL module");
 	}
-	c->nspace = newModule(NULL, putName("user"));
 	if ( (msg = defaultScenario(c)) ) {
-		GDKerror("malBootstrap: Failed to initialize default scenario");
-		goto cleanup;
+		return msg;
+	}
+	if((msg = MSinitClientPrg(c, "user", "main")) != MAL_SUCCEED) {
+		return msg;
 	}
-	MSinitClientPrg(c, "user", "main");
 	if( MCinitClientThread(c) < 0){
-		GDKerror("malBootstrap: Failed to create client thread");
-		goto cleanup;
+		return GDKstrdup("#malBootstrap:Failed to create client thread");
 	}
 	msg = malInclude(c, bootfile, 0);
 	if (msg != NULL) {
-		GDKerror("malBootstrap: Failed to load startup script %s", msg);
-		goto cleanup;
+		return msg;
 	}
 	pushEndInstruction(c->curprg->def);
-	chkProgram(c->fdout, c->nspace, c->curprg->def);
-	if (c->curprg->def->errors) {
-		GDKerror("malBootstrap: Failed to check startup script %s", GDKerrbuf);
-		goto cleanup;
+	chkProgram(c->usermodule, c->curprg->def);
+	if ( (msg= c->curprg->def->errors) != MAL_SUCCEED ) {
+		return msg;
 	}
 	msg = MALengine(c);
-	if (msg != MAL_SUCCEED) {
-		goto cleanup;
-	}
-	ret = 1;
-cleanup:
-	freeException(msg);
-	if (c) {
-		MALexitClient(c);
-	}
-	return ret;
+	return msg;
 }
 
 /*
@@ -84,67 +75,73 @@ cleanup:
  * was added.  At the end of the session we have to garbage collect the
  * BATs introduced.
  */
-static void
-MSresetClientPrg(Client cntxt)
+static str
+MSresetClientPrg(Client cntxt, str mod, str fcn)
 {
 	MalBlkPtr mb;
 	InstrPtr p;
 
 	cntxt->itrace = 0;  /* turn off any debugging */
 	mb = cntxt->curprg->def;
-	mb->typefixed = 0;
-	mb->flowfixed = 0;
 	mb->stop = 1;
-	mb->errors = 0;
+	mb->errors = MAL_SUCCEED;
 	p = mb->stmt[0];
 
 	p->gc = 0;
 	p->retc = 1;
 	p->argc = 1;
-	setModuleId(p, putName("user"));
-	setFunctionId(p, putName("main"));
+	p->argv[0] = 0;
+
+#ifdef _DEBUG_SESSION_
+	fprintf(stderr,"reset sym %s %s to %s, id %d\n", 
+		cntxt->curprg->name, getFunctionId(p), nme, findVariable(mb,nme) );
+	fprintf(stderr,"vtop %d\n", mb->vtop);
+	if( mb->vtop)
+	fprintf(stderr,"first var %s\n", mb->var[0].id);
+#endif
+
+	setModuleId(p, mod);
+	setFunctionId(p, fcn);
+	if( findVariable(mb,fcn) < 0)
+		p->argv[0] = newVariable(mb, fcn, strlen(fcn), TYPE_void);
+
+	setVarType(mb, findVariable(mb, fcn), TYPE_void);
 	/* remove any MAL history */
 	if (mb->history) {
 		freeMalBlk(mb->history);
 		mb->history = 0;
 	}
+	return MAL_SUCCEED;
 }
 
-void
+/*
+ * Create a new container block
+ */
+
+str
 MSinitClientPrg(Client cntxt, str mod, str nme)
 {
-	InstrPtr p;
-	MalBlkPtr mb;
+	int idx;
 
-	if (cntxt->curprg && idcmp(nme, cntxt->curprg->name) == 0) {
-		MSresetClientPrg(cntxt);
-		return;
-	}
-	if (cntxt->nspace == 0) {
-		cntxt->nspace = newModule(NULL, putName("user"));
-		if (cntxt->nspace == NULL) /* alloc failed, GDKerror already called */
-			return;
-	}
-	cntxt->curprg = newFunction(putName("user"), putName(nme), FUNCTIONsymbol);
-	if( cntxt->curprg == 0){
-		GDKerror("MSinitClientPrg" "Failed to create function");
-		return;
-	}
-	mb = cntxt->curprg->def;
-	p = getSignature(cntxt->curprg);
-	if (mod)
-		setModuleId(p, mod);
-	else
-		setModuleScope(p, cntxt->nspace);
-	setVarType(mb, findVariable(mb, nme), TYPE_void);
-	insertSymbol(cntxt->nspace, cntxt->curprg);
-	cntxt->glb = 0;
+	if (cntxt->curprg  && idcmp(nme, cntxt->curprg->name) == 0)
+		return MSresetClientPrg(cntxt, putName(mod), putName(nme));
+	cntxt->curprg = newFunction(putName(mod), putName(nme), FUNCTIONsymbol);
+	if( cntxt->curprg == 0)
+		throw(MAL, "initClientPrg", SQLSTATE(HY001) MAL_MALLOC_FAIL);
+	if( (idx= findVariable(cntxt->curprg->def,"main")) >=0)
+		setVarType(cntxt->curprg->def, idx, TYPE_void);
+	insertSymbol(cntxt->usermodule,cntxt->curprg);
+	
+	if (cntxt->glb == NULL )
+		cntxt->glb = newGlobalStack(MAXGLOBALS + cntxt->curprg->def->vsize);
+	if( cntxt->glb == NULL)
+		throw(MAL,"initClientPrg", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	assert(cntxt->curprg->def != NULL);
+	assert(cntxt->curprg->def->vtop >0);
+	return MAL_SUCCEED;
 }
 
 
-
-
 /*
  * After the client initialization has been finished, we can start the
  * interaction protocol. This involves parsing the input in the context
@@ -189,9 +186,12 @@ MSresetVariables(Client cntxt, MalBlkPtr mb, MalStkPtr glb, int start)
 {
 	int i;
 
-	for (i = 0; i < start && start < mb->vtop; i++)
+#ifdef _DEBUG_SESSION_
+	fprintf(stderr,"resetVarables %d  vtop %d errors %s\n", start, mb->vtop,mb->errors);
+#endif
+	for (i = 0; i < start && i < mb->vtop ; i++)
 		setVarUsed(mb,i);
-	if (mb->errors == 0)
+	if (mb->errors == MAL_SUCCEED)
 		for (i = start; i < mb->vtop; i++) {
 			if (isVarUsed(mb,i) || !isTmpVar(mb,i)){
 				assert(!mb->var[i].value.vtype || isVarConstant(mb, i));
@@ -207,11 +207,18 @@ MSresetVariables(Client cntxt, MalBlkPtr mb, MalStkPtr glb, int start)
 			}
 		}
 
-	if (mb->errors == 0)
+#ifdef _DEBUG_SESSION_
+	fprintf(stderr,"resetVar %s %d\n", getFunctionId(mb->stmt[0]), mb->var[mb->stmt[0]->argv[0]].used);
+#endif
+	if (mb->errors == MAL_SUCCEED)
 		trimMalVariables_(mb, glb);
+#ifdef _DEBUG_SESSION_
+	fprintf(stderr,"after trim %s %d\n", getFunctionId(mb->stmt[0]), mb->vtop);
+#endif
 }
 
 /*
+
  * The stages of processing user requests are controlled by a scenario.
  * The routines below are the default implementation.  The main issues
  * to deal after parsing it to clean out the Admin.main function from
@@ -233,21 +240,19 @@ MALinitClient(Client c)
 str
 MALexitClient(Client c)
 {
-	if (c->glb && c->curprg->def->errors == 0)
+	if (c->glb && c->curprg->def->errors == MAL_SUCCEED)
 		garbageCollector(c, c->curprg->def, c->glb, TRUE);
-	if (c->bak)
-		return NULL;
 	c->mode = FINISHCLIENT;
 	if (c->backup) {
 		assert(0);
 		freeSymbol(c->backup);
 		c->backup = NULL;
 	}
-	/* should be in the nspace */
+	/* should be in the usermodule */
 	c->curprg = NULL;
-	if (c->nspace) {
-		freeModule(c->nspace);
-		c->nspace = NULL;
+	if (c->usermodule){
+		freeModule(c->usermodule);
+		c->usermodule = NULL;
 	}
 	return NULL;
 }
@@ -262,8 +267,6 @@ MALreader(Client c)
 	MT_lock_unset(&mal_contextLock);
 	if (c->fdin)
 		c->fdin->buf[c->fdin->pos] = 0;
-	else
-		throw(MAL, "mal.reader", RUNTIME_IO_EOF);
 	return MAL_SUCCEED;
 }
 
@@ -272,56 +275,44 @@ MALparser(Client c)
 {
 	InstrPtr p;
 	MalBlkRecord oldstate;
+	str msg= MAL_SUCCEED;
 
+	assert(c->curprg->def->errors == NULL);
 	c->curprg->def->errors = 0;
 	oldstate = *c->curprg->def;
 
-	if( prepareMalBlk(c->curprg->def, CURRENT(c))){
-		throw(MAL, "mal.parser", MAL_MALLOC_FAIL);
-	}
-	if (parseMAL(c, c->curprg, 0) || c->curprg->def->errors) {
-		/* just complete it for visibility */
-		pushEndInstruction(c->curprg->def);
-		/* caught errors */
-		showErrors(c);
-		if (c->listing)
-			printFunction(c->fdout, c->curprg->def, 0, c->listing);
-		MSresetVariables(c, c->curprg->def, c->glb, oldstate.vtop);
-		resetMalBlk(c->curprg->def, 1);
-		/* now the parsing is done we should advance the stream */
-		c->fdin->pos += c->yycur;
-		c->yycur = 0;
-		throw(SYNTAX, "mal.parser", SYNTAX_GENERAL MANUAL_HELP);
-	}
+	prepareMalBlk(c->curprg->def, CURRENT(c));
+	parseMAL(c, c->curprg, 0, INT_MAX);
 
 	/* now the parsing is done we should advance the stream */
 	c->fdin->pos += c->yycur;
 	c->yycur = 0;
 
 	/* check for unfinished blocks */
-	if (c->blkmode)
+	if(!c->curprg->def->errors && c->blkmode)
 		return MAL_SUCCEED;
 	/* empty files should be skipped as well */
-	if (c->curprg->def->stop == 1)
-		return MAL_SUCCEED;
+	if (c->curprg->def->stop == 1){
+		if ( (msg =c->curprg->def->errors) )
+			c->curprg->def->errors = 0;
+		return msg;
+	}
 
 	p = getInstrPtr(c->curprg->def, 0);
 	if (p->token != FUNCTIONsymbol) {
-		if (c->listing)
-			printFunction(c->fdout, c->curprg->def, 0, c->listing);
+		msg =c->curprg->def->errors;
+		c->curprg->def->errors = 0;
 		MSresetVariables(c, c->curprg->def, c->glb, oldstate.vtop);
 		resetMalBlk(c->curprg->def, 1);
-		throw(SYNTAX, "mal.parser", SYNTAX_SIGNATURE);
+		return msg;
 	}
 	pushEndInstruction(c->curprg->def);
-	chkProgram(c->fdout, c->nspace, c->curprg->def);
-	if (c->curprg->def->errors) {
-		showErrors(c);
-		if (c->listing)
-			printFunction(c->fdout, c->curprg->def, 0, c->listing);
+	chkProgram(c->usermodule, c->curprg->def);
+	if ( (msg =c->curprg->def->errors) ){
+		c->curprg->def->errors = 0;
 		MSresetVariables(c, c->curprg->def, c->glb, oldstate.vtop);
 		resetMalBlk(c->curprg->def, 1);
-		throw(MAL, "MAL.parser", SEMANTIC_GENERAL);
+		return msg;
 	}
 	return MAL_SUCCEED;
 }
@@ -337,6 +328,29 @@ MALcommentsOnly(MalBlkPtr mb)
 	return 1;
 }
 
+str
+MALcallback(Client c, str msg)
+{
+	if (msg) {
+		/* don't print exception decoration, just the message */
+		char *n = NULL;
+		char *o = msg;
+		while ((n = strchr(o, '\n')) != NULL) {
+			if (*o == '!')
+				o++;
+			mnstr_printf(c->fdout, "!%.*s\n", (int) (n - o), o);
+			o = ++n;
+		}
+		if (*o != 0) {
+			if (*o == '!')
+				o++;
+			mnstr_printf(c->fdout, "!%s\n", o);
+		}
+		freeException(msg);
+	}
+	return MAL_SUCCEED;
+}
+
 str
 MALengine(Client c)
 {
@@ -353,13 +367,12 @@ MALengine(Client c)
 	if (prg->def == NULL)
 		throw(SYNTAX, "mal.engine", SYNTAX_SIGNATURE);
 
-	if (prg->def->errors > 0) {
-		showErrors(c);
-		if (c->listing)
-			printFunction(c->fdout, c->curprg->def, 0, c->listing);
+	if (prg->def->errors != MAL_SUCCEED) {
+		msg = prg->def->errors;
+		prg->def->errors = NULL;
 		MSresetVariables(c, c->curprg->def, c->glb, oldstate.vtop);
 		resetMalBlk(c->curprg->def, 1);
-		throw(MAL, "mal.engine", PROGRAM_GENERAL);
+		return msg;
 	}
 	if (prg->def->stop == 1 || MALcommentsOnly(prg->def))
 		return 0;   /* empty block */
@@ -367,14 +380,12 @@ MALengine(Client c)
 		if (prg->def && c->glb->stksize < prg->def->vsize){
 			c->glb = reallocGlobalStack(c->glb, prg->def->vsize);
 			if( c->glb == NULL)
-				throw(MAL, "mal.engine", MAL_MALLOC_FAIL);
+				throw(MAL, "mal.engine", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 		}
 		c->glb->stktop = prg->def->vtop;
 		c->glb->blk = prg->def;
 		c->glb->cmd = (c->itrace && c->itrace != 'C') ? 'n' : 0;
 	}
-	if (c->listing > 1)
-		printFunction(c->fdout, c->curprg->def, 0, c->listing);
 
 	/*
 	 * In interactive mode we should avoid early garbage collection of values.
@@ -385,17 +396,14 @@ MALengine(Client c)
 		c->glb->pcup = 0;
 		c->glb->keepAlive = TRUE; /* no garbage collection */
 	}
-	if (prg->def->errors == 0)
+	if (prg->def->errors == MAL_SUCCEED)
 		msg = (str) runMAL(c, prg->def, 0, c->glb);
 	if (msg) {
 		/* ignore "internal" exceptions */
-		str fcn = getExceptionPlace(msg); /* retrieves from "first" exception */
-		if (strcmp(fcn, "client.quit") != 0)
-			dumpExceptionsToStream(c->fdout, msg);
-		GDKfree(fcn);
-		if (!c->listing)
-			printFunction(c->fdout, c->curprg->def, 0, c->listing);
-		showErrors(c);
+		if (strstr(msg, "client.quit") ) {
+			freeException(msg);
+			msg = MAL_SUCCEED;
+		}
 	}
 	MSresetVariables(c, prg->def, c->glb, 0);
 	resetMalBlk(prg->def, 1);
@@ -403,7 +411,10 @@ MALengine(Client c)
 		/* for global stacks avoid reinitialization from this point */
 		c->glb->stkbot = prg->def->vtop;
 	}
-	prg->def->errors = 0;
+	
+	if (prg->def->errors)
+		GDKfree(prg->def->errors);
+	prg->def->errors = NULL;
 	if (c->itrace)
 		mnstr_printf(c->fdout, "mdb>#EOD\n");
 	return msg;
