@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -54,7 +54,7 @@ AGGRgrouped(bat *retval1, bat *retval2, const bat *bid, const bat *gid, const ba
 			BBPunfix(s->batCacheid);
 		if (q)
 			BBPunfix(q->batCacheid);
-		throw(MAL, malfunc, RUNTIME_OBJECT_MISSING);
+		throw(MAL, malfunc, SQLSTATE(HY002) RUNTIME_OBJECT_MISSING);
 	}
 	if (tp == TYPE_any &&
 		(grpfunc1 == BATgroupmedian || quantilefunc == BATgroupquantile))
