@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -23,7 +23,7 @@ ALGprojectionpath(Client cntxt, MalBlkPtr mb, MalStkPtr stk, InstrPtr pci)
 
 	assert(pci->argc > 1);
 	if ( joins == NULL)
-		throw(MAL, "algebra.projectionpath", MAL_MALLOC_FAIL);
+		throw(MAL, "algebra.projectionpath", SQLSTATE(HY001) MAL_MALLOC_FAIL);
 	for (i = pci->retc; i < pci->argc; i++) {
 		bid = getArgReference_bat(stk, pci, i);
 		b = BATdescriptor(*bid);
