@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef SQL_CATALOG_H
@@ -35,6 +35,7 @@
 #define PRIV_DELETE 8
 #define PRIV_EXECUTE 16
 #define PRIV_GRANT 32
+#define PRIV_TRUNCATE 64
 /* global privs */
 #define PRIV_COPYFROMFILE 1
 #define PRIV_COPYINTOFILE 2
@@ -58,6 +59,8 @@
 #define NO_DEPENDENCY 0
 #define HAS_DEPENDENCY 1
 #define CICLE_DEPENDENCY 2
+#define DEPENDENCY_CHECK_ERROR 3
+#define DEPENDENCY_CHECK_OK 0
 
 #define NO_TRIGGER 0
 #define IS_TRIGGER 1
@@ -295,8 +298,8 @@ typedef struct sql_arg {
 #define FUNC_LANG_MAL 1 /* create sql external mod.func */
 #define FUNC_LANG_SQL 2 /* create ... sql function/procedure */
 #define FUNC_LANG_R   3 /* create .. language R */
-#define FUNC_LANG_C   4 /* create .. language C, Not used/implemented */
-#define FUNC_LANG_J   5 /* create .. language JavaScript, Not used/implemented */
+#define FUNC_LANG_C   4 /* create .. language C */
+#define FUNC_LANG_J   5
 // this should probably be done in a better way
 #define FUNC_LANG_PY  6 /* create .. language PYTHON */
 #define FUNC_LANG_MAP_PY  7 /* create .. language PYTHON_MAP */
@@ -304,6 +307,7 @@ typedef struct sql_arg {
 #define FUNC_LANG_MAP_PY2  9 /* create .. language PYTHON2_MAP */
 #define FUNC_LANG_PY3  10 /* create .. language PYTHON3 */
 #define FUNC_LANG_MAP_PY3  11 /* create .. language PYTHON3_MAP */
+#define FUNC_LANG_CPP   12 /* create .. language CPP */
 
 #define LANG_EXT(l)  (l>FUNC_LANG_SQL)
 
@@ -345,6 +349,7 @@ typedef struct sql_func {
 typedef struct sql_subfunc {
 	sql_func *func;
 	list *res;
+	list *coltypes; /* we need this for copy into from loader */
 	list *colnames; /* we need this for copy into from loader */
 	char *sname, *tname; /* we need this for create table from loader */
 } sql_subfunc;
@@ -417,7 +422,7 @@ typedef struct sql_trigger {
 	sql_base base;
 	sht time;		/* before or after */
 	sht orientation; 	/* row or statement */
-	sht event;		/* insert, delete, update */
+	sht event;		/* insert, delete, update, truncate */
 	/* int action_order;	 TODO, order within the set of triggers */
 	struct list *columns;	/* update trigger on list of (sql_kc) columns */
 
