@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -23,6 +23,15 @@ insert_value(mvc *sql, sql_column *c, sql_rel **r, symbol *s)
 {
 	if (s->token == SQL_NULL) {
 		return exp_atom(sql->sa, atom_general(sql->sa, &c->type, NULL));
+	} else if (s->token == SQL_DEFAULT) {
+		if (c->def) {
+			sql_exp *e = rel_parse_val(sql, sa_message(sql->sa, "select CAST(%s AS %s);", c->def, c->type.type->sqlname), sql->emode);
+			if (!e || (e = rel_check_type(sql, &c->type, e, type_equal)) == NULL)
+				return NULL;
+			return e;
+		} else {
+			return sql_error(sql, 02, SQLSTATE(42000) "INSERT INTO: column '%s' has no valid default value", c->base.name);
+		}
 	} else {
 		int is_last = 0;
 		exp_kind ek = {type_value, card_value, FALSE};
@@ -293,7 +302,7 @@ check_table_columns(mvc *sql, sql_table *t, dlist *columns, char *op, char *tnam
 			if (c) {
 				list_append(collist, c);
 			} else {
-				return sql_error(sql, 02, "42S22!%s INTO: no such column '%s.%s'", op, tname, n->data.sval);
+				return sql_error(sql, 02, SQLSTATE(42S22) "%s INTO: no such column '%s.%s'", op, tname, n->data.sval);
 			}
 		}
 	} else {
@@ -348,7 +357,7 @@ rel_inserts(mvc *sql, sql_table *t, sql_rel *r, list *collist, size_t rowcount,
 							e = exp_atom(sql->sa, a);
 						}
 						if (!e) 
-							return sql_error(sql, 02, "INSERT INTO: column '%s' has no valid default value", c->base.name);
+							return sql_error(sql, 02, SQLSTATE(42000) "INSERT INTO: column '%s' has no valid default value", c->base.name);
 						if (exps) {
 							list *vals_list = exps->f;
 			
@@ -380,21 +389,21 @@ static sql_table *
 insert_allowed(mvc *sql, sql_table *t, char *tname, char *op, char *opname)
 {
 	if (!t) {
-		return sql_error(sql, 02, "42S02!%s: no such table '%s'", op, tname);
+		return sql_error(sql, 02, SQLSTATE(42S02) "%s: no such table '%s'", op, tname);
 	} else if (isView(t)) {
-		return sql_error(sql, 02, "%s: cannot %s view '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s view '%s'", op, opname, tname);
 	} else if (isMergeTable(t)) {
-		return sql_error(sql, 02, "%s: cannot %s merge table '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s merge table '%s'", op, opname, tname);
 	} else if (isStream(t)) {
-		return sql_error(sql, 02, "%s: cannot %s stream '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s stream '%s'", op, opname, tname);
 	} else if (t->access == TABLE_READONLY) {
-		return sql_error(sql, 02, "%s: cannot %s read only table '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s read only table '%s'", op, opname, tname);
 	}
 	if (t && !isTempTable(t) && STORE_READONLY)
-		return sql_error(sql, 02, "%s: %s table '%s' not allowed in readonly mode", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: %s table '%s' not allowed in readonly mode", op, opname, tname);
 
 	if (!table_privs(sql, t, PRIV_INSERT)) {
-		return sql_error(sql, 02, "%s: insufficient privileges for user '%s' to %s table '%s'", op, stack_get_string(sql, "current_user"), opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: insufficient privileges for user '%s' to %s table '%s'", op, stack_get_string(sql, "current_user"), opname, tname);
 	}
 	return t;
 }
@@ -411,20 +420,20 @@ static sql_table *
 update_allowed(mvc *sql, sql_table *t, char *tname, char *op, char *opname, int is_delete)
 {
 	if (!t) {
-		return sql_error(sql, 02, "42S02!%s: no such table '%s'", op, tname);
+		return sql_error(sql, 02, SQLSTATE(42S02) "%s: no such table '%s'", op, tname);
 	} else if (isView(t)) {
-		return sql_error(sql, 02, "%s: cannot %s view '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s view '%s'", op, opname, tname);
 	} else if (isMergeTable(t)) {
-		return sql_error(sql, 02, "%s: cannot %s merge table '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s merge table '%s'", op, opname, tname);
 	} else if (isStream(t)) {
-		return sql_error(sql, 02, "%s: cannot %s stream '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s stream '%s'", op, opname, tname);
 	} else if (t->access == TABLE_READONLY || t->access == TABLE_APPENDONLY) {
-		return sql_error(sql, 02, "%s: cannot %s read or append only table '%s'", op, opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: cannot %s read or append only table '%s'", op, opname, tname);
 	}
 	if (t && !isTempTable(t) && STORE_READONLY)
-		return sql_error(sql, 02, "%s: %s table '%s' not allowed in readonly mode", op, opname, tname);
-	if (is_delete && !table_privs(sql, t, PRIV_DELETE)) 
-		return sql_error(sql, 02, "%s: insufficient privileges for user '%s' to %s table '%s'", op, stack_get_string(sql, "current_user"), opname, tname);
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: %s table '%s' not allowed in readonly mode", op, opname, tname);
+	if ((is_delete == 1 && !table_privs(sql, t, PRIV_DELETE)) || (is_delete == 2 && !table_privs(sql, t, PRIV_TRUNCATE)))
+		return sql_error(sql, 02, SQLSTATE(42000) "%s: insufficient privileges for user '%s' to %s table '%s'", op, stack_get_string(sql, "current_user"), opname, tname);
 	return t;
 }
 
@@ -440,7 +449,7 @@ insert_into(mvc *sql, dlist *qname, dlist *columns, symbol *val_or_q)
 	sql_rel *r = NULL;
 
 	if (sname && !(s=mvc_bind_schema(sql, sname))) {
-		(void) sql_error(sql, 02, "3F000!INSERT INTO: no such schema '%s'", sname);
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "INSERT INTO: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!s)
@@ -474,7 +483,7 @@ insert_into(mvc *sql, dlist *qname, dlist *columns, symbol *val_or_q)
 			values = o->data.lval;
 
 			if (dlist_length(values) != list_length(collist)) {
-				return sql_error(sql, 02, "21S01!INSERT INTO: number of values doesn't match number of columns of table '%s'", tname);
+				return sql_error(sql, 02, SQLSTATE(21S01) "INSERT INTO: number of values doesn't match number of columns of table '%s'", tname);
 			} else {
 				dnode *n;
 				node *v, *m;
@@ -543,7 +552,7 @@ insert_into(mvc *sql, dlist *qname, dlist *columns, symbol *val_or_q)
 		r = rel_project(sql->sa, r, rel_projections(sql, r, NULL, 1, 0));
 	if ((r->exps && list_length(r->exps) != list_length(collist)) ||
 	   (!r->exps && collist)) 
-		return sql_error(sql, 02, "21S01!INSERT INTO: query result doesn't match number of columns in table '%s'", tname);
+		return sql_error(sql, 02, SQLSTATE(21S01) "INSERT INTO: query result doesn't match number of columns in table '%s'", tname);
 
 	r->exps = rel_inserts(sql, t, r, collist, rowcount, 0);
 	return rel_insert_table(sql, t, tname, r);
@@ -824,7 +833,7 @@ rel_update(mvc *sql, sql_rel *t, sql_rel *uprel, sql_exp **updates, list *exps)
 		if (tab->idxs.set && !v) 
 			v = exp_column(sql->sa, tab->base.name, c->base.name, &c->type, CARD_MULTI, c->null, 0);
 		if (v)
-			rel_project_add_exp(sql, uprel, v);
+			v = rel_project_add_exp(sql, uprel, v);
 	}
 
 	r->op = op_update;
@@ -843,10 +852,10 @@ update_check_column(mvc *sql, sql_table *t, sql_column *c, sql_exp *v, sql_rel *
 {
 	if (!c) {
 		rel_destroy(r);
-		return sql_error(sql, 02, "42S22!UPDATE: no such column '%s.%s'", t->base.name, cname);
+		return sql_error(sql, 02, SQLSTATE(42S22) "UPDATE: no such column '%s.%s'", t->base.name, cname);
 	}
 	if (!table_privs(sql, t, PRIV_UPDATE) && !sql_privilege(sql, sql->user_id, c->base.id, PRIV_UPDATE, 0)) 
-		return sql_error(sql, 02, "UPDATE: insufficient privileges for user '%s' to update table '%s' on column '%s'", stack_get_string(sql, "current_user"), t->base.name, cname);
+		return sql_error(sql, 02, SQLSTATE(42000) "UPDATE: insufficient privileges for user '%s' to update table '%s' on column '%s'", stack_get_string(sql, "current_user"), t->base.name, cname);
 	if (!v || (v = rel_check_type(sql, &c->type, v, type_equal)) == NULL) {
 		rel_destroy(r);
 		return NULL;
@@ -863,7 +872,7 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 	sql_table *t = NULL;
 
 	if (sname && !(s=mvc_bind_schema(sql,sname))) {
-		(void) sql_error(sql, 02, "3F000!UPDATE: no such schema '%s'", sname);
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "UPDATE: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!s)
@@ -927,7 +936,7 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 			sym = newSelectNode(sql->sa, 0, selection, NULL, symbol_create_list(sql->sa, SQL_FROM, from_list), opt_where, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 			sq = rel_selects(sql, sym);
 			if (sq)
-				sq = rel_optimizer(sql, sq);
+				sq = rel_optimizer(sql, sq, 0);
 		}
 #endif
 
@@ -950,7 +959,7 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 			int status = sql->session->status;
 	
 			if (!table_privs(sql, t, PRIV_SELECT)) 
-				return sql_error(sql, 02, "UPDATE: insufficient privileges for user '%s' to update table '%s'", stack_get_string(sql, "current_user"), tname);
+				return sql_error(sql, 02, SQLSTATE(42000) "UPDATE: insufficient privileges for user '%s' to update table '%s'", stack_get_string(sql, "current_user"), tname);
 			r = rel_logical_exp(sql, NULL, opt_where, sql_where);
 			if (r) { /* simple predicate which is not using the to 
 				    be updated table. We add a select all */
@@ -961,6 +970,10 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 				r = rel_logical_exp(sql, res, opt_where, sql_where);
 				if (!opt_from && r && is_join(r->op))
 					r->op = op_semi;
+				else if (r && res && r->nrcols != res->nrcols) {
+					list *exps = rel_projections(sql, res, NULL, 1, 1);
+					r = rel_project(sql->sa, r, exps);
+				}
 			}
 			if (!r) 
 				return NULL;
@@ -986,11 +999,19 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 				int status = sql->session->status;
 				exp_kind ek = {type_value, (single)?card_column:card_relation, FALSE};
 
-				if (single) 
+				if(single && a->token == SQL_DEFAULT) {
+					char *colname = assignment->h->next->data.sval;
+					sql_column *col = mvc_bind_column(sql, t, colname);
+					if (col->def) {
+						v = rel_parse_val(sql, sa_message(sql->sa, "select CAST(%s AS %s);", col->def, col->type.type->sqlname), sql->emode);
+					} else {
+						return sql_error(sql, 02, SQLSTATE(42000) "UPDATE: column '%s' has no valid default value", col->base.name);
+					}
+				} else if (single) {
 					v = rel_value_exp(sql, &rel_val, a, sql_sel, ek);
-				else
+				} else {
 					rel_val = rel_subquery(sql, NULL, a, ek, APPLY_JOIN);
-
+				}
 				if (!v) {
 					sql->errstr[0] = 0;
 					sql->session->status = status;
@@ -1017,7 +1038,8 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 						if (!exp_name(v))
 							exp_label(sql->sa, v, ++sql->label);
 						rel_val = rel_project(sql->sa, rel_val, rel_projections(sql, rel_val, NULL, 0, 1));
-						rel_project_add_exp(sql, rel_val, v);
+						v = rel_project_add_exp(sql, rel_val, v);
+						reset_processed(rel_val);
 					}
 					r = rel_crossproduct(sql->sa, r, rel_val, op_left);
 					if (single) 
@@ -1035,7 +1057,7 @@ update_table(mvc *sql, dlist *qname, dlist *assignmentlist, symbol *opt_from, sy
 				if (!rel_val || !is_project(rel_val->op) ||
 				    dlist_length(cols) > list_length(rel_val->exps)) {
 					rel_destroy(r);
-					return sql_error(sql, 02, "UPDATE: too many columns specified");
+					return sql_error(sql, 02, SQLSTATE(42000) "UPDATE: too many columns specified");
 				}
 				nr = (list_length(rel_val->exps)-dlist_length(cols));
 				for(n=rel_val->exps->h; nr; nr--, n = n->next)
@@ -1093,6 +1115,21 @@ rel_delete(sql_allocator *sa, sql_rel *t, sql_rel *deletes)
 	return r;
 }
 
+sql_rel *
+rel_truncate(sql_allocator *sa, sql_rel *t, int restart_sequences, int drop_action)
+{
+	sql_rel *r = rel_create(sa);
+	list *exps = new_exp_list(sa);
+
+	append(exps, exp_atom_int(sa, restart_sequences));
+	append(exps, exp_atom_int(sa, drop_action));
+	r->exps = exps;
+	r->op = op_truncate;
+	r->l = t;
+	r->r = NULL;
+	return r;
+}
+
 static sql_rel *
 delete_table(mvc *sql, dlist *qname, symbol *opt_where)
 {
@@ -1102,7 +1139,7 @@ delete_table(mvc *sql, dlist *qname, symbol *opt_where)
 	sql_table *t = NULL;
 
 	if (sname && !(schema=mvc_bind_schema(sql, sname))) {
-		(void) sql_error(sql, 02, "3F000!DELETE FROM: no such schema '%s'", sname);
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "DELETE FROM: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!schema)
@@ -1123,7 +1160,7 @@ delete_table(mvc *sql, dlist *qname, symbol *opt_where)
 			int status = sql->session->status;
 
 			if (!table_privs(sql, t, PRIV_SELECT)) 
-				return sql_error(sql, 02, "DELETE FROM: insufficient privileges for user '%s' to delete from table '%s'", stack_get_string(sql, "current_user"), tname);
+				return sql_error(sql, 02, SQLSTATE(42000) "DELETE FROM: insufficient privileges for user '%s' to delete from table '%s'", stack_get_string(sql, "current_user"), tname);
 
 			r = rel_logical_exp(sql, NULL, opt_where, sql_where);
 			if (r) { /* simple predicate which is not using the to 
@@ -1153,6 +1190,35 @@ delete_table(mvc *sql, dlist *qname, symbol *opt_where)
 	return NULL;
 }
 
+static sql_rel *
+truncate_table(mvc *sql, dlist *qname, int restart_sequences, int drop_action)
+{
+	char *sname = qname_schema(qname);
+	char *tname = qname_table(qname);
+	sql_schema *schema = NULL;
+	sql_table *t = NULL;
+
+	if (sname && !(schema=mvc_bind_schema(sql, sname))) {
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "TRUNCATE: no such schema '%s'", sname);
+		return NULL;
+	}
+	if (!schema)
+		schema = cur_schema(sql);
+	t = mvc_bind_table(sql, schema, tname);
+	if (!t && !sname) {
+		schema = tmp_schema(sql);
+		t = mvc_bind_table(sql, schema, tname);
+		if (!t)
+			t = mvc_bind_table(sql, NULL, tname);
+		if (!t)
+			t = stack_find_table(sql, tname);
+	}
+	if (update_allowed(sql, t, tname, "TRUNCATE", "truncate", 2) != NULL) {
+		return rel_truncate(sql->sa, rel_basetable(sql, t, tname), restart_sequences, drop_action);
+	}
+	return NULL;
+}
+
 static list *
 table_column_types(sql_allocator *sa, sql_table *t)
 {
@@ -1215,7 +1281,7 @@ rel_import(mvc *sql, sql_table *t, char *tsep, char *rsep, char *ssep, char *ns,
 			ncol++;
 		}
 		if(list_length(f->res) != ncol) {
-			(void) sql_error(sql, 02, "3F000!COPY INTO: fixed width import for %d columns but %d widths given.", list_length(f->res), ncol);
+			(void) sql_error(sql, 02, SQLSTATE(3F000) "COPY INTO: fixed width import for %d columns but %d widths given.", list_length(f->res), ncol);
 			return NULL;
 		}
 		*fwf_string_cur = '\0';
@@ -1263,7 +1329,7 @@ copyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, dlist *headers, d
 	assert(!nr_offset || nr_offset->h->type == type_lng);
 	assert(!nr_offset || nr_offset->h->next->type == type_lng);
 	if (sname && !(s=mvc_bind_schema(sql, sname))) {
-		(void) sql_error(sql, 02, "3F000!COPY INTO: no such schema '%s'", sname);
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "COPY INTO: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!s)
@@ -1280,23 +1346,23 @@ copyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, dlist *headers, d
 	/* Only the MONETDB user is allowed copy into with 
 	   a lock and only on tables without idx */
 	if (locked && !copy_allowed(sql, 1)) {
-		return sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+		return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 		    "COPY INTO from .. LOCKED requires database administrator rights");
 	}
 	if (locked && (!list_empty(t->idxs.set) || !list_empty(t->keys.set))) {
-		return sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+		return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 		    "COPY INTO from .. LOCKED requires tables without indices");
 	}
 	if (locked && has_snapshots(sql->session->tr)) {
-		return sql_error(sql, 02, "COPY INTO .. LOCKED: not allowed on snapshots");
+		return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO .. LOCKED: not allowed on snapshots");
 	}
 	if (locked && !sql->session->auto_commit) {
-		return sql_error(sql, 02, "COPY INTO .. LOCKED: only allowed in auto commit mode");
+		return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO .. LOCKED: only allowed in auto commit mode");
 	}
 	/* lock the store, for single user/transaction */
 	if (locked) { 
 		if (headers)
-			return sql_error(sql, 02, "COPY INTO .. LOCKED: not allowed with column lists");
+			return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO .. LOCKED: not allowed with column lists");
 		store_lock();
 		while (store_nr_active > 1) {
 			store_unlock();
@@ -1355,7 +1421,7 @@ copyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, dlist *headers, d
 		dnode *n = files->h;
 
 		if (!copy_allowed(sql, 1))
-			return sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+			return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 					"COPY INTO from file(s) requires database administrator rights, "
 					"use 'COPY INTO \"%s\" FROM STDIN' instead", tname);
 
@@ -1364,7 +1430,7 @@ copyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, dlist *headers, d
 			sql_rel *nrel;
 
 			if (fname && !MT_path_absolute(fname))
-				return sql_error(sql, 02, "COPY INTO: filename must "
+				return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: filename must "
 						"have absolute path: %s", fname);
 
 			nrel = rel_import(sql, nt, tsep, rsep, ssep, ns, fname, nr, offset, locked, best_effort, fwf_widths);
@@ -1409,7 +1475,7 @@ copyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, dlist *headers, d
 				sql_find_subtype(&st, "clob", 0, 0);
 				f = sql_bind_func_result(sql->sa, sys, fname, &st, &st, &cs->type); 
 				if (!f)
-					return sql_error(sql, 02, "COPY INTO: '%s' missing for type %s", fname, cs->type.type->sqlname);
+					return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: '%s' missing for type %s", fname, cs->type.type->sqlname);
 				append(args, e);
 				append(args, exp_atom_clob(sql->sa, format));
 				ne = exp_op(sql->sa, args, f);
@@ -1459,13 +1525,13 @@ bincopyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, int constraint
 
 	assert(f);
 	if (!copy_allowed(sql, 1)) {
-		(void) sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+		(void) sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 				"binary COPY INTO requires database administrator rights");
 		return NULL;
 	}
 
 	if (sname && !(s=mvc_bind_schema(sql, sname))) {
-		(void) sql_error(sql, 02, "3F000!COPY INTO: no such schema '%s'", sname);
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "COPY INTO: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!s)
@@ -1480,7 +1546,7 @@ bincopyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, int constraint
 	if (insert_allowed(sql, t, tname, "COPY INTO", "copy into") == NULL) 
 		return NULL;
 	if (files == NULL)
-		return sql_error(sql, 02, "COPY INTO: must specify files");
+		return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: must specify files");
 
 	collist = check_table_columns(sql, t, columns, "COPY BINARY", tname);
 	if (!collist)
@@ -1527,31 +1593,23 @@ bincopyfrom(mvc *sql, dlist *qname, dlist *columns, dlist *files, int constraint
 	return res;
 }
 
-
 static sql_rel *
 copyfromloader(mvc *sql, dlist *qname, symbol *fcall)
 {
+	sql_schema *s = NULL;
 	char *sname = qname_schema(qname);
 	char *tname = qname_table(qname);
-
-	sql_schema *s = NULL;
-	sql_table *t = NULL;
-
-	node *n;
-	sql_rel res_obj ;
-	sql_rel *res = &res_obj;
-	list *exps = new_exp_list(sql->sa); //, *args = NULL;
-	sql_exp *import;
-	exp_kind ek = {type_value, card_loader, FALSE};
+	sql_subfunc *loader = NULL;
+	sql_rel* rel = NULL;
+	sql_table* t;
 
 	if (!copy_allowed(sql, 1)) {
-		(void) sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+		(void) sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 				"binary COPY INTO requires database administrator rights");
 		return NULL;
 	}
-
-	if (sname && !(s=mvc_bind_schema(sql, sname))) {
-		(void) sql_error(sql, 02, "3F000!COPY INTO: no such schema '%s'", sname);
+	if (sname && !(s = mvc_bind_schema(sql, sname))) {
+		(void) sql_error(sql, 02, SQLSTATE(3F000) "COPY INTO: no such schema '%s'", sname);
 		return NULL;
 	}
 	if (!s)
@@ -1567,20 +1625,20 @@ copyfromloader(mvc *sql, dlist *qname, symbol *fcall)
 		return NULL;
 	}
 
-	import = rel_value_exp(sql, &res, fcall, sql_sel, ek);
-	if (!import) {
+	rel = rel_loader_function(sql, fcall, new_exp_list(sql->sa), &loader);
+	if (!rel || !loader) {
 		return NULL;
 	}
-	((sql_subfunc*) import->f)->res = table_column_types(sql->sa, t);
-	((sql_subfunc*) import->f)->colnames = table_column_names_and_defaults(sql->sa, t);
 
-	for (n = t->columns.set->h; n; n = n->next) {
-		sql_column *c = n->data;
-		append(exps, exp_column(sql->sa, t->base.name, c->base.name, &c->type, CARD_MULTI, c->null, 0));
-	}
+	loader->sname = sname ? sa_zalloc(sql->sa, strlen(sname) + 1) : NULL;
+	loader->tname = tname ? sa_zalloc(sql->sa, strlen(tname) + 1) : NULL;
+	loader->coltypes = table_column_types(sql->sa, t);
+	loader->colnames = table_column_names_and_defaults(sql->sa, t);
 
-	res = rel_table_func(sql->sa, NULL, import, exps, 1);
-	return  rel_insert_table(sql, t, t->base.name, res);
+	if (sname) strcpy(loader->sname, sname);
+	if (tname) strcpy(loader->tname, tname);
+
+	return rel;
 }
 
 
@@ -1631,14 +1689,14 @@ copyto(mvc *sql, symbol *sq, str filename, dlist *seps, str null_string)
 	if (filename) {
 		struct stat fs;
 		if (!copy_allowed(sql, 0)) 
-			return sql_error(sql, 02, "COPY INTO: insufficient privileges: "
+			return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: insufficient privileges: "
 					"COPY INTO file requires database administrator rights, "
 					"use 'COPY ... INTO STDOUT' instead");
 		if (filename && !MT_path_absolute(filename))
-			return sql_error(sql, 02, "COPY INTO: filename must "
+			return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: filename must "
 					"have absolute path: %s", filename);
 		if (stat(filename, &fs) == 0)
-			return sql_error(sql, 02, "COPY INTO: file already "
+			return sql_error(sql, 02, SQLSTATE(42000) "COPY INTO: file already "
 					"exists: %s", filename);
 	}
 
@@ -1722,7 +1780,10 @@ rel_parse_val(mvc *m, char *query, char emode)
 		m->session->status = status;
 		strcpy(m->errstr, errstr);
 	} else {
+		int label = m->label;
 		*m = o;
+
+		m->label = label;
 	}
 	return e;
 }
@@ -1764,10 +1825,12 @@ rel_updates(mvc *sql, symbol *s)
 		break;
 	case SQL_COPYLOADER:
 	{
-		dlist *l = s->data.lval;
+	    dlist *l = s->data.lval;
+	    dlist *qname = l->h->data.lval;
+	    symbol *sym = l->h->next->data.sym;
 
-		ret = copyfromloader(sql, l->h->data.lval, l->h->next->data.sym);
-		sql->type = Q_UPDATE;
+	    ret = rel_psm_stmt(sql->sa, exp_rel(sql, copyfromloader(sql, qname, sym)));
+	    sql->type = Q_SCHEMA;
 	}
 		break;
 	case SQL_COPYTO:
@@ -1802,9 +1865,19 @@ rel_updates(mvc *sql, symbol *s)
 		sql->type = Q_UPDATE;
 	}
 		break;
+	case SQL_TRUNCATE:
+	{
+		dlist *l = s->data.lval;
+
+		int restart_sequences = l->h->next->data.i_val;
+		int drop_action = l->h->next->next->data.i_val;
+		ret = truncate_table(sql, l->h->data.lval, restart_sequences, drop_action);
+		sql->type = Q_UPDATE;
+	}
+		break;
 	default:
 		sql->use_views = old;
-		return sql_error(sql, 01, "Updates statement unknown Symbol(" PTRFMT ")->token = %s", PTRFMTCAST s, token2string(s->token));
+		return sql_error(sql, 01, SQLSTATE(42000) "Updates statement unknown Symbol(%p)->token = %s", (void*) s, token2string(s->token));
 	}
 	sql->use_views = old;
 	return ret;
