@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /* multi version catalog */
@@ -23,6 +23,23 @@
 
 static int mvc_debug = 0;
 
+static void
+sql_create_comments(mvc *m, sql_schema *s)
+{
+	sql_table *t;
+	sql_column *c;
+	sql_key *k;
+
+	t = mvc_create_table(m, s, "comments", tt_table, 1, SQL_PERSIST, 0, -1);
+	c = mvc_create_column_(m, t, "id", "int", 32);
+	k = sql_trans_create_ukey(m->session->tr, t, "comments_id_pkey", pkey);
+	k = sql_trans_create_kc(m->session->tr, k, c);
+	k = sql_trans_key_done(m->session->tr, k);
+	sql_trans_create_dependency(m->session->tr, c->base.id, k->idx->base.id, INDEX_DEPENDENCY);
+	c = mvc_create_column_(m, t, "remark", "varchar", 65000);
+	sql_trans_alter_null(m->session->tr, c, 0);
+}
+
 int
 mvc_init(int debug, store_type store, int ro, int su, backend_stack stk)
 {
@@ -87,7 +104,11 @@ mvc_init(int debug, store_type store, int ro, int su, backend_stack stk)
 		m->history = 0;
 		/* disable size header */
 		m->sizeheader = 0;
-		mvc_trans(m);
+		if(mvc_trans(m) < 0) {
+			mvc_destroy(m);
+			fprintf(stderr, "!mvc_init: failed to start transaction\n");
+			return -1;
+		}
 		s = m->session->schema = mvc_bind_schema(m, "sys");
 		assert(m->session->schema != NULL);
 
@@ -120,11 +141,14 @@ mvc_init(int debug, store_type store, int ro, int su, backend_stack stk)
 			sql_table *deps = find_sql_table(s, "dependencies");
 			sql_column *depids = find_sql_column(deps, "id");
 			oid rid;
+			rids *rs;
 
 			table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
-			while ((rid = table_funcs.column_find_row(m->session->tr, depids, &tid, NULL)) != oid_nil) {
+			rs = table_funcs.rids_select(m->session->tr, depids, &tid, &tid, NULL);
+			while ((rid = table_funcs.rids_next(rs)), !is_oid_nil(rid)) {
 				table_funcs.column_update_value(m->session->tr, depids, rid, &ntid);
 			}
+			table_funcs.rids_destroy(rs);
 		}
 
 		t = mvc_create_view(m, s, "columns", SQL_PERSIST, "SELECT * FROM (SELECT p.* FROM \"sys\".\"_columns\" AS p UNION ALL SELECT t.* FROM \"tmp\".\"_columns\" AS t) AS columns;", 1);
@@ -148,13 +172,17 @@ mvc_init(int debug, store_type store, int ro, int su, backend_stack stk)
 			sql_table *deps = find_sql_table(s, "dependencies");
 			sql_column *depids = find_sql_column(deps, "id");
 			oid rid;
+			rids *rs;
 
 			table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
-			while ((rid = table_funcs.column_find_row(m->session->tr, depids, &cid, NULL)) != oid_nil) {
+			rs = table_funcs.rids_select(m->session->tr, depids, &cid, &cid, NULL);
+			while ((rid = table_funcs.rids_next(rs)), !is_oid_nil(rid)) {
 				table_funcs.column_update_value(m->session->tr, depids, rid, &ncid);
 			}
-		} else { 
+			table_funcs.rids_destroy(rs);
+		} else {
 			sql_create_env(m, s);
+			sql_create_comments(m, s);
 			sql_create_privileges(m, s);
 		}
 
@@ -227,6 +255,14 @@ mvc_debug_on(mvc *m, int flg)
 }
 
 void
+mvc_cancel_session(mvc *m)
+{
+	store_lock();
+	sql_trans_end(m->session);
+	store_unlock();
+}
+
+int
 mvc_trans(mvc *m)
 {
 	int schema_changed = 0, err = m->session->status;
@@ -240,11 +276,17 @@ mvc_trans(mvc *m)
 			if (m->qc)
 				qc_destroy(m->qc);
 			m->qc = qc_create(m->clientid, seqnr);
+			if (!m->qc) {
+				sql_trans_end(m->session);
+				store_unlock();
+				return -1;
+			}
 		} else { /* clean all but the prepared statements */
 			qc_clean(m->qc);
 		}
 	}
 	store_unlock();
+	return 0;
 }
 
 static sql_trans *
@@ -301,12 +343,12 @@ mvc_commit(mvc *m, int chain, const char *name)
 
 	assert(tr);
 	assert(m->session->active);	/* only commit an active transaction */
-	
+
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_commit %s\n", (name) ? name : "");
 
 	if (m->session->status < 0) {
-		(void)sql_error(m, 010, "40000!COMMIT: transaction is aborted, will ROLLBACK instead");
+		(void)sql_error(m, 010, SQLSTATE(40000) "COMMIT: transaction is aborted, will ROLLBACK instead");
 		mvc_rollback(m, chain, name);
 		return -1;
 	}
@@ -318,6 +360,12 @@ mvc_commit(mvc *m, int chain, const char *name)
 			fprintf(stderr, "#mvc_savepoint\n");
 		store_lock();
 		m->session->tr = sql_trans_create(m->session->stk, tr, name);
+		if(!m->session->tr) {
+			store_unlock();
+			(void) sql_error(m, 02, SQLSTATE(HY001) "Allocation failure while committing the transaction, will ROLLBACK instead");
+			mvc_rollback(m, chain, name);
+			return -1;
+		}
 		store_unlock();
 		m->type = Q_TRANS;
 		if (m->qc) /* clean query cache, protect against concurrent access on the hash tables (when functions already exists, concurrent mal will
@@ -353,9 +401,9 @@ build up the hash (not copied in the trans dup)) */
 		if (!chain) 
 			sql_trans_end(m->session);
 		m->type = Q_TRANS;
+		store_unlock();
 		if (mvc_debug)
 			fprintf(stderr, "#mvc_commit %s done\n", (name) ? name : "");
-		store_unlock();
 		return 0;
 	}
 
@@ -365,7 +413,7 @@ build up the hash (not copied in the trans dup)) */
 		MT_sleep_ms(100);
 		wait += 100;
 		if (wait > 1000) {
-			(void)sql_error(m, 010, "40000!COMMIT: transaction is aborted because of DDL concurrency conflicts, will ROLLBACK instead");
+			(void)sql_error(m, 010, SQLSTATE(40000) "COMMIT: transaction is aborted because of DDL concurrency conflicts, will ROLLBACK instead");
 			mvc_rollback(m, chain, name);
 			return -1;
 		}
@@ -375,13 +423,13 @@ build up the hash (not copied in the trans dup)) */
 	/* validation phase */
 	if (sql_trans_validate(tr)) {
 		if ((ok = sql_trans_commit(tr)) != SQL_OK) {
-			char *msg = sql_message("40000!COMMIT: transaction commit failed (perhaps your disk is full?) exiting (kernel error: %s)", GDKerrbuf);
+			char *msg = sql_message(SQLSTATE(40000) "COMMIT: transaction commit failed (perhaps your disk is full?) exiting (kernel error: %s)", GDKerrbuf);
 			GDKfatal("%s", msg);
 			_DELETE(msg);
 		}
 	} else {
 		store_unlock();
-		(void)sql_error(m, 010, "40000!COMMIT: transaction is aborted because of concurrency conflicts, will ROLLBACK instead");
+		(void)sql_error(m, 010, SQLSTATE(40000) "COMMIT: transaction is aborted because of concurrency conflicts, will ROLLBACK instead");
 		mvc_rollback(m, chain, name);
 		return -1;
 	}
@@ -414,7 +462,7 @@ mvc_rollback(mvc *m, int chain, const char *name)
 		while (tr && (!tr->name || strcmp(tr->name, name) != 0))
 			tr = tr->parent;
 		if (!tr) {
-			(void)sql_error(m, 010, "ROLLBACK: no such savepoint: '%s'", name);
+			(void)sql_error(m, 010, SQLSTATE(42000) "ROLLBACK: no such savepoint: '%s'", name);
 			m->session->status = -1;
 			store_unlock();
 			return -1;
@@ -473,7 +521,7 @@ mvc_release(mvc *m, const char *name)
 	while (tr && (!tr->name || strcmp(tr->name, name) != 0))
 		tr = tr->parent;
 	if (!tr || !tr->name || strcmp(tr->name, name) != 0) {
-		(void)sql_error(m, 010, "release savepoint %s doesn't exists", name);
+		(void)sql_error(m, 010, SQLSTATE(42000) "Release savepoint %s doesn't exists", name);
 		m->session->status = -1;
 		return -1;
 	}
@@ -550,6 +598,7 @@ mvc_create(int clientid, backend_stack stk, int debug, bstream *rs, stream *ws)
 	m->history = 0;
 
 	m->label = 0;
+	m->remote = 0;
 	m->cascade_action = NULL;
 	for(i=0;i<MAXSTATS;i++)
 		m->opt_stats[i] = 0;
@@ -575,10 +624,10 @@ mvc_create(int clientid, backend_stack stk, int debug, bstream *rs, stream *ws)
 	return m;
 }
 
-void
+int
 mvc_reset(mvc *m, bstream *rs, stream *ws, int debug, int globalvars)
 {
-	int i;
+	int i, res = 1;
 	sql_trans *tr;
 
 	if (mvc_debug)
@@ -591,13 +640,15 @@ mvc_reset(mvc *m, bstream *rs, stream *ws, int debug, int globalvars)
 			tr = sql_trans_destroy(tr);
 		store_unlock();
 	}
-	if (tr)
-		sql_session_reset(m->session, 1 /*autocommit on*/);
+	if (tr && !sql_session_reset(m->session, 1 /*autocommit on*/))
+		res = 0;
 
 	if (m->sa)
 		m->sa = sa_reset(m->sa);
-	else 
+	else
 		m->sa = sa_create();
+	if(!m->sa)
+		res = 0;
 
 	m->errstr[0] = '\0';
 
@@ -629,6 +680,7 @@ mvc_reset(mvc *m, bstream *rs, stream *ws, int debug, int globalvars)
 	m->history = 0;
 
 	m->label = 0;
+	m->remote = 0;
 	m->cascade_action = NULL;
 	m->type = Q_PARSE;
 	m->pushdown = 1;
@@ -640,6 +692,7 @@ mvc_reset(mvc *m, bstream *rs, stream *ws, int debug, int globalvars)
 	m->results = NULL;
 
 	scanner_init(&m->scanner, rs, ws);
+	return res;
 }
 
 void
@@ -757,7 +810,7 @@ mvc_bind_table(mvc *m, sql_schema *s, const char *tname)
 		if (tpe) {
 			t = tpe;
 		} else { /* during exection they are in the declared table schema */
-			s = mvc_bind_schema(m, dt_schema);
+				s = mvc_bind_schema(m, dt_schema);
 			return mvc_bind_table(m, s, tname);
 		}
 	} else {
@@ -936,22 +989,22 @@ mvc_create_func(mvc *sql, sql_allocator *sa, sql_schema * s, const char *name, l
 	return f;
 }
 
-void
+int
 mvc_drop_func(mvc *m, sql_schema *s, sql_func *f, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_func %s %s\n", s->base.name, f->base.name);
 
-	sql_trans_drop_func(m->session->tr, s, f->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+	return sql_trans_drop_func(m->session->tr, s, f->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
-void
+int
 mvc_drop_all_func(mvc *m, sql_schema *s, list *list_func, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_all_func %s %s\n", s->base.name, ((sql_func *) list_func->h->data)->base.name);
 
-	sql_trans_drop_all_func(m->session->tr, s, list_func, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+	return sql_trans_drop_all_func(m->session->tr, s, list_func, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
 sql_schema *
@@ -966,12 +1019,12 @@ mvc_create_schema(mvc *m, const char *name, int auth_id, int owner)
 	return s;
 }
 
-void
+int
 mvc_drop_schema(mvc *m, sql_schema * s, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_schema %s\n", s->base.name);
-	sql_trans_drop_schema(m->session->tr, s->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+	return sql_trans_drop_schema(m->session->tr, s->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
 sql_ukey *
@@ -998,7 +1051,7 @@ sql_fkey *
 mvc_create_fkey(mvc *m, sql_table *t, const char *name, key_type kt, sql_key *rkey, int on_delete, int on_update)
 {
 	if (mvc_debug)
-		fprintf(stderr, "#mvc_create_fkey %s %u " PTRFMT "\n", t->base.name, kt, PTRFMTCAST rkey);
+		fprintf(stderr, "#mvc_create_fkey %s %u %p\n", t->base.name, kt, (void*) rkey);
 	if (t->persistence == SQL_DECLARED_TABLE)
 		return create_sql_fkey(m->sa, t, name, kt, rkey, on_delete, on_update);	
 	else
@@ -1025,15 +1078,16 @@ mvc_create_fkc(mvc *m, sql_fkey *fk, sql_column *c)
 		return sql_trans_create_fkc(m->session->tr, fk, c);
 }
 
-void
+int
 mvc_drop_key(mvc *m, sql_schema *s, sql_key *k, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_key %s %s\n", s->base.name, k->base.name);
-	if (k->t->persistence == SQL_DECLARED_TABLE)
+	if (k->t->persistence == SQL_DECLARED_TABLE) {
 		drop_sql_key(k->t, k->base.id, drop_action);
-	else
-		sql_trans_drop_key(m->session->tr, s, k->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+		return 0;
+	} else
+		return sql_trans_drop_key(m->session->tr, s, k->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
 sql_idx *
@@ -1062,17 +1116,18 @@ mvc_create_ic(mvc *m, sql_idx * i, sql_column *c)
 		return sql_trans_create_ic(m->session->tr, i, c);
 }
 
-void
+int
 mvc_drop_idx(mvc *m, sql_schema *s, sql_idx *i)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_idx %s %s\n", s->base.name, i->base.name);
 
-	if (i->t->persistence == SQL_DECLARED_TABLE)
+	if (i->t->persistence == SQL_DECLARED_TABLE) {
 		/* declared tables should not end up in the catalog */
 		drop_sql_idx(i->t, i->base.id);
-	else
-		sql_trans_drop_idx(m->session->tr, s, i->base.id, DROP_RESTRICT);
+		return 0;
+	} else
+		return sql_trans_drop_idx(m->session->tr, s, i->base.id, DROP_RESTRICT);
 }
 
 sql_trigger * 
@@ -1095,13 +1150,13 @@ mvc_create_tc(mvc *m, sql_trigger * i, sql_column *c /*, extra options such as t
 	return i;
 }
 
-void
+int
 mvc_drop_trigger(mvc *m, sql_schema *s, sql_trigger *tri)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_trigger %s %s\n", s->base.name, tri->base.name);
 
-	sql_trans_drop_trigger(m->session->tr, s, tri->base.id, DROP_RESTRICT);
+	return sql_trans_drop_trigger(m->session->tr, s, tri->base.id, DROP_RESTRICT);
 }
 
 
@@ -1158,13 +1213,13 @@ mvc_create_remote(mvc *m, sql_schema *s, const char *name, int persistence, cons
 	return t;
 }
 
-void
+int
 mvc_drop_table(mvc *m, sql_schema *s, sql_table *t, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_table %s %s\n", s->base.name, t->base.name);
 
-	sql_trans_drop_table(m->session->tr, s, t->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+	return sql_trans_drop_table(m->session->tr, s, t->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
 BUN
@@ -1196,15 +1251,16 @@ mvc_create_column(mvc *m, sql_table *t, const char *name, sql_subtype *tpe)
 		return sql_trans_create_column(m->session->tr, t, name, tpe);
 }
 
-void
+int
 mvc_drop_column(mvc *m, sql_table *t, sql_column *col, int drop_action)
 {
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_drop_column %s %s\n", t->base.name, col->base.name);
-	if (col->t->persistence == SQL_DECLARED_TABLE)
+	if (col->t->persistence == SQL_DECLARED_TABLE) {
 		drop_sql_column(t, col->base.id, drop_action);
-	else
-		sql_trans_drop_column(m->session->tr, t, col->base.id,  drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
+		return 0;
+	} else
+		return sql_trans_drop_column(m->session->tr, t, col->base.id, drop_action ? DROP_CASCADE_START : DROP_RESTRICT);
 }
 
 void
@@ -1238,11 +1294,10 @@ int
 mvc_check_dependency(mvc * m, int id, int type, list *ignore_ids)
 {
 	list *dep_list = NULL;
-	
+
 	if (mvc_debug)
 		fprintf(stderr, "#mvc_check_dependency on %d\n", id);
 
-
 	switch(type) {
 		case OWNER_DEPENDENCY : 
 			dep_list = sql_trans_owner_schema_dependencies(m->session->tr, id);
@@ -1263,12 +1318,15 @@ mvc_check_dependency(mvc * m, int id, int type, list *ignore_ids)
 		default: 
 			dep_list =  sql_trans_get_dependencies(m->session->tr, id, COLUMN_DEPENDENCY, NULL);
 	}
-	
+
+	if(!dep_list)
+		return DEPENDENCY_CHECK_ERROR;
+
 	if ( list_length(dep_list) >= 2 ) {
 		list_destroy(dep_list);
 		return HAS_DEPENDENCY;
 	}
-	
+
 	list_destroy(dep_list);
 	return NO_DEPENDENCY;
 }
@@ -1351,13 +1409,20 @@ mvc_is_sorted(mvc *m, sql_column *col)
 }
 
 /* variable management */
-static void
+static sql_var*
 stack_set(mvc *sql, int var, const char *name, sql_subtype *type, sql_rel *rel, sql_table *t, int view, int frame)
 {
-	sql_var *v;
-	if (var == sql->sizevars) {
-		sql->sizevars <<= 1;
-		sql->vars = RENEW_ARRAY(sql_var,sql->vars,sql->sizevars);
+	sql_var *v, *nvars;
+	int nextsize = sql->sizevars;
+	if (var == nextsize) {
+		nextsize <<= 1;
+		nvars = RENEW_ARRAY(sql_var,sql->vars,nextsize);
+		if(!nvars) {
+			return NULL;
+		} else {
+			sql->vars = nvars;
+			sql->sizevars = nextsize;
+		}
 	}
 	v = sql->vars+var;
 
@@ -1372,52 +1437,70 @@ stack_set(mvc *sql, int var, const char *name, sql_subtype *type, sql_rel *rel,
 		VALset(&sql->vars[var].a.data, tpe, (ptr) ATOMnilptr(tpe));
 		v->a.tpe = *type;
 	}
-	if (name)
+	if (name) {
 		v->name = _STRDUP(name);
+		if(!v->name)
+			return NULL;
+	}
+	return v;
 }
 
-void 
+sql_var*
 stack_push_var(mvc *sql, const char *name, sql_subtype *type)
 {
-	stack_set(sql, sql->topvars++, name, type, NULL, NULL, 0, 0);
+	sql_var* res = stack_set(sql, sql->topvars, name, type, NULL, NULL, 0, 0);
+	if(res)
+		sql->topvars++;
+	return res;
 }
 
-void 
+sql_var*
 stack_push_rel_var(mvc *sql, const char *name, sql_rel *var, sql_subtype *type)
 {
-	stack_set(sql, sql->topvars++, name, type, var, NULL, 0, 0);
+	sql_var* res = stack_set(sql, sql->topvars, name, type, var, NULL, 0, 0);
+	if(res)
+		sql->topvars++;
+	return res;
 }
 
-void 
+sql_var*
 stack_push_table(mvc *sql, const char *name, sql_rel *var, sql_table *t)
 {
-	stack_set(sql, sql->topvars++, name, NULL, var, t, 0, 0);
+	sql_var* res = stack_set(sql, sql->topvars, name, NULL, var, t, 0, 0);
+	if(res)
+		sql->topvars++;
+	return res;
 }
 
-
-void 
+sql_var*
 stack_push_rel_view(mvc *sql, const char *name, sql_rel *var)
 {
-	stack_set(sql, sql->topvars++, name, NULL, var, NULL, 1, 0);
+	sql_var* res = stack_set(sql, sql->topvars, name, NULL, var, NULL, 1, 0);
+	if(res)
+		sql->topvars++;
+	return res;
 }
 
-
-void
+atom *
 stack_set_var(mvc *sql, const char *name, ValRecord *v)
 {
 	int i;
+	atom *res = NULL;
 
 	for (i = sql->topvars-1; i >= 0; i--) {
 		if (!sql->vars[i].frame && strcmp(sql->vars[i].name, name)==0) {
 			VALclear(&sql->vars[i].a.data);
-			VALcopy(&sql->vars[i].a.data, v);
+			if(VALcopy(&sql->vars[i].a.data, v) == NULL)
+				return NULL;
 			sql->vars[i].a.isnull = VALisnil(v);
 			if (v->vtype == TYPE_flt)
 				sql->vars[i].a.d = v->val.fval;
 			else if (v->vtype == TYPE_dbl)
 				sql->vars[i].a.d = v->val.dval;
+			res = &sql->vars[i].a;
 		}
 	}
+	return res;
 }
 
 atom *
@@ -1433,11 +1516,15 @@ stack_get_var(mvc *sql, const char *name)
 	return NULL;
 }
 
-void 
+sql_var*
 stack_push_frame(mvc *sql, const char *name)
 {
-	stack_set(sql, sql->topvars++, name, NULL, NULL, NULL, 0, 1);
-	sql->frame++;
+	sql_var* res = stack_set(sql, sql->topvars, name, NULL, NULL, NULL, 0, 1);
+	if(res) {
+		sql->topvars++;
+		sql->frame++;
+	}
+	return res;
 }
 
 void
@@ -1604,18 +1691,23 @@ stack_nr_of_declared_tables(mvc *sql)
 	return dt;
 }
 
-void
+str
 stack_set_string(mvc *sql, const char *name, const char *val)
 {
 	atom *a = stack_get_var(sql, name);
+	str new_val = _STRDUP(val);
 
-	if (a != NULL) {
+	if (a != NULL && new_val != NULL) {
 		ValRecord *v = &a->data;
 
 		if (v->val.sval)
 			_DELETE(v->val.sval);
-		v->val.sval = _STRDUP(val);
+		v->val.sval = new_val;
+		return new_val;
+	} else if(new_val) {
+		_DELETE(new_val);
 	}
+	return NULL;
 }
 
 str
@@ -1717,25 +1809,26 @@ mvc_copy_idx(mvc *m, sql_table *t, sql_idx *i)
 	return sql_trans_copy_idx(m->session->tr, t, i);
 }
 
-sql_rel *
+sql_subquery *
 mvc_push_subquery(mvc *m, const char *name, sql_rel *r)
 {
-	sql_rel *res = NULL;
+	sql_subquery *res = NULL;
 
 	if (!m->sqs)
 		m->sqs = sa_list(m->sa);
 	if (m->sqs) {
-		sql_var *v = SA_NEW(m->sa, sql_var);
+		sql_subquery *v = SA_NEW(m->sa, sql_subquery);
 
 		v->name = name;
 		v->rel = r;
+		v->s = NULL;
 		list_append(m->sqs, v);
-		res = r;
+		res = v;
 	}
 	return res;
 }
 
-sql_rel *
+sql_subquery *
 mvc_find_subquery(mvc *m, const char *rname, const char *name) 
 {
 	node *n;
@@ -1743,13 +1836,13 @@ mvc_find_subquery(mvc *m, const char *rname, const char *name)
 	if (!m->sqs)
 		return NULL;
 	for (n = m->sqs->h; n; n = n->next) {
-		sql_var *v = n->data;
+		sql_subquery *v = n->data;
 
 		if (strcmp(v->name, rname) == 0) {
 			sql_exp *ne = exps_bind_column2(v->rel->exps, rname, name);
 
 			if (ne)
-				return v->rel;
+				return v;
 		}
 	}
 	return NULL;
@@ -1763,7 +1856,7 @@ mvc_find_subexp(mvc *m, const char *rname, const char *name)
 	if (!m->sqs)
 		return NULL;
 	for (n = m->sqs->h; n; n = n->next) {
-		sql_var *v = n->data;
+		sql_subquery *v = n->data;
 
 		if (strcmp(v->name, rname) == 0) {
 			sql_exp *ne = exps_bind_column2(v->rel->exps, rname, name);
