@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 /*
@@ -22,6 +22,7 @@
 #include "sql_privileges.h"
 #include "sql_semantic.h"
 #include "sql_parser.h"
+#include "mal_exception.h"
 
 #define PRIV_ROLE_ADMIN 0
 
@@ -39,6 +40,8 @@ priv2string(int priv)
 		return "INSERT";
 	case PRIV_DELETE:
 		return "DELETE";
+	case PRIV_TRUNCATE:
+		return "TRUNCATE";
 	case PRIV_EXECUTE:
 		return "EXECUTE";
 	}
@@ -61,6 +64,7 @@ sql_insert_all_privs(mvc *sql, int auth_id, int obj_id, int grantor, int grantab
 	sql_insert_priv(sql, auth_id, obj_id, PRIV_UPDATE, grantor, grantable);
 	sql_insert_priv(sql, auth_id, obj_id, PRIV_INSERT, grantor, grantable);
 	sql_insert_priv(sql, auth_id, obj_id, PRIV_DELETE, grantor, grantable);
+	sql_insert_priv(sql, auth_id, obj_id, PRIV_TRUNCATE, grantor, grantable);
 }
 
 static int
@@ -97,7 +101,7 @@ schema_privs(int grantor, sql_schema *s)
 }
 
 
-char *
+str
 sql_grant_global_privs( mvc *sql, char *grantee, int privs, int grant, int grantor)
 {
 	sql_trans *tr = sql->session->tr;
@@ -109,17 +113,17 @@ sql_grant_global_privs( mvc *sql, char *grantee, int privs, int grant, int grant
 		allowed = sql_grantable(sql, grantor, GLOBAL_OBJID, privs, 0);
 
 	if (!allowed) 
-		return sql_message("0L000!GRANT: grantor '%s' is not allowed to grant global privileges", stack_get_string(sql,"current_user"));
+		throw(SQL,"sql.grant_global",SQLSTATE(0L000) "Grantor '%s' is not allowed to grant global privileges", stack_get_string(sql,"current_user"));
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!GRANT: user/role '%s' unknown", grantee);
+		throw(SQL,"sql.grant_global",SQLSTATE(42M32) "User/role '%s' unknown", grantee);
 	/* first check if privilege isn't already given */
 	if ((sql_privilege(sql, grantee_id, GLOBAL_OBJID, privs, 0))) 
-		return sql_message("42M32!GRANT: user/role '%s' already has this privilege", grantee);
+		throw(SQL,"sql.grant_global",SQLSTATE(42M32) "User/role '%s' already has this privilege", grantee);
 	sql_insert_priv(sql, grantee_id, GLOBAL_OBJID, privs, grantor, grant);
 	tr->schema_updates++;
-	return NULL;
+	return MAL_SUCCEED;
 }
 
 char *
@@ -130,14 +134,14 @@ sql_grant_table_privs( mvc *sql, char *grantee, int privs, char *sname, char *tn
 	sql_table *t = NULL;
 	sql_column *c = NULL;
 	int allowed, grantee_id;
-	int all = PRIV_SELECT | PRIV_UPDATE | PRIV_INSERT | PRIV_DELETE;
+	int all = PRIV_SELECT | PRIV_UPDATE | PRIV_INSERT | PRIV_DELETE | PRIV_TRUNCATE;
 
  	if (sname)
 		s = mvc_bind_schema(sql, sname);
 	if (s)
  		t = mvc_bind_table(sql, s, tname);
 	if (!t) 
-		return sql_message("42S02!GRANT no such table '%s'", tname);
+		throw(SQL,"sql.grant_table",SQLSTATE(42S02) "GRANT no such table '%s'", tname);
 
 	allowed = schema_privs(grantor, t->s);
 
@@ -146,32 +150,33 @@ sql_grant_table_privs( mvc *sql, char *grantee, int privs, char *sname, char *tn
 			allowed = sql_grantable(sql, grantor, t->base.id, privs, 0);
 
 		if (!allowed) 
-			return sql_message("0L000!GRANT: grantor '%s' is not allowed to grant privileges for table '%s'", stack_get_string(sql,"current_user"), tname);
+			throw(SQL,"sql.grant_table", SQLSTATE(0L000) "Grantor '%s' is not allowed to grant privileges for table '%s'", stack_get_string(sql,"current_user"), tname);
 	}
 	if (cname) { 
 		c = mvc_bind_column(sql, t, cname);
 		if (!c) 
-			return sql_message("42S22!GRANT: table %s has no column %s", tname, cname);
+			throw(SQL,"sql.grant_table",SQLSTATE(42S22) "Table %s has no column %s", tname, cname);
 		/* allowed on column */
 		if (!allowed)
 			allowed = sql_grantable(sql, grantor, c->base.id, privs, 0);
 
 		if (!allowed) 
-			return sql_message("0L000!GRANT: grantor %s is not allowed to grant privilege %s for table %s", stack_get_string(sql, "current_user"), priv2string(privs), tname);
+			throw(SQL, "sql.grant_table", SQLSTATE(0L000) "Grantor %s is not allowed to grant privilege %s for table %s", stack_get_string(sql, "current_user"), priv2string(privs), tname);
 	}
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!GRANT: user/role '%s' unknown", grantee);
+		throw(SQL,"sql.grant_table", SQLSTATE(42M32) "User/role '%s' unknown", grantee);
 	/* first check if privilege isn't already given */
 	if ((privs == all && 
 	    (sql_privilege(sql, grantee_id, t->base.id, PRIV_SELECT, 0) ||
 	     sql_privilege(sql, grantee_id, t->base.id, PRIV_UPDATE, 0) ||
 	     sql_privilege(sql, grantee_id, t->base.id, PRIV_INSERT, 0) ||
-	     sql_privilege(sql, grantee_id, t->base.id, PRIV_DELETE, 0))) ||
+	     sql_privilege(sql, grantee_id, t->base.id, PRIV_DELETE, 0) ||
+	     sql_privilege(sql, grantee_id, t->base.id, PRIV_TRUNCATE, 0))) ||
 	    (privs != all && !c && sql_privilege(sql, grantee_id, t->base.id, privs, 0)) || 
 	    (privs != all && c && sql_privilege(sql, grantee_id, c->base.id, privs, 0))) {
-		return sql_message("42M32!GRANT: user/role '%s' already has this privilege", grantee);
+		throw(SQL, "sql.grant", SQLSTATE(42M32) "User/role '%s' already has this privilege", grantee);
 	}
 	if (privs == all) {
 		sql_insert_all_privs(sql, grantee_id, t->base.id, grantor, grant);
@@ -206,14 +211,14 @@ sql_grant_func_privs( mvc *sql, char *grantee, int privs, char *sname, int func_
 		allowed = sql_grantable(sql, grantor, f->base.id, privs, 0);
 
 	if (!allowed) 
-		return sql_message("0L000!GRANT: grantor '%s' is not allowed to grant privileges for function '%s'", stack_get_string(sql,"current_user"), f->base.name);
+		throw(SQL, "sql.grant_func", SQLSTATE(0L000) "Grantor '%s' is not allowed to grant privileges for function '%s'", stack_get_string(sql,"current_user"), f->base.name);
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!GRANT: user/role '%s' unknown", grantee);
+		throw(SQL, "sql.grant_func", SQLSTATE(42M32) "User/role '%s' unknown", grantee);
 	/* first check if privilege isn't already given */
 	if (sql_privilege(sql, grantee_id, f->base.id, privs, 0)) 
-		return sql_message("42M32!GRANT: user/role '%s' already has this privilege", grantee);
+		throw(SQL,"sql.grant", SQLSTATE(42M32) "User/role '%s' already has this privilege", grantee);
 	sql_insert_priv(sql, grantee_id, f->base.id, privs, grantor, grant);
 	tr->schema_updates++;
 	return NULL;
@@ -238,7 +243,7 @@ sql_delete_priv(mvc *sql, int auth_id, int obj_id, int privilege, int grantor, i
 	A = table_funcs.rids_select(tr, priv_auth, &auth_id, &auth_id, priv_priv, &privilege, &privilege, priv_obj, &obj_id, &obj_id, NULL );
 
 	/* remove them */
-	for(rid = table_funcs.rids_next(A); rid != oid_nil; rid = table_funcs.rids_next(A)) 
+	for(rid = table_funcs.rids_next(A); !is_oid_nil(rid); rid = table_funcs.rids_next(A)) 
 		table_funcs.table_delete(tr, privs, rid); 
 	table_funcs.rids_destroy(A);
 }
@@ -254,11 +259,11 @@ sql_revoke_global_privs( mvc *sql, char *grantee, int privs, int grant, int gran
 		allowed = sql_grantable(sql, grantor, GLOBAL_OBJID, privs, 0);
 
 	if (!allowed) 
-		return sql_message("0L000!REVOKE: grantor '%s' is not allowed to revoke global privileges", stack_get_string(sql,"current_user"));
+		throw(SQL, "sql.revoke_global", SQLSTATE(0L000) "Grantor '%s' is not allowed to revoke global privileges", stack_get_string(sql,"current_user"));
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!REVOKE: user/role '%s' unknown", grantee);
+		throw(SQL, "sql.revoke_global", SQLSTATE(42M32) "REVOKE: user/role '%s' unknown", grantee);
 	sql_delete_priv(sql, grantee_id, GLOBAL_OBJID, privs, grantor, grant);
 	sql->session->tr->schema_updates++;
 	return NULL;
@@ -271,42 +276,43 @@ sql_revoke_table_privs( mvc *sql, char *grantee, int privs, char *sname, char *t
 	sql_table *t = NULL;
 	sql_column *c = NULL;
 	int allowed, grantee_id;
-	int all = PRIV_SELECT | PRIV_UPDATE | PRIV_INSERT | PRIV_DELETE;
+	int all = PRIV_SELECT | PRIV_UPDATE | PRIV_INSERT | PRIV_DELETE | PRIV_TRUNCATE;
 
  	if (sname)
 		s = mvc_bind_schema(sql, sname);
 	if (s)
  		t = mvc_bind_table(sql, s, tname);
 	if (!t) 
-		return sql_message("42S02!REVOKE: no such table '%s'", tname);
+		throw(SQL,"sql.revoke_table", SQLSTATE(42S02) "Revoke: no such table '%s'", tname);
 
 	allowed = schema_privs(grantor, t->s);
 	if (!allowed)
 		allowed = sql_grantable(sql, grantor, t->base.id, privs, 0);
 
 	if (!allowed) 
-		return sql_message("0L000!REVOKE: grantor '%s' is not allowed to revoke privileges for table '%s'", stack_get_string(sql,"current_user"), tname);
+		throw(SQL, "sql.revoke_table", SQLSTATE(0L000) "Grantor '%s' is not allowed to revoke privileges for table '%s'", stack_get_string(sql,"current_user"), tname);
 
 	if (cname) { 
 		c = mvc_bind_column(sql, t, cname);
 		if (!c) 
-			return sql_message("42S22!REVOKE: table %s has no column %s", tname, cname);
+			throw(SQL,"sql.revoke_table", SQLSTATE(42S22) "REVOKE: table %s has no column %s", tname, cname);
 		/* allowed on column */
 		if (!allowed)
 			allowed = sql_grantable(sql, grantor, c->base.id, privs, 0);
 
 		if (!allowed) 
-			return sql_message("0L000!REVOKE: grantor %s is not allowed to revoke privilege %s for table %s", stack_get_string(sql, "current_user"), priv2string(privs), tname);
+			throw(SQL, "sql.revoke_table", SQLSTATE(0L000) "Grantor %s is not allowed to revoke privilege %s for table %s", stack_get_string(sql, "current_user"), priv2string(privs), tname);
 	}
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!REVOKE: user/role '%s' unknown", grantee);
+		 throw(SQL,"sql.revoke_table", SQLSTATE(42M32) "REVOKE: user/role '%s' unknown", grantee);
 	if (privs == all) {
 		sql_delete_priv(sql, grantee_id, t->base.id, PRIV_SELECT, grantor, grant);
 		sql_delete_priv(sql, grantee_id, t->base.id, PRIV_UPDATE, grantor, grant);
 		sql_delete_priv(sql, grantee_id, t->base.id, PRIV_INSERT, grantor, grant);
 		sql_delete_priv(sql, grantee_id, t->base.id, PRIV_DELETE, grantor, grant);
+		sql_delete_priv(sql, grantee_id, t->base.id, PRIV_TRUNCATE, grantor, grant);
 	} else if (!c) {
 		sql_delete_priv(sql, grantee_id, t->base.id, privs, grantor, grant);
 	} else {
@@ -336,11 +342,11 @@ sql_revoke_func_privs( mvc *sql, char *grantee, int privs, char *sname, int func
 		allowed = sql_grantable(sql, grantor, f->base.id, privs, 0);
 
 	if (!allowed) 
-		return sql_message("0L000!REVOKE: grantor '%s' is not allowed to revoke privileges for function '%s'", stack_get_string(sql,"current_user"), f->base.name);
+		throw(SQL, "sql.revoke_func", SQLSTATE(0L000) "Grantor '%s' is not allowed to revoke privileges for function '%s'", stack_get_string(sql,"current_user"), f->base.name);
 
 	grantee_id = sql_find_auth(sql, grantee);
 	if (grantee_id <= 0) 
-		return sql_message("42M32!REVOKE: user/role '%s' unknown", grantee);
+		throw(SQL, "sql.revoke_func", SQLSTATE(42M32) "REVOKE: user/role '%s' unknown", grantee);
 	sql_delete_priv(sql, grantee_id, f->base.id, privs, grantor, grant);
 	sql->session->tr->schema_updates++;
 	return NULL;
@@ -354,7 +360,7 @@ sql_create_auth_id(mvc *m, unsigned int id, str auth)
 	sql_table *auths = find_sql_table(sys, "auths");
 	sql_column *auth_name = find_sql_column(auths, "name");
 
-	if (table_funcs.column_find_row(m->session->tr, auth_name, auth, NULL) != oid_nil)
+	if (!is_oid_nil(table_funcs.column_find_row(m->session->tr, auth_name, auth, NULL)))
 		return FALSE;
 
 	table_funcs.table_insert(m->session->tr, auths, &id, auth, &grantor);
@@ -371,10 +377,10 @@ sql_create_role(mvc *m, str auth, int grantor)
 	sql_column *auth_name = find_sql_column(auths, "name");
 
 	if (!admin_privs(grantor)) 
-		return sql_message("0P000!CREATE ROLE: insufficient privileges to create role '%s'", auth);
+		throw(SQL, "sql.create_role", SQLSTATE(0P000) "Insufficient privileges to create role '%s'", auth);
 
-	if (table_funcs.column_find_row(m->session->tr, auth_name, auth, NULL) != oid_nil)
-		return sql_message("0P000!CREATE ROLE: role '%s' already exists", auth);
+	if (!is_oid_nil(table_funcs.column_find_row(m->session->tr, auth_name, auth, NULL)))
+		throw(SQL, "sql.create_role", SQLSTATE(0P000) "Role '%s' already exists", auth);
 
 	id = store_next_oid();
 	table_funcs.table_insert(m->session->tr, auths, &id, auth, &grantor);
@@ -391,8 +397,8 @@ sql_drop_role(mvc *m, str auth)
 	sql_column *auth_name = find_sql_column(auths, "name");
 
 	rid = table_funcs.column_find_row(m->session->tr, auth_name, auth, NULL);
-	if (rid == oid_nil)
-		return sql_message("0P000!DROP ROLE: no such role '%s'", auth);
+	if (is_oid_nil(rid))
+		throw(SQL, "sql.drop_role", SQLSTATE(0P000) "DROP ROLE: no such role '%s'", auth);
 	table_funcs.table_delete(m->session->tr, auths, rid);
 	m->session->tr->schema_updates++;
 	return NULL;
@@ -417,7 +423,7 @@ sql_privilege(mvc *m, int auth_id, int obj_id, int priv, int sub)
 	oid rid = sql_privilege_rid(m, auth_id, obj_id, priv, sub);
 	int res = 0;
 
-	if (rid != oid_nil) {
+	if (!is_oid_nil(rid)) {
 		/* found priv */
 		res = priv;
 	}
@@ -440,9 +446,9 @@ int
 table_privs(mvc *m, sql_table *t, int priv)
 {
 	/* temporary tables are owned by the session user */
-	if (t->persistence != SQL_PERSIST || t->commit_action)
+	if (t->persistence == SQL_DECLARED_TABLE || (priv == PRIV_SELECT && (t->persistence != SQL_PERSIST || t->commit_action)))
 		return 1;
-	if (admin_privs(m->user_id) || admin_privs(m->role_id) || m->user_id == t->s->auth_id || m->role_id == t->s->auth_id || sql_privilege(m, m->user_id, t->base.id, priv, 0) == priv || sql_privilege(m, m->role_id, t->base.id, priv, 0) == priv || sql_privilege(m, ROLE_PUBLIC, t->base.id, priv, 0) == priv) {
+	if (admin_privs(m->user_id) || admin_privs(m->role_id) || (t->s && (m->user_id == t->s->auth_id || m->role_id == t->s->auth_id)) || sql_privilege(m, m->user_id, t->base.id, priv, 0) == priv || sql_privilege(m, m->role_id, t->base.id, priv, 0) == priv || sql_privilege(m, ROLE_PUBLIC, t->base.id, priv, 0) == priv) {
 		return 1;
 	}
 	return 0;
@@ -498,19 +504,19 @@ sql_grant_role(mvc *m, str grantee, str role, int grantor, int admin)
 	void *val;
 
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, role, NULL);
-	if (rid == oid_nil) 
-		return sql_message("M1M05!GRANT: cannot grant ROLE '%s' to ROLE '%s'", role, grantee);
+	if (is_oid_nil(rid)) 
+		throw(SQL,  "sql.grant_role", SQLSTATE(M1M05) "Cannot grant ROLE '%s' to ROLE '%s'", role, grantee);
 	val = table_funcs.column_find_value(m->session->tr, auths_id, rid);
 	role_id = *(int*)val; 
 	_DELETE(val);
 
 	if (backend_find_user(m, role) >= 0) 
-		return sql_message("M1M05!GRANT: '%s' is a USER not a ROLE", role);
+		throw(SQL,"sql.grant_role", SQLSTATE(M1M05) "GRANT: '%s' is a USER not a ROLE", role);
 	if (!admin_privs(grantor) && !role_granting_privs(m, rid, role_id, grantor)) 
-		return sql_message("0P000!GRANT: insufficient privileges to grant ROLE '%s'", role);
+		throw(SQL,"sql.grant_role", SQLSTATE(0P000) "Insufficient privileges to grant ROLE '%s'", role);
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, grantee, NULL);
-	if (rid == oid_nil)
-		return sql_message("M1M05!GRANT: cannot grant ROLE '%s' to ROLE '%s'", role, grantee);
+	if (is_oid_nil(rid))
+		throw(SQL,"sql.grant_role", SQLSTATE(M1M05) "Cannot grant ROLE '%s' to ROLE '%s'", role, grantee);
 	val = table_funcs.column_find_value(m->session->tr, auths_id, rid);
 	grantee_id = *(int*)val; 
 	_DELETE(val);
@@ -542,28 +548,28 @@ sql_revoke_role(mvc *m, str grantee, str role, int grantor, int admin)
 	void *val;
 
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, grantee, NULL);
-	if (rid == oid_nil)
-		return sql_message("42M32!REVOKE: no such role '%s' or grantee '%s'", role, grantee);
+	if (is_oid_nil(rid))
+		throw(SQL,"sql.revoke_role", SQLSTATE(42M32) "REVOKE: no such role '%s' or grantee '%s'", role, grantee);
 	val = table_funcs.column_find_value(m->session->tr, auths_id, rid);
 	grantee_id = *(int*)val; 
 	_DELETE(val);
 
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, role, NULL);
-	if (rid == oid_nil) 
-		return sql_message("42M32!REVOKE: no such role '%s' or grantee '%s'", role, grantee);
+	if (is_oid_nil(rid)) 
+		throw(SQL,"sql.revoke_role", SQLSTATE(42M32) "REVOKE: no such role '%s' or grantee '%s'", role, grantee);
 	val = table_funcs.column_find_value(m->session->tr, auths_id, rid);
 	role_id = *(int*)val; 
 	_DELETE(val);
-	if (!admin_privs(grantor) && !role_granting_privs(m, rid, role_id, grantor)) 
-		return sql_message("0P000!REVOKE: insufficient privileges to revoke ROLE '%s'", role);
+	if (!admin_privs(grantor) && !role_granting_privs(m, rid, role_id, grantor))
+		throw(SQL,"sql.revoke_role", SQLSTATE(0P000) "REVOKE: insufficient privileges to revoke ROLE '%s'", role);
 
 	if (!admin) { 
 		rid = table_funcs.column_find_row(m->session->tr, roles_login_id, &grantee_id, roles_role_id, &role_id, NULL);
-		if (rid != oid_nil) 
+		if (!is_oid_nil(rid)) 
 			table_funcs.table_delete(m->session->tr, roles, rid);
 	} else {
 		rid = sql_privilege_rid(m, grantee_id, role_id, PRIV_ROLE_ADMIN, 0);
-		if (rid != oid_nil) 
+		if (!is_oid_nil(rid)) 
 			table_funcs.table_delete(m->session->tr, roles, rid);
 	}
 	m->session->tr->schema_updates++;
@@ -581,7 +587,7 @@ sql_find_auth(mvc *m, str auth)
 
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, auth, NULL);
 
-	if (rid != oid_nil) {
+	if (!is_oid_nil(rid)) {
 		sql_column *auths_id = find_sql_column(auths, "id");
 		int *p = (int *) table_funcs.column_find_value(m->session->tr, auths_id, rid);
 
@@ -604,7 +610,7 @@ sql_find_schema(mvc *m, str schema)
 
 	rid = table_funcs.column_find_row(m->session->tr, schemas_name, schema, NULL);
 
-	if (rid != oid_nil) {
+	if (!is_oid_nil(rid)) {
 		sql_column *schemas_id = find_sql_column(schemas, "id");
 		int *p = (int *) table_funcs.column_find_value(m->session->tr, schemas_id, rid);
 
@@ -639,7 +645,7 @@ sql_grantable_(mvc *m, int grantorid, int obj_id, int privs, int sub)
 		if (!(priv & privs))
 			continue;
 		rid = table_funcs.column_find_row(m->session->tr, priv_obj, &obj_id, priv_auth, &grantorid, priv_priv, &priv, NULL);
-		if (rid != oid_nil) {
+		if (!is_oid_nil(rid)) {
 			void *p = table_funcs.column_find_value(m->session->tr, priv_allowed, rid);
 			int allowed = *(int *)p;
 
@@ -675,7 +681,7 @@ mvc_set_role(mvc *m, char *role)
 		fprintf(stderr, "mvc_set_role %s\n", role);
 
 	rid = table_funcs.column_find_row(m->session->tr, auths_name, role, NULL);
-	if (rid != oid_nil) {
+	if (!is_oid_nil(rid)) {
 		sql_column *auths_id = find_sql_column(auths, "id");
 		void *p = table_funcs.column_find_value(m->session->tr, auths_id, rid);
 		int id = *(int *)p;
@@ -692,7 +698,7 @@ mvc_set_role(mvc *m, char *role)
 
 			rid = table_funcs.column_find_row(m->session->tr, login_id, &m->user_id, role_id, &id, NULL);
 		
-			if (rid != oid_nil) {
+			if (!is_oid_nil(rid)) {
 				m->role_id = id;
 				res = 1;
 			}
@@ -706,15 +712,18 @@ mvc_set_schema(mvc *m, char *schema)
 {
 	int ret = 0;
 	sql_schema *s = find_sql_schema(m->session->tr, schema);
+	char* new_schema_name = _STRDUP(schema);
 
-	if (s) {
+	if (s && new_schema_name) {
 		if (m->session->schema_name)
 			_DELETE(m->session->schema_name);
-		m->session->schema_name = _STRDUP(schema);
+		m->session->schema_name = new_schema_name;
 		m->type = Q_TRANS;
-		if (m->session->active) 
+		if (m->session->active)
 			m->session->schema = s;
 		ret = 1;
+	} else if(new_schema_name) {
+		_DELETE(new_schema_name);
 	}
 	return ret;
 }
@@ -726,13 +735,13 @@ sql_create_user(mvc *sql, char *user, char *passwd, char enc, char *fullname, ch
 	int schema_id = 0;
 
 	if (!admin_privs(sql->user_id) && !admin_privs(sql->role_id)) 
-		return sql_message("42M31!CREATE USER: insufficient privileges to create user '%s'", user);
+		throw(SQL,"sql.create_user", SQLSTATE(42M31) "Insufficient privileges to create user '%s'", user);
 
 	if (backend_find_user(sql, user) >= 0) {
-		return sql_message("42M31!CREATE USER: user '%s' already exists", user);
+		throw(SQL,"sql.create_user", SQLSTATE(42M31) "CREATE USER: user '%s' already exists", user);
 	}
 	if ((schema_id = sql_find_schema(sql, schema)) < 0) {
-		return sql_message("3F000!CREATE USER: no such schema '%s'", schema);
+		throw(SQL,"sql.create_user", SQLSTATE(3F000) "CREATE USER: no such schema '%s'", schema);
 	}
 	if ((err = backend_create_user(sql, user, passwd, enc, fullname,
 					schema_id, sql->user_id)) != NULL)
@@ -747,7 +756,7 @@ sql_create_user(mvc *sql, char *user, char *passwd, char enc, char *fullname, ch
 		} else {
 			e++;
 		}
-		r = sql_message("M0M27!CREATE USER: %s", e);
+		r = createException(SQL,"sql.create_user", SQLSTATE(M0M27) "CREATE USER: %s", e);
 		_DELETE(err);
 		return r;
 	}
@@ -760,9 +769,9 @@ sql_drop_user(mvc *sql, char *user)
 	int user_id = sql_find_auth(sql, user);
 
 	if (mvc_check_dependency(sql, user_id, OWNER_DEPENDENCY, NULL))
-		return sql_message("M1M05!DROP USER: '%s' owns a schema", user);
+		throw(SQL,"sql.drop_user",SQLSTATE(M1M05) "DROP USER: '%s' owns a schema", user);
 	if (backend_drop_user(sql,user) == FALSE)
-		return sql_message("M0M27!%s", sql->errstr);
+		throw(SQL,"sql.drop_user",SQLSTATE(M0M27) "%s", sql->errstr);
 	return sql_drop_role(sql, user);
 }
 
@@ -776,15 +785,15 @@ sql_alter_user(mvc *sql, char *user, char *passwd, char enc,
 		user = NULL;
 	/* USER == NULL -> current_user */
 	if (user != NULL && backend_find_user(sql, user) < 0)
-		return sql_message("42M32!ALTER USER: no such user '%s'", user);
+		throw(SQL,"sql.alter_user", SQLSTATE(42M32) "ALTER USER: no such user '%s'", user);
 
 	if (!admin_privs(sql->user_id) && !admin_privs(sql->role_id) && user != NULL && strcmp(user, stack_get_string(sql, "current_user")) != 0)
-		return sql_message("M1M05!ALTER USER: insufficient privileges to change user '%s'", user);
+		throw(SQL,"sql.alter_user", SQLSTATE(M1M05) "Insufficient privileges to change user '%s'", user);
 	if (schema && (schema_id = sql_find_schema(sql, schema)) < 0) {
-		return sql_message("3F000!ALTER USER: no such schema '%s'", schema);
+		throw(SQL,"sql.alter_user", SQLSTATE(3F000) "ALTER USER: no such schema '%s'", schema);
 	}
 	if (backend_alter_user(sql, user, passwd, enc, schema_id, oldpasswd) == FALSE)
-		return sql_message("M0M27!%s", sql->errstr);
+		throw(SQL,"sql.alter_user", SQLSTATE(M0M27) "%s", sql->errstr);
 	return NULL;
 }
 
@@ -792,15 +801,15 @@ char *
 sql_rename_user(mvc *sql, char *olduser, char *newuser)
 {
 	if (backend_find_user(sql, olduser) < 0)
-		return sql_message("42M32!ALTER USER: no such user '%s'", olduser);
+		throw(SQL,"sql.rename_user", SQLSTATE(42M32) "ALTER USER: no such user '%s'", olduser);
 	if (backend_find_user(sql, newuser) >= 0)
-		return sql_message("42M31!ALTER USER: user '%s' already exists", newuser);
+		throw(SQL,"sql.rename_user", SQLSTATE(42M31) "ALTER USER: user '%s' already exists", newuser);
 	if (!admin_privs(sql->user_id) && !admin_privs(sql->role_id))
-		return sql_message("M1M05!ALTER USER: insufficient privileges to "
+		throw(SQL,"sql.rename_user", SQLSTATE(M1M05) "ALTER USER: insufficient privileges to "
 				"rename user '%s'", olduser);
 
 	if (backend_rename_user(sql, olduser, newuser) == FALSE)
-		return sql_message("M1M05!%s", sql->errstr);
+		throw(SQL,"sql.rename_user", SQLSTATE(M1M05) "%s", sql->errstr);
 	return NULL;
 
 }
@@ -868,6 +877,8 @@ sql_create_privileges(mvc *m, sql_schema *s)
 	table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
 	t = find_sql_table(s, "columns");
 	table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
+	t = find_sql_table(s, "comments");
+	table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
 	t = find_sql_table(s, "user_role");
 	table_funcs.table_insert(m->session->tr, privs, &t->base.id, &pub, &p, &zero, &zero);
 	t = find_sql_table(s, "auths");
