@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #include "monetdb_config.h"
@@ -21,7 +21,6 @@
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include "sql_keyword.h"
 #ifdef HAVE_HGE
 #include "mal.h"		/* for have_hge */
@@ -108,8 +107,6 @@ scanner_init_keywords(void)
 	failed += keywords_insert("PROD", AGGR);
 	failed += keywords_insert("COUNT", AGGR);
 
-	failed += keywords_insert("LAG", AGGR);
-	failed += keywords_insert("LEAD", AGGR);
 	failed += keywords_insert("LAG", AGGR2);
 	failed += keywords_insert("LEAD", AGGR2);
 
@@ -143,6 +140,7 @@ scanner_init_keywords(void)
 	failed += keywords_insert("TINYTEXT", sqlTEXT);
 	failed += keywords_insert("STRING", CLOB);	/* ? */
 	failed += keywords_insert("CHECK", CHECK);
+	failed += keywords_insert("COMMENT", COMMENT);
 	failed += keywords_insert("CONSTRAINT", CONSTRAINT);
 	failed += keywords_insert("CREATE", CREATE);
 	failed += keywords_insert("CROSS", CROSS);
@@ -191,6 +189,7 @@ scanner_init_keywords(void)
 	failed += keywords_insert("INSERT", INSERT);
 	failed += keywords_insert("UPDATE", UPDATE);
 	failed += keywords_insert("DELETE", sqlDELETE);
+	failed += keywords_insert("TRUNCATE", TRUNCATE);
 
 	failed += keywords_insert("ACTION", ACTION);
 	failed += keywords_insert("CASCADE", CASCADE);
@@ -293,7 +292,9 @@ scanner_init_keywords(void)
 	failed += keywords_insert("ZONE", ZONE);
 
 	failed += keywords_insert("YEAR", YEAR);
+	failed += keywords_insert("QUARTER", QUARTER);
 	failed += keywords_insert("MONTH", MONTH);
+	failed += keywords_insert("WEEK", WEEK);
 	failed += keywords_insert("DAY", DAY);
 	failed += keywords_insert("HOUR", HOUR);
 	failed += keywords_insert("MINUTE", MINUTE);
@@ -383,6 +384,7 @@ scanner_init_keywords(void)
 	failed += keywords_insert("SERIAL", SERIAL);
 	failed += keywords_insert("BIGSERIAL", BIGSERIAL);
 	failed += keywords_insert("AUTO_INCREMENT", AUTO_INCREMENT);
+	failed += keywords_insert("CONTINUE", CONTINUE);
 
 	failed += keywords_insert("TRIGGER", TRIGGER);
 	failed += keywords_insert("ATOMIC", ATOMIC);
@@ -534,13 +536,13 @@ scanner_error(mvc *lc, int cur)
 {
 	switch (cur) {
 	case EOF:
-		(void) sql_error(lc, 1, "unexpected end of input");
+		(void) sql_error(lc, 1, SQLSTATE(42000) "Unexpected end of input");
 		return -1;	/* EOF needs -1 result */
 	default:
 		/* on Windows at least, iswcntrl returns TRUE for
 		 * U+FEFF, but we just want consistent error
 		 * messages */
-		(void) sql_error(lc, 1, "unexpected%s character (U+%04X)", iswcntrl(cur) && cur != 0xFEFF ? " control" : "", cur);
+		(void) sql_error(lc, 1, SQLSTATE(42000) "Unexpected%s character (U+%04X)", iswcntrl(cur) && cur != 0xFEFF ? " control" : "", (unsigned) cur);
 	}
 	return LEX_ERROR;
 }
@@ -638,7 +640,7 @@ scanner_getc(struct scanner *lc)
 		/* incorrect UTF-8 sequence */
 		/* n==0: c == 10xxxxxx */
 		/* n>=6: c == 1111111x */
-		lc->errstr = "!invalid start of UTF-8 sequence";
+		lc->errstr = SQLSTATE(42000) "invalid start of UTF-8 sequence";
 		goto error;
 	}
 
@@ -654,7 +656,7 @@ scanner_getc(struct scanner *lc)
 		if (((m = *s++) & 0xC0) != 0x80) {
 			/* incorrect UTF-8 sequence: byte is not 10xxxxxx */
 			/* this includes end-of-string (m == 0) */
-			lc->errstr = "!invalid continuation in UTF-8 sequence";
+			lc->errstr = SQLSTATE(42000) "invalid continuation in UTF-8 sequence";
 			goto error;
 		}
 		c |= m & 0x3F;
@@ -716,7 +718,7 @@ scanner_string(mvc *c, int quote)
 		/* long utf8, if correct isn't the quote */
 		if (!cur) {
 			if (lc->rs->len >= lc->rs->pos + lc->yycur + 1) {
-				(void) sql_error(c, 2, "NULL byte in string");
+				(void) sql_error(c, 2, SQLSTATE(42000) "NULL byte in string");
 				return LEX_ERROR;
 			}
 			cur = scanner_read_more(lc, 1);
@@ -724,7 +726,7 @@ scanner_string(mvc *c, int quote)
 			cur = scanner_getc(lc);
 		}
 	}
-	(void) sql_error(c, 2, "%s", lc->errstr ? lc->errstr : "unexpected end of input");
+	(void) sql_error(c, 2, SQLSTATE(42000) "%s", lc->errstr ? lc->errstr : "unexpected end of input");
 	return LEX_ERROR;
 }
 
@@ -763,7 +765,7 @@ scanner_body(mvc *c)
 		lc->yycur--;	/* go back to current (possibly invalid) char */
 		if (!cur) {
 			if (lc->rs->len >= lc->rs->pos + lc->yycur + 1) {
-				(void) sql_error(c, 2, "NULL byte in string");
+				(void) sql_error(c, 2, SQLSTATE(42000) "NULL byte in string");
 				return LEX_ERROR;
 			}
 			cur = scanner_read_more(lc, 1);
@@ -771,7 +773,7 @@ scanner_body(mvc *c)
 			cur = scanner_getc(lc);
 		}
 	}
-	(void) sql_error(c, 2, "unexpected end of input");
+	(void) sql_error(c, 2, SQLSTATE(42000) "Unexpected end of input");
 	return LEX_ERROR;
 }
 
@@ -915,7 +917,7 @@ number(mvc * c, int cur)
 			utf8_putchar(lc, before_cur);
 		return scanner_token(lc, token);
 	} else {
-		(void)sql_error( c, 2, "unexpected symbol %lc", (wint_t) cur);
+		(void)sql_error( c, 2, SQLSTATE(42000) "Unexpected symbol %lc", (wint_t) cur);
 		return LEX_ERROR;
 	}
 }
@@ -1103,7 +1105,7 @@ int scanner_symbol(mvc * c, int cur)
 			return scanner_token(lc, '|');
 		}
 	}
-	(void)sql_error( c, 3, "unexpected symbol (%lc)", (wint_t) cur);
+	(void)sql_error( c, 3, SQLSTATE(42000) "Unexpected symbol (%lc)", (wint_t) cur);
 	return LEX_ERROR;
 }
 
@@ -1227,7 +1229,7 @@ sql_get_next_token(YYSTYPE *yylval, void *parm) {
 			if (valid_ident(yylval->sval+1,str)) {
 				token = IDENT;
 			} else {
-				sql_error(c, 1, "Invalid identifier '%s'", yylval->sval+1);
+				sql_error(c, 1, SQLSTATE(42000) "Invalid identifier '%s'", yylval->sval+1);
 				return LEX_ERROR;
 			}
 		} else {
