@@ -3,7 +3,7 @@
  * License, v. 2.0.  If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * Copyright 1997 - July 2008 CWI, August 2008 - 2017 MonetDB B.V.
+ * Copyright 1997 - July 2008 CWI, August 2008 - 2018 MonetDB B.V.
  */
 
 #ifndef STORE_SEQ_H
@@ -11,7 +11,7 @@
 
 #include "sql_catalog.h"
 
-extern void sequences_init(void);
+extern void* sequences_init(void);
 extern void sequences_exit(void);
 
 extern int seq_next_value(sql_sequence *seq, lng *val);
